@@ -47,11 +47,11 @@ LPITEMIDLIST (WINAPI *COMDLG32_PIDL_ILCombine)(LPCITEMIDLIST,LPCITEMIDLIST);
 LPITEMIDLIST (WINAPI *COMDLG32_PIDL_ILGetNext)(LPITEMIDLIST);
 BOOL (WINAPI *COMDLG32_PIDL_ILRemoveLastID)(LPCITEMIDLIST);
 BOOL (WINAPI *COMDLG32_PIDL_ILIsEqual)(LPCITEMIDLIST, LPCITEMIDLIST);
+UINT (WINAPI *COMDLG32_PIDL_ILGetSize)(LPCITEMIDLIST);
 
 /* SHELL */
 LPVOID (WINAPI *COMDLG32_SHAlloc)(DWORD);
 DWORD (WINAPI *COMDLG32_SHFree)(LPVOID);
-BOOL (WINAPI *COMDLG32_SHGetFolderPathA)(HWND,int,HANDLE,DWORD,LPSTR);
 BOOL (WINAPI *COMDLG32_SHGetFolderPathW)(HWND,int,HANDLE,DWORD,LPWSTR);
 
 /***********************************************************************
@@ -83,30 +83,18 @@ BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD Reason, LPVOID Reserved)
 
 		SHELL32_hInstance = GetModuleHandleA("SHELL32.DLL");
 
-		if (!SHELL32_hInstance)
-		{
-			ERR("loading of shell32 failed\n");
-			return FALSE;
-		}
-
 		/* ITEMIDLIST */
 		GPA(COMDLG32_PIDL_ILIsEqual, SHELL32_hInstance, (LPCSTR)21L);
 		GPA(COMDLG32_PIDL_ILCombine, SHELL32_hInstance, (LPCSTR)25L);
 		GPA(COMDLG32_PIDL_ILGetNext, SHELL32_hInstance, (LPCSTR)153L);
 		GPA(COMDLG32_PIDL_ILClone, SHELL32_hInstance, (LPCSTR)18L);
 		GPA(COMDLG32_PIDL_ILRemoveLastID, SHELL32_hInstance, (LPCSTR)17L);
+		GPA(COMDLG32_PIDL_ILGetSize, SHELL32_hInstance, (LPCSTR)152L);
 
 		/* SHELL */
 
 		GPA(COMDLG32_SHAlloc, SHELL32_hInstance, (LPCSTR)196L);
 		GPA(COMDLG32_SHFree, SHELL32_hInstance, (LPCSTR)195L);
-		/* for the first versions of shell32 SHGetFolderPathA is in SHFOLDER.DLL */
-		COMDLG32_SHGetFolderPathA = (void*)GetProcAddress(SHELL32_hInstance,"SHGetFolderPathA");
-		if (!COMDLG32_SHGetFolderPathA)
-		{
-		  SHFOLDER_hInstance = LoadLibraryA("SHFOLDER.DLL");
-		  GPA(COMDLG32_SHGetFolderPathA, SHFOLDER_hInstance,"SHGetFolderPathA");
-		}
 
 		/* for the first versions of shell32 SHGetFolderPathW is in SHFOLDER.DLL */
 		COMDLG32_SHGetFolderPathW = (void*)GetProcAddress(SHELL32_hInstance,"SHGetFolderPathW");
@@ -166,7 +154,6 @@ void COMDLG32_SetCommDlgExtendedError(DWORD err)
 
 /***********************************************************************
  *	CommDlgExtendedError			(COMDLG32.@)
- *	CommDlgExtendedError			(COMMDLG.26)
  *
  * Get the thread's local error value if a comdlg32 function fails.
  *	RETURNS
