@@ -40,15 +40,15 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_inner_QueryInterface(IUnknown *ifa
     if (IsEqualGUID(riid, &IID_IUnknown)
             || IsEqualGUID(riid, &IID_ID3D10Device))
     {
-        ID3D10Device_AddRef(&This->ID3D10Device_iface);
+        IUnknown_AddRef((IUnknown *)This);
         *object = This;
         return S_OK;
     }
 
-    if (IsEqualGUID(riid, &IID_IWineDXGIDeviceParent))
+    if (IsEqualGUID(riid, &IID_IWineD3DDeviceParent))
     {
-        IWineDXGIDeviceParent_AddRef(&This->IWineDXGIDeviceParent_iface);
-        *object = &This->IWineDXGIDeviceParent_iface;
+        IUnknown_AddRef((IUnknown *)&This->device_parent_vtbl);
+        *object = &This->device_parent_vtbl;
         return S_OK;
     }
 
@@ -77,8 +77,7 @@ static ULONG STDMETHODCALLTYPE d3d10_device_inner_Release(IUnknown *iface)
 
     if (!refcount)
     {
-        if (This->wined3d_device)
-            wined3d_device_decref(This->wined3d_device);
+        if (This->wined3d_device) IWineD3DDevice_Release(This->wined3d_device);
     }
 
     return refcount;
@@ -86,29 +85,23 @@ static ULONG STDMETHODCALLTYPE d3d10_device_inner_Release(IUnknown *iface)
 
 /* IUnknown methods */
 
-static inline struct d3d10_device *impl_from_ID3D10Device(ID3D10Device *iface)
+static HRESULT STDMETHODCALLTYPE d3d10_device_QueryInterface(ID3D10Device *iface, REFIID riid, void **object)
 {
-    return CONTAINING_RECORD(iface, struct d3d10_device, ID3D10Device_iface);
-}
-
-static HRESULT STDMETHODCALLTYPE d3d10_device_QueryInterface(ID3D10Device *iface, REFIID riid,
-        void **object)
-{
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     TRACE("Forwarding to outer IUnknown\n");
     return IUnknown_QueryInterface(This->outer_unknown, riid, object);
 }
 
 static ULONG STDMETHODCALLTYPE d3d10_device_AddRef(ID3D10Device *iface)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     TRACE("Forwarding to outer IUnknown\n");
     return IUnknown_AddRef(This->outer_unknown);
 }
 
 static ULONG STDMETHODCALLTYPE d3d10_device_Release(ID3D10Device *iface)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     TRACE("Forwarding to outer IUnknown\n");
     return IUnknown_Release(This->outer_unknown);
 }
@@ -129,15 +122,14 @@ static void STDMETHODCALLTYPE d3d10_device_PSSetShaderResources(ID3D10Device *if
             iface, start_slot, view_count, views);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_PSSetShader(ID3D10Device *iface,
-        ID3D10PixelShader *shader)
+static void STDMETHODCALLTYPE d3d10_device_PSSetShader(ID3D10Device *iface, ID3D10PixelShader *shader)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
-    struct d3d10_pixel_shader *ps = unsafe_impl_from_ID3D10PixelShader(shader);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_pixel_shader *ps = (struct d3d10_pixel_shader *)shader;
 
     TRACE("iface %p, shader %p\n", iface, shader);
 
-    wined3d_device_set_pixel_shader(This->wined3d_device, ps ? ps->wined3d_shader : NULL);
+    IWineD3DDevice_SetPixelShader(This->wined3d_device, ps ? ps->wined3d_shader : NULL);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_PSSetSamplers(ID3D10Device *iface,
@@ -147,38 +139,37 @@ static void STDMETHODCALLTYPE d3d10_device_PSSetSamplers(ID3D10Device *iface,
             iface, start_slot, sampler_count, samplers);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_VSSetShader(ID3D10Device *iface,
-        ID3D10VertexShader *shader)
+static void STDMETHODCALLTYPE d3d10_device_VSSetShader(ID3D10Device *iface, ID3D10VertexShader *shader)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
-    struct d3d10_vertex_shader *vs = unsafe_impl_from_ID3D10VertexShader(shader);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_vertex_shader *vs = (struct d3d10_vertex_shader *)shader;
 
     TRACE("iface %p, shader %p\n", iface, shader);
 
-    wined3d_device_set_vertex_shader(This->wined3d_device, vs ? vs->wined3d_shader : NULL);
+    IWineD3DDevice_SetVertexShader(This->wined3d_device, vs ? vs->wined3d_shader : NULL);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_DrawIndexed(ID3D10Device *iface, UINT index_count,
-        UINT start_index_location, INT base_vertex_location)
+static void STDMETHODCALLTYPE d3d10_device_DrawIndexed(ID3D10Device *iface,
+        UINT index_count, UINT start_index_location, INT base_vertex_location)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
 
     TRACE("iface %p, index_count %u, start_index_location %u, base_vertex_location %d.\n",
             iface, index_count, start_index_location, base_vertex_location);
 
-    wined3d_device_set_base_vertex_index(This->wined3d_device, base_vertex_location);
-    wined3d_device_draw_indexed_primitive(This->wined3d_device, start_index_location, index_count);
+    IWineD3DDevice_SetBaseVertexIndex(This->wined3d_device, base_vertex_location);
+    IWineD3DDevice_DrawIndexedPrimitive(This->wined3d_device, start_index_location, index_count);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_Draw(ID3D10Device *iface, UINT vertex_count,
-        UINT start_vertex_location)
+static void STDMETHODCALLTYPE d3d10_device_Draw(ID3D10Device *iface,
+        UINT vertex_count, UINT start_vertex_location)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
 
     TRACE("iface %p, vertex_count %u, start_vertex_location %u\n",
             iface, vertex_count, start_vertex_location);
 
-    wined3d_device_draw_primitive(This->wined3d_device, start_vertex_location, vertex_count);
+    IWineD3DDevice_DrawPrimitive(This->wined3d_device, start_vertex_location, vertex_count);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_PSSetConstantBuffers(ID3D10Device *iface,
@@ -188,22 +179,21 @@ static void STDMETHODCALLTYPE d3d10_device_PSSetConstantBuffers(ID3D10Device *if
             iface, start_slot, buffer_count, buffers);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IASetInputLayout(ID3D10Device *iface,
-        ID3D10InputLayout *input_layout)
+static void STDMETHODCALLTYPE d3d10_device_IASetInputLayout(ID3D10Device *iface, ID3D10InputLayout *input_layout)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
-    struct d3d10_input_layout *layout = unsafe_impl_from_ID3D10InputLayout(input_layout);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
 
     TRACE("iface %p, input_layout %p\n", iface, input_layout);
 
-    wined3d_device_set_vertex_declaration(This->wined3d_device,
-            layout ? layout->wined3d_decl : NULL);
+    IWineD3DDevice_SetVertexDeclaration(This->wined3d_device,
+            input_layout ? ((struct d3d10_input_layout *)input_layout)->wined3d_decl : NULL);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *iface, UINT start_slot,
-        UINT buffer_count, ID3D10Buffer *const *buffers, const UINT *strides, const UINT *offsets)
+static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *iface,
+        UINT start_slot, UINT buffer_count, ID3D10Buffer *const *buffers,
+        const UINT *strides, const UINT *offsets)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     unsigned int i;
 
     TRACE("iface %p, start_slot %u, buffer_count %u, buffers %p, strides %p, offsets %p\n",
@@ -211,7 +201,7 @@ static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *ifac
 
     for (i = 0; i < buffer_count; ++i)
     {
-        wined3d_device_set_stream_source(This->wined3d_device, start_slot,
+        IWineD3DDevice_SetStreamSource(This->wined3d_device, start_slot,
                 buffers[i] ? ((struct d3d10_buffer *)buffers[i])->wined3d_buffer : NULL,
                 offsets[i], strides[i]);
     }
@@ -220,13 +210,12 @@ static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *ifac
 static void STDMETHODCALLTYPE d3d10_device_IASetIndexBuffer(ID3D10Device *iface,
         ID3D10Buffer *buffer, DXGI_FORMAT format, UINT offset)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
 
     TRACE("iface %p, buffer %p, format %s, offset %u.\n",
             iface, buffer, debug_dxgi_format(format), offset);
 
-    wined3d_device_set_index_buffer(This->wined3d_device,
-            buffer ? ((struct d3d10_buffer *)buffer)->wined3d_buffer : NULL,
+    IWineD3DDevice_SetIndexBuffer(This->wined3d_device, buffer ? ((struct d3d10_buffer *)buffer)->wined3d_buffer : NULL,
             wined3dformat_from_dxgi_format(format));
     if (offset) FIXME("offset %u not supported.\n", offset);
 }
@@ -263,14 +252,13 @@ static void STDMETHODCALLTYPE d3d10_device_GSSetShader(ID3D10Device *iface, ID3D
     else WARN("iface %p, shader %p stub!\n", iface, shader);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IASetPrimitiveTopology(ID3D10Device *iface,
-        D3D10_PRIMITIVE_TOPOLOGY topology)
+static void STDMETHODCALLTYPE d3d10_device_IASetPrimitiveTopology(ID3D10Device *iface, D3D10_PRIMITIVE_TOPOLOGY topology)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
 
     TRACE("iface %p, topology %s\n", iface, debug_d3d10_primitive_topology(topology));
 
-    wined3d_device_set_primitive_type(This->wined3d_device, (WINED3DPRIMITIVETYPE)topology);
+    IWineD3DDevice_SetPrimitiveType(This->wined3d_device, (WINED3DPRIMITIVETYPE)topology);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_VSSetShaderResources(ID3D10Device *iface,
@@ -383,14 +371,14 @@ static void STDMETHODCALLTYPE d3d10_device_UpdateSubresource(ID3D10Device *iface
 static void STDMETHODCALLTYPE d3d10_device_ClearRenderTargetView(ID3D10Device *iface,
         ID3D10RenderTargetView *render_target_view, const FLOAT color_rgba[4])
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
-    struct d3d10_rendertarget_view *view = unsafe_impl_from_ID3D10RenderTargetView(render_target_view);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
+    IWineD3DRendertargetView *wined3d_view = ((struct d3d10_rendertarget_view *)render_target_view)->wined3d_view;
     const WINED3DCOLORVALUE color = {color_rgba[0], color_rgba[1], color_rgba[2], color_rgba[3]};
 
     TRACE("iface %p, render_target_view %p, color_rgba [%f %f %f %f]\n",
             iface, render_target_view, color_rgba[0], color_rgba[1], color_rgba[2], color_rgba[3]);
 
-    wined3d_device_clear_rendertarget_view(This->wined3d_device, view->wined3d_view, &color);
+    IWineD3DDevice_ClearRendertargetView(This->wined3d_device, wined3d_view, &color);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_ClearDepthStencilView(ID3D10Device *iface,
@@ -483,14 +471,13 @@ static void STDMETHODCALLTYPE d3d10_device_GSGetShader(ID3D10Device *iface, ID3D
     FIXME("iface %p, shader %p stub!\n", iface, shader);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IAGetPrimitiveTopology(ID3D10Device *iface,
-        D3D10_PRIMITIVE_TOPOLOGY *topology)
+static void STDMETHODCALLTYPE d3d10_device_IAGetPrimitiveTopology(ID3D10Device *iface, D3D10_PRIMITIVE_TOPOLOGY *topology)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
 
     TRACE("iface %p, topology %p\n", iface, topology);
 
-    wined3d_device_get_primitive_type(This->wined3d_device, (WINED3DPRIMITIVETYPE *)topology);
+    IWineD3DDevice_GetPrimitiveType(This->wined3d_device, (WINED3DPRIMITIVETYPE *)topology);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_VSGetShaderResources(ID3D10Device *iface,
@@ -631,7 +618,7 @@ static void STDMETHODCALLTYPE d3d10_device_Flush(ID3D10Device *iface)
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBuffer(ID3D10Device *iface,
         const D3D10_BUFFER_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Buffer **buffer)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_buffer *object;
     HRESULT hr;
 
@@ -668,10 +655,9 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture1D(ID3D10Device *ifac
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device *iface,
-        const D3D10_TEXTURE2D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data,
-        ID3D10Texture2D **texture)
+        const D3D10_TEXTURE2D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Texture2D **texture)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_texture2d *object;
     HRESULT hr;
 
@@ -692,7 +678,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device *ifac
         return hr;
     }
 
-    *texture = &object->ID3D10Texture2D_iface;
+    *texture = (ID3D10Texture2D *)object;
 
     TRACE("Created ID3D10Texture2D %p\n", object);
 
@@ -700,10 +686,9 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device *ifac
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture3D(ID3D10Device *iface,
-        const D3D10_TEXTURE3D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data,
-        ID3D10Texture3D **texture)
+        const D3D10_TEXTURE3D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Texture3D **texture)
 {
-    struct d3d10_device *device = impl_from_ID3D10Device(iface);
+    struct d3d10_device *device = (struct d3d10_device *)iface;
     struct d3d10_texture3d *object;
     HRESULT hr;
 
@@ -725,7 +710,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture3D(ID3D10Device *ifac
     }
 
     TRACE("Created 3D texture %p.\n", object);
-    *texture = &object->ID3D10Texture3D_iface;
+    *texture = (ID3D10Texture3D *)object;
 
     return S_OK;
 }
@@ -754,7 +739,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateShaderResourceView(ID3D10Dev
     }
 
     TRACE("Created shader resource view %p.\n", object);
-    *view = &object->ID3D10ShaderResourceView_iface;
+    *view = (ID3D10ShaderResourceView *)object;
 
     return S_OK;
 }
@@ -774,7 +759,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRenderTargetView(ID3D10Devic
         return E_OUTOFMEMORY;
     }
 
-    hr = d3d10_rendertarget_view_init(object, resource, desc);
+    hr = d3d10_rendertarget_view_init(object, (struct d3d10_device *)iface, resource, desc);
     if (FAILED(hr))
     {
         WARN("Failed to initialize rendertarget view, hr %#x.\n", hr);
@@ -783,7 +768,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRenderTargetView(ID3D10Devic
     }
 
     TRACE("Created rendertarget view %p.\n", object);
-    *view = &object->ID3D10RenderTargetView_iface;
+    *view = (ID3D10RenderTargetView *)object;
 
     return S_OK;
 }
@@ -812,17 +797,16 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateDepthStencilView(ID3D10Devic
     }
 
     TRACE("Created depthstencil view %p.\n", object);
-    *view = &object->ID3D10DepthStencilView_iface;
+    *view = (ID3D10DepthStencilView *)object;
 
     return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *iface,
-        const D3D10_INPUT_ELEMENT_DESC *element_descs, UINT element_count,
-        const void *shader_byte_code, SIZE_T shader_byte_code_length,
-        ID3D10InputLayout **input_layout)
+        const D3D10_INPUT_ELEMENT_DESC *element_descs, UINT element_count, const void *shader_byte_code,
+        SIZE_T shader_byte_code_length, ID3D10InputLayout **input_layout)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_input_layout *object;
     HRESULT hr;
 
@@ -848,7 +832,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *if
     }
 
     TRACE("Created input layout %p.\n", object);
-    *input_layout = &object->ID3D10InputLayout_iface;
+    *input_layout = (ID3D10InputLayout *)object;
 
     return S_OK;
 }
@@ -856,7 +840,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *if
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *iface,
         const void *byte_code, SIZE_T byte_code_length, ID3D10VertexShader **shader)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_vertex_shader *object;
     HRESULT hr;
 
@@ -879,7 +863,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *i
     }
 
     TRACE("Created vertex shader %p.\n", object);
-    *shader = &object->ID3D10VertexShader_iface;
+    *shader = (ID3D10VertexShader *)object;
 
     return S_OK;
 }
@@ -887,7 +871,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *i
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShader(ID3D10Device *iface,
         const void *byte_code, SIZE_T byte_code_length, ID3D10GeometryShader **shader)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_geometry_shader *object;
     HRESULT hr;
 
@@ -909,7 +893,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShader(ID3D10Device
     }
 
     TRACE("Created geometry shader %p.\n", object);
-    *shader = &object->ID3D10GeometryShader_iface;
+    *shader = (ID3D10GeometryShader *)object;
 
     return S_OK;
 }
@@ -929,7 +913,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShaderWithStreamOutp
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *iface,
         const void *byte_code, SIZE_T byte_code_length, ID3D10PixelShader **shader)
 {
-    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_device *This = (struct d3d10_device *)iface;
     struct d3d10_pixel_shader *object;
     HRESULT hr;
 
@@ -952,7 +936,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *if
     }
 
     TRACE("Created pixel shader %p.\n", object);
-    *shader = &object->ID3D10PixelShader_iface;
+    *shader = (ID3D10PixelShader *)object;
 
     return S_OK;
 }
@@ -981,7 +965,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBlendState(ID3D10Device *ifa
     }
 
     TRACE("Created blend state %p.\n", object);
-    *blend_state = &object->ID3D10BlendState_iface;
+    *blend_state = (ID3D10BlendState *)object;
 
     return S_OK;
 }
@@ -1010,7 +994,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateDepthStencilState(ID3D10Devi
     }
 
     TRACE("Created depthstencil state %p.\n", object);
-    *depth_stencil_state = &object->ID3D10DepthStencilState_iface;
+    *depth_stencil_state = (ID3D10DepthStencilState *)object;
 
     return S_OK;
 }
@@ -1039,7 +1023,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRasterizerState(ID3D10Device
     }
 
     TRACE("Created rasterizer state %p.\n", object);
-    *rasterizer_state = &object->ID3D10RasterizerState_iface;
+    *rasterizer_state = (ID3D10RasterizerState *)object;
 
     return S_OK;
 }
@@ -1068,7 +1052,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateSamplerState(ID3D10Device *i
     }
 
     TRACE("Created sampler state %p.\n", object);
-    *sampler_state = &object->ID3D10SamplerState_iface;
+    *sampler_state = (ID3D10SamplerState *)object;
 
     return S_OK;
 }
@@ -1097,7 +1081,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateQuery(ID3D10Device *iface,
     }
 
     TRACE("Created query %p.\n", object);
-    *query = &object->ID3D10Query_iface;
+    *query = (ID3D10Query *)object;
 
     return S_OK;
 }
@@ -1293,82 +1277,60 @@ static const struct IUnknownVtbl d3d10_device_inner_unknown_vtbl =
 
 static void STDMETHODCALLTYPE d3d10_subresource_destroyed(void *parent) {}
 
-static const struct wined3d_parent_ops d3d10_subresource_parent_ops =
+const struct wined3d_parent_ops d3d10_subresource_parent_ops =
 {
     d3d10_subresource_destroyed,
 };
 
-/* IWineDXGIDeviceParent IUnknown methods */
+/* IWineD3DDeviceParent IUnknown methods */
 
-static inline struct d3d10_device *device_from_dxgi_device_parent(IWineDXGIDeviceParent *iface)
+static inline struct d3d10_device *device_from_device_parent(IWineD3DDeviceParent *iface)
 {
-    return CONTAINING_RECORD(iface, struct d3d10_device, IWineDXGIDeviceParent_iface);
+    return (struct d3d10_device *)((char*)iface - FIELD_OFFSET(struct d3d10_device, device_parent_vtbl));
 }
 
-static HRESULT STDMETHODCALLTYPE dxgi_device_parent_QueryInterface(IWineDXGIDeviceParent *iface,
-        REFIID riid, void **object)
+static HRESULT STDMETHODCALLTYPE device_parent_QueryInterface(IWineD3DDeviceParent *iface, REFIID riid, void **object)
 {
-    struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return d3d10_device_QueryInterface(&device->ID3D10Device_iface, riid, object);
+    struct d3d10_device *This = device_from_device_parent(iface);
+    return d3d10_device_QueryInterface((ID3D10Device *)This, riid, object);
 }
 
-static ULONG STDMETHODCALLTYPE dxgi_device_parent_AddRef(IWineDXGIDeviceParent *iface)
+static ULONG STDMETHODCALLTYPE device_parent_AddRef(IWineD3DDeviceParent *iface)
 {
-    struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return d3d10_device_AddRef(&device->ID3D10Device_iface);
+    struct d3d10_device *This = device_from_device_parent(iface);
+    return d3d10_device_AddRef((ID3D10Device *)This);
 }
 
-static ULONG STDMETHODCALLTYPE dxgi_device_parent_Release(IWineDXGIDeviceParent *iface)
+static ULONG STDMETHODCALLTYPE device_parent_Release(IWineD3DDeviceParent *iface)
 {
-    struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return d3d10_device_Release(&device->ID3D10Device_iface);
+    struct d3d10_device *This = device_from_device_parent(iface);
+    return d3d10_device_Release((ID3D10Device *)This);
 }
 
-static struct wined3d_device_parent * STDMETHODCALLTYPE dxgi_device_parent_get_wined3d_device_parent(
-        IWineDXGIDeviceParent *iface)
+/* IWineD3DDeviceParent methods */
+
+static void STDMETHODCALLTYPE device_parent_WineD3DDeviceCreated(IWineD3DDeviceParent *iface, IWineD3DDevice *device)
 {
-    struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return &device->device_parent;
+    struct d3d10_device *This = device_from_device_parent(iface);
+
+    TRACE("iface %p, device %p\n", iface, device);
+
+    IWineD3DDevice_AddRef(device);
+    This->wined3d_device = device;
 }
 
-static const struct IWineDXGIDeviceParentVtbl d3d10_dxgi_device_parent_vtbl =
-{
-    /* IUnknown methods */
-    dxgi_device_parent_QueryInterface,
-    dxgi_device_parent_AddRef,
-    dxgi_device_parent_Release,
-    /* IWineDXGIDeviceParent methods */
-    dxgi_device_parent_get_wined3d_device_parent,
-};
-
-static inline struct d3d10_device *device_from_wined3d_device_parent(struct wined3d_device_parent *device_parent)
-{
-    return CONTAINING_RECORD(device_parent, struct d3d10_device, device_parent);
-}
-
-static void CDECL device_parent_wined3d_device_created(struct wined3d_device_parent *device_parent,
-        struct wined3d_device *wined3d_device)
-{
-    struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
-
-    TRACE("device_parent %p, wined3d_device %p.\n", device_parent, wined3d_device);
-
-    wined3d_device_incref(wined3d_device);
-    device->wined3d_device = wined3d_device;
-}
-
-static HRESULT CDECL device_parent_create_surface(struct wined3d_device_parent *device_parent,
+static HRESULT STDMETHODCALLTYPE device_parent_CreateSurface(IWineD3DDeviceParent *iface,
         void *container_parent, UINT width, UINT height, enum wined3d_format_id format, DWORD usage,
-        WINED3DPOOL pool, UINT level, WINED3DCUBEMAP_FACES face, struct wined3d_surface **surface)
+        WINED3DPOOL pool, UINT level, WINED3DCUBEMAP_FACES face, IWineD3DSurface **surface)
 {
-    struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
+    struct d3d10_device *This = device_from_device_parent(iface);
     struct d3d10_texture2d *texture;
     D3D10_TEXTURE2D_DESC desc;
     HRESULT hr;
 
-    FIXME("device_parent %p, container_parent %p, width %u, height %u, format %#x, usage %#x,\n"
+    FIXME("iface %p, container_parent %p, width %u, height %u, format %#x, usage %#x,\n"
             "\tpool %#x, level %u, face %u, surface %p partial stub!\n",
-            device_parent, container_parent, width, height, format, usage, pool, level, face, surface);
+            iface, container_parent, width, height, format, usage, pool, level, face, surface);
 
     FIXME("Implement DXGI<->wined3d usage conversion\n");
 
@@ -1384,8 +1346,7 @@ static HRESULT CDECL device_parent_create_surface(struct wined3d_device_parent *
     desc.CPUAccessFlags = 0;
     desc.MiscFlags = 0;
 
-    hr = d3d10_device_CreateTexture2D(&device->ID3D10Device_iface, &desc, NULL,
-            (ID3D10Texture2D **)&texture);
+    hr = d3d10_device_CreateTexture2D((ID3D10Device *)This, &desc, NULL, (ID3D10Texture2D **)&texture);
     if (FAILED(hr))
     {
         ERR("CreateTexture2D failed, returning %#x\n", hr);
@@ -1393,26 +1354,25 @@ static HRESULT CDECL device_parent_create_surface(struct wined3d_device_parent *
     }
 
     *surface = texture->wined3d_surface;
-    wined3d_surface_incref(*surface);
-    ID3D10Texture2D_Release(&texture->ID3D10Texture2D_iface);
+    IWineD3DSurface_AddRef(*surface);
+    ID3D10Texture2D_Release((ID3D10Texture2D *)texture);
 
     return S_OK;
 }
 
-static HRESULT CDECL device_parent_create_rendertarget(struct wined3d_device_parent *device_parent,
+static HRESULT STDMETHODCALLTYPE device_parent_CreateRenderTarget(IWineD3DDeviceParent *iface,
         void *container_parent, UINT width, UINT height, enum wined3d_format_id format,
         WINED3DMULTISAMPLE_TYPE multisample_type, DWORD multisample_quality, BOOL lockable,
-        struct wined3d_surface **surface)
+        IWineD3DSurface **surface)
 {
-    struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
+    struct d3d10_device *This = device_from_device_parent(iface);
     struct d3d10_texture2d *texture;
     D3D10_TEXTURE2D_DESC desc;
     HRESULT hr;
 
-    FIXME("device_parent %p, container_parent %p, width %u, height %u, format %#x, multisample_type %#x,\n"
+    FIXME("iface %p, container_parent %p, width %u, height %u, format %#x, multisample_type %#x,\n"
             "\tmultisample_quality %u, lockable %u, surface %p partial stub!\n",
-            device_parent, container_parent, width, height, format, multisample_type,
-            multisample_quality, lockable, surface);
+            iface, container_parent, width, height, format, multisample_type, multisample_quality, lockable, surface);
 
     FIXME("Implement DXGI<->wined3d usage conversion\n");
 
@@ -1428,8 +1388,7 @@ static HRESULT CDECL device_parent_create_rendertarget(struct wined3d_device_par
     desc.CPUAccessFlags = 0;
     desc.MiscFlags = 0;
 
-    hr = d3d10_device_CreateTexture2D(&device->ID3D10Device_iface, &desc, NULL,
-            (ID3D10Texture2D **)&texture);
+    hr = d3d10_device_CreateTexture2D((ID3D10Device *)This, &desc, NULL, (ID3D10Texture2D **)&texture);
     if (FAILED(hr))
     {
         ERR("CreateTexture2D failed, returning %#x\n", hr);
@@ -1437,24 +1396,24 @@ static HRESULT CDECL device_parent_create_rendertarget(struct wined3d_device_par
     }
 
     *surface = texture->wined3d_surface;
-    wined3d_surface_incref(*surface);
-    ID3D10Texture2D_Release(&texture->ID3D10Texture2D_iface);
+    IWineD3DSurface_AddRef(*surface);
+    ID3D10Texture2D_Release((ID3D10Texture2D *)texture);
 
     return S_OK;
 }
 
-static HRESULT CDECL device_parent_create_depth_stencil(struct wined3d_device_parent *device_parent,
+static HRESULT STDMETHODCALLTYPE device_parent_CreateDepthStencilSurface(IWineD3DDeviceParent *iface,
         UINT width, UINT height, enum wined3d_format_id format, WINED3DMULTISAMPLE_TYPE multisample_type,
-        DWORD multisample_quality, BOOL discard, struct wined3d_surface **surface)
+        DWORD multisample_quality, BOOL discard, IWineD3DSurface **surface)
 {
-    struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
+    struct d3d10_device *This = device_from_device_parent(iface);
     struct d3d10_texture2d *texture;
     D3D10_TEXTURE2D_DESC desc;
     HRESULT hr;
 
-    FIXME("device_parent %p, width %u, height %u, format %#x, multisample_type %#x,\n"
+    FIXME("iface %p, width %u, height %u, format %#x, multisample_type %#x,\n"
             "\tmultisample_quality %u, discard %u, surface %p partial stub!\n",
-            device_parent, width, height, format, multisample_type, multisample_quality, discard, surface);
+            iface, width, height, format, multisample_type, multisample_quality, discard, surface);
 
     FIXME("Implement DXGI<->wined3d usage conversion\n");
 
@@ -1470,8 +1429,7 @@ static HRESULT CDECL device_parent_create_depth_stencil(struct wined3d_device_pa
     desc.CPUAccessFlags = 0;
     desc.MiscFlags = 0;
 
-    hr = d3d10_device_CreateTexture2D(&device->ID3D10Device_iface, &desc, NULL,
-            (ID3D10Texture2D **)&texture);
+    hr = d3d10_device_CreateTexture2D((ID3D10Device *)This, &desc, NULL, (ID3D10Texture2D **)&texture);
     if (FAILED(hr))
     {
         ERR("CreateTexture2D failed, returning %#x\n", hr);
@@ -1479,24 +1437,22 @@ static HRESULT CDECL device_parent_create_depth_stencil(struct wined3d_device_pa
     }
 
     *surface = texture->wined3d_surface;
-    wined3d_surface_incref(*surface);
-    ID3D10Texture2D_Release(&texture->ID3D10Texture2D_iface);
+    IWineD3DSurface_AddRef(*surface);
+    ID3D10Texture2D_Release((ID3D10Texture2D *)texture);
 
     return S_OK;
 }
 
-static HRESULT CDECL device_parent_create_volume(struct wined3d_device_parent *device_parent,
+static HRESULT STDMETHODCALLTYPE device_parent_CreateVolume(IWineD3DDeviceParent *iface,
         void *container_parent, UINT width, UINT height, UINT depth, enum wined3d_format_id format,
-        WINED3DPOOL pool, DWORD usage, struct wined3d_volume **volume)
+        WINED3DPOOL pool, DWORD usage, IWineD3DVolume **volume)
 {
     HRESULT hr;
 
-    TRACE("device_parent %p, container_parent %p, width %u, height %u, depth %u, "
-            "format %#x, pool %#x, usage %#x, volume %p.\n",
-            device_parent, container_parent, width, height, depth,
-            format, pool, usage, volume);
+    TRACE("iface %p, container_parent %p, width %u, height %u, depth %u, format %#x, pool %#x, usage %#x, volume %p.\n",
+            iface, container_parent, width, height, depth, format, pool, usage, volume);
 
-    hr = wined3d_volume_create(device_from_wined3d_device_parent(device_parent)->wined3d_device,
+    hr = IWineD3DDevice_CreateVolume(device_from_device_parent(iface)->wined3d_device,
             width, height, depth, usage, format, pool, NULL, &d3d10_subresource_parent_ops, volume);
     if (FAILED(hr))
     {
@@ -1507,17 +1463,15 @@ static HRESULT CDECL device_parent_create_volume(struct wined3d_device_parent *d
     return S_OK;
 }
 
-static HRESULT CDECL device_parent_create_swapchain(struct wined3d_device_parent *device_parent,
-        WINED3DPRESENT_PARAMETERS *present_parameters, struct wined3d_swapchain **swapchain)
+static HRESULT STDMETHODCALLTYPE device_parent_CreateSwapChain(IWineD3DDeviceParent *iface,
+        WINED3DPRESENT_PARAMETERS *present_parameters, IWineD3DSwapChain **swapchain)
 {
-    struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
     IWineDXGIDevice *wine_device;
     HRESULT hr;
 
-    TRACE("device_parent %p, present_parameters %p, swapchain %p\n", device_parent, present_parameters, swapchain);
+    TRACE("iface %p, present_parameters %p, swapchain %p\n", iface, present_parameters, swapchain);
 
-    hr = d3d10_device_QueryInterface(&device->ID3D10Device_iface, &IID_IWineDXGIDevice,
-            (void **)&wine_device);
+    hr = IWineD3DDeviceParent_QueryInterface(iface, &IID_IWineDXGIDevice, (void **)&wine_device);
     if (FAILED(hr))
     {
         ERR("Device should implement IWineDXGIDevice\n");
@@ -1535,22 +1489,26 @@ static HRESULT CDECL device_parent_create_swapchain(struct wined3d_device_parent
     return S_OK;
 }
 
-static const struct wined3d_device_parent_ops d3d10_wined3d_device_parent_ops =
+static const struct IWineD3DDeviceParentVtbl d3d10_wined3d_device_parent_vtbl =
 {
-    device_parent_wined3d_device_created,
-    device_parent_create_surface,
-    device_parent_create_rendertarget,
-    device_parent_create_depth_stencil,
-    device_parent_create_volume,
-    device_parent_create_swapchain,
+    /* IUnknown methods */
+    device_parent_QueryInterface,
+    device_parent_AddRef,
+    device_parent_Release,
+    /* IWineD3DDeviceParent methods */
+    device_parent_WineD3DDeviceCreated,
+    device_parent_CreateSurface,
+    device_parent_CreateRenderTarget,
+    device_parent_CreateDepthStencilSurface,
+    device_parent_CreateVolume,
+    device_parent_CreateSwapChain,
 };
 
 void d3d10_device_init(struct d3d10_device *device, void *outer_unknown)
 {
-    device->ID3D10Device_iface.lpVtbl = &d3d10_device_vtbl;
+    device->vtbl = &d3d10_device_vtbl;
     device->inner_unknown_vtbl = &d3d10_device_inner_unknown_vtbl;
-    device->IWineDXGIDeviceParent_iface.lpVtbl = &d3d10_dxgi_device_parent_vtbl;
-    device->device_parent.ops = &d3d10_wined3d_device_parent_ops;
+    device->device_parent_vtbl = &d3d10_wined3d_device_parent_vtbl;
     device->refcount = 1;
     device->outer_unknown = outer_unknown;
 }
