@@ -340,12 +340,8 @@ struct IDirect3DCubeTexture8Impl
 {
     IDirect3DCubeTexture8   IDirect3DCubeTexture8_iface;
     LONG                    ref;
-
-    /* IDirect3DResource8 fields */
-    IWineD3DCubeTexture    *wineD3DCubeTexture;
-
-    /* Parent reference */
-    IDirect3DDevice8       *parentDevice;
+    IWineD3DBaseTexture *wined3d_texture;
+    IDirect3DDevice8 *parentDevice;
 };
 
 HRESULT cubetexture_init(IDirect3DCubeTexture8Impl *texture, IDirect3DDevice8Impl *device,
@@ -362,12 +358,8 @@ struct IDirect3DTexture8Impl
 {
     IDirect3DTexture8   IDirect3DTexture8_iface;
     LONG                ref;
-
-    /* IDirect3DResourc8 fields */
-    IWineD3DTexture    *wineD3DTexture;
-
-    /* Parent reference */
-    IDirect3DDevice8   *parentDevice;
+    IWineD3DBaseTexture *wined3d_texture;
+    IDirect3DDevice8 *parentDevice;
 };
 
 HRESULT texture_init(IDirect3DTexture8Impl *texture, IDirect3DDevice8Impl *device,
@@ -382,15 +374,10 @@ HRESULT texture_init(IDirect3DTexture8Impl *texture, IDirect3DDevice8Impl *devic
  */
 struct IDirect3DVolumeTexture8Impl
 {
-    /* IUnknown fields */
-    const IDirect3DVolumeTexture8Vtbl *lpVtbl;
-    LONG                   ref;
-
-    /* IDirect3DResource8 fields */
-    IWineD3DVolumeTexture             *wineD3DVolumeTexture;
-
-    /* Parent reference */
-    LPDIRECT3DDEVICE8                  parentDevice;
+    IDirect3DVolumeTexture8 IDirect3DVolumeTexture8_iface;
+    LONG ref;
+    IWineD3DBaseTexture *wined3d_texture;
+    IDirect3DDevice8 *parentDevice;
 };
 
 HRESULT volumetexture_init(IDirect3DVolumeTexture8Impl *texture, IDirect3DDevice8Impl *device,
@@ -484,8 +471,8 @@ DECLARE_INTERFACE_(IDirect3DPixelShader8,IUnknown)
  */
 
 struct IDirect3DVertexShader8Impl {
-  const IDirect3DVertexShader8Vtbl *lpVtbl;
-  LONG ref;
+  IDirect3DVertexShader8            IDirect3DVertexShader8_iface;
+  LONG                              ref;
 
   IDirect3DVertexDeclaration8      *vertex_declaration;
   IWineD3DVertexShader             *wineD3DVertexShader;
@@ -500,7 +487,7 @@ HRESULT vertexshader_init(IDirect3DVertexShader8Impl *shader, IDirect3DDevice8Im
  * IDirect3DPixelShader implementation structure
  */
 typedef struct IDirect3DPixelShader8Impl {
-    const IDirect3DPixelShader8Vtbl *lpVtbl;
+    IDirect3DPixelShader8            IDirect3DPixelShader8_iface;
     LONG                             ref;
 
     DWORD                            handle;
