@@ -52,10 +52,10 @@ static ULONG WINAPI d3d8_vertexshader_AddRef(IDirect3DVertexShader8 *iface)
 
     TRACE("%p increasing refcount to %u.\n", iface, refcount);
 
-    if (refcount == 1 && shader->wined3d_shader)
+    if (refcount == 1 && shader->wineD3DVertexShader)
     {
         wined3d_mutex_lock();
-        wined3d_shader_incref(shader->wined3d_shader);
+        IWineD3DVertexShader_AddRef(shader->wineD3DVertexShader);
         wined3d_mutex_unlock();
     }
 
@@ -78,10 +78,10 @@ static ULONG WINAPI d3d8_vertexshader_Release(IDirect3DVertexShader8 *iface)
 
     if (!refcount)
     {
-        if (shader->wined3d_shader)
+        if (shader->wineD3DVertexShader)
         {
             wined3d_mutex_lock();
-            wined3d_shader_decref(shader->wined3d_shader);
+            IWineD3DVertexShader_Release(shader->wineD3DVertexShader);
             wined3d_mutex_unlock();
         }
         else
@@ -176,8 +176,8 @@ HRESULT vertexshader_init(IDirect3DVertexShader8Impl *shader, IDirect3DDevice8Im
         if (usage) FIXME("Usage %#x not implemented.\n", usage);
 
         wined3d_mutex_lock();
-        hr = wined3d_shader_create_vs(device->wined3d_device, byte_code, NULL /* output signature */,
-                shader, &d3d8_vertexshader_wined3d_parent_ops, &shader->wined3d_shader);
+        hr = IWineD3DDevice_CreateVertexShader(device->WineD3DDevice, byte_code, NULL /* output signature */,
+                shader, &d3d8_vertexshader_wined3d_parent_ops, &shader->wineD3DVertexShader);
         wined3d_mutex_unlock();
         if (FAILED(hr))
         {
@@ -186,7 +186,7 @@ HRESULT vertexshader_init(IDirect3DVertexShader8Impl *shader, IDirect3DDevice8Im
             return hr;
         }
 
-        load_local_constants(declaration, shader->wined3d_shader);
+        load_local_constants(declaration, shader->wineD3DVertexShader);
     }
 
     return D3D_OK;
@@ -225,7 +225,7 @@ static ULONG WINAPI d3d8_pixelshader_AddRef(IDirect3DPixelShader8 *iface)
     if (refcount == 1)
     {
         wined3d_mutex_lock();
-        wined3d_shader_incref(shader->wined3d_shader);
+        IWineD3DPixelShader_AddRef(shader->wineD3DPixelShader);
         wined3d_mutex_unlock();
     }
 
@@ -242,7 +242,7 @@ static ULONG WINAPI d3d8_pixelshader_Release(IDirect3DPixelShader8 *iface)
     if (!refcount)
     {
         wined3d_mutex_lock();
-        wined3d_shader_decref(shader->wined3d_shader);
+        IWineD3DPixelShader_Release(shader->wineD3DPixelShader);
         wined3d_mutex_unlock();
     }
 
@@ -277,8 +277,8 @@ HRESULT pixelshader_init(IDirect3DPixelShader8Impl *shader, IDirect3DDevice8Impl
     shader->handle = shader_handle;
 
     wined3d_mutex_lock();
-    hr = wined3d_shader_create_ps(device->wined3d_device, byte_code, NULL, shader,
-            &d3d8_pixelshader_wined3d_parent_ops, &shader->wined3d_shader);
+    hr = IWineD3DDevice_CreatePixelShader(device->WineD3DDevice, byte_code, NULL, shader,
+            &d3d8_pixelshader_wined3d_parent_ops, &shader->wineD3DPixelShader);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
