@@ -732,7 +732,7 @@ static int elf_new_wine_thunks(struct module* module, const struct hash_table* h
             struct location loc;
 
             symt = symt_find_nearest(module, addr);
-            if (symt && !symt_get_info(module, &symt->symt, TI_GET_ADDRESS, &ref_addr))
+            if (symt && !symt_get_address(&symt->symt, &ref_addr))
                 ref_addr = addr;
             if (!symt || addr != ref_addr)
             {
@@ -772,7 +772,7 @@ static int elf_new_wine_thunks(struct module* module, const struct hash_table* h
                 ULONG64 xaddr = 0, xsize = 0;
                 DWORD   kind = -1;
 
-                symt_get_info(module, &symt->symt, TI_GET_ADDRESS,  &xaddr);
+                symt_get_address(&symt->symt, &xaddr);
                 symt_get_info(module, &symt->symt, TI_GET_LENGTH,   &xsize);
                 symt_get_info(module, &symt->symt, TI_GET_DATAKIND, &kind);
 
