@@ -721,9 +721,8 @@ static void symt_fill_sym_info(struct module_pair* pair,
                         break;
                     case loc_regrel:
                         sym_info->Flags |= SYMFLAG_LOCAL | SYMFLAG_REGREL;
-                        sym_info->Register = loc.reg;
-                        if (loc.reg == CV_REG_NONE || (int)loc.reg < 0 /* error */)
-                            FIXME("suspicious register value %x\n", loc.reg);
+                        /* FIXME: it's i386 dependent !!! */
+                        sym_info->Register = loc.reg ? loc.reg : CV_REG_EBP;
                         sym_info->Address = loc.offset;
                         break;
                     case loc_absolute:
@@ -2087,27 +2086,3 @@ BOOL WINAPI SymGetLineFromNameW64(HANDLE hProcess, PCWSTR ModuleName, PCWSTR Fil
                 dwLineNumber, plDisplacement, Line);
     return FALSE;
 }
-
-/******************************************************************
- *		SymFromIndex (DBGHELP.@)
- *
- */
-BOOL WINAPI SymFromIndex(HANDLE hProcess, ULONG64 BaseOfDll, DWORD index, PSYMBOL_INFO symbol)
-{
-    FIXME("hProcess = %p, BaseOfDll = %s, index = %d, symbol = %p\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), index, symbol);
-
-    return FALSE;
-}
-
-/******************************************************************
- *		SymFromIndexW (DBGHELP.@)
- *
- */
-BOOL WINAPI SymFromIndexW(HANDLE hProcess, ULONG64 BaseOfDll, DWORD index, PSYMBOL_INFOW symbol)
-{
-    FIXME("hProcess = %p, BaseOfDll = %s, index = %d, symbol = %p\n",
-          hProcess, wine_dbgstr_longlong(BaseOfDll), index, symbol);
-
-    return FALSE;
-}
