@@ -37,7 +37,6 @@
 #include "wine/list.h"
 #include "wine/wined3d.h"
 
-extern const struct wined3d_parent_ops ddraw_surface_wined3d_parent_ops DECLSPEC_HIDDEN;
 extern const struct wined3d_parent_ops ddraw_null_wined3d_parent_ops DECLSPEC_HIDDEN;
 
 /* Typdef the interfaces */
@@ -52,6 +51,9 @@ typedef struct IDirect3DMaterialImpl      IDirect3DMaterialImpl;
 typedef struct IDirect3DExecuteBufferImpl IDirect3DExecuteBufferImpl;
 typedef struct IDirect3DVertexBufferImpl  IDirect3DVertexBufferImpl;
 
+/* Callbacks for implicit object destruction */
+extern ULONG WINAPI D3D7CB_DestroySwapChain(IWineD3DSwapChain *pSwapChain) DECLSPEC_HIDDEN;
+
 /* Global critical section */
 extern CRITICAL_SECTION ddraw_cs DECLSPEC_HIDDEN;
 
@@ -78,14 +80,14 @@ struct IDirectDrawImpl
     IDirect3D3 IDirect3D3_iface;
     IDirect3D2 IDirect3D2_iface;
     IDirect3D IDirect3D_iface;
-    struct wined3d_device_parent device_parent;
+    const IWineD3DDeviceParentVtbl *device_parent_vtbl;
 
     /* See comment in IDirectDraw::AddRef */
     LONG                    ref7, ref4, ref2, ref3, ref1, numIfaces;
 
     /* WineD3D linkage */
     struct wined3d *wineD3D;
-    struct wined3d_device *wined3d_device;
+    IWineD3DDevice          *wineD3DDevice;
     BOOL                    d3d_initialized;
 
     /* Misc ddraw fields */
@@ -111,6 +113,9 @@ struct IDirectDrawImpl
     HWND                    devicewindow;
     HWND                    dest_window;
 
+    /* The surface type to request */
+    WINED3DSURFTYPE         ImplType;
+
     /* Helpers for surface creation */
     IDirectDrawSurfaceImpl *tex_root;
     BOOL                    depthstencil;
@@ -132,6 +137,21 @@ struct IDirectDrawImpl
 
 HRESULT ddraw_init(IDirectDrawImpl *ddraw, WINED3DDEVTYPE device_type) DECLSPEC_HIDDEN;
 
+/* Helper structures */
+typedef struct EnumDisplayModesCBS
+{
+    void *context;
+    LPDDENUMMODESCALLBACK2 callback;
+} EnumDisplayModesCBS;
+
+typedef struct EnumSurfacesCBS
+{
+    void *context;
+    LPDDENUMSURFACESCALLBACK7 callback;
+    LPDDSURFACEDESC2 pDDSD;
+    DWORD Flags;
+} EnumSurfacesCBS;
+
 /* Utility functions */
 void DDRAW_Convert_DDSCAPS_1_To_2(const DDSCAPS *pIn, DDSCAPS2 *pOut) DECLSPEC_HIDDEN;
 void DDRAW_Convert_DDDEVICEIDENTIFIER_2_To_1(const DDDEVICEIDENTIFIER2 *pIn, DDDEVICEIDENTIFIER *pOut) DECLSPEC_HIDDEN;
@@ -149,25 +169,22 @@ extern WINED3DSURFTYPE DefaultSurfaceType DECLSPEC_HIDDEN;
 struct IDirectDrawSurfaceImpl
 {
     /* IUnknown fields */
-    IDirectDrawSurface7 IDirectDrawSurface7_iface;
-    IDirectDrawSurface4 IDirectDrawSurface4_iface;
-    IDirectDrawSurface3 IDirectDrawSurface3_iface;
-    IDirectDrawSurface2 IDirectDrawSurface2_iface;
-    IDirectDrawSurface IDirectDrawSurface_iface;
-    IDirectDrawGammaControl IDirectDrawGammaControl_iface;
-    IDirect3DTexture2 IDirect3DTexture2_iface;
-    IDirect3DTexture IDirect3DTexture_iface;
+    const IDirectDrawSurface7Vtbl *lpVtbl;
+    const IDirectDrawSurface3Vtbl *IDirectDrawSurface3_vtbl;
+    const IDirectDrawGammaControlVtbl *IDirectDrawGammaControl_vtbl;
+    const IDirect3DTexture2Vtbl *IDirect3DTexture2_vtbl;
+    const IDirect3DTextureVtbl *IDirect3DTexture_vtbl;
 
-    LONG                     ref7, ref4, ref3, ref2, ref1, iface_count, gamma_count;
+    LONG                     ref;
     IUnknown                *ifaceToRelease;
 
     int                     version;
 
     /* Connections to other Objects */
     IDirectDrawImpl         *ddraw;
-    struct wined3d_surface *wined3d_surface;
-    struct wined3d_texture *wined3d_texture;
-    struct wined3d_swapchain *wined3d_swapchain;
+    IWineD3DSurface         *WineD3DSurface;
+    IWineD3DBaseTexture     *wineD3DTexture;
+    IWineD3DSwapChain       *wineD3DSwapChain;
 
     /* This implementation handles attaching surfaces to other surfaces */
     IDirectDrawSurfaceImpl  *next_attached;
@@ -191,6 +208,7 @@ struct IDirectDrawSurfaceImpl
     /* Misc things */
     DWORD                   uniqueness_value;
     UINT                    mipmap_level;
+    WINED3DSURFTYPE         ImplType;
 
     /* For D3DDevice creation */
     BOOL                    isRenderTarget;
@@ -204,54 +222,25 @@ struct IDirectDrawSurfaceImpl
     DWORD                   Handle;
 };
 
-HRESULT ddraw_surface_create_texture(IDirectDrawSurfaceImpl *surface) DECLSPEC_HIDDEN;
 void ddraw_surface_destroy(IDirectDrawSurfaceImpl *surface) DECLSPEC_HIDDEN;
 HRESULT ddraw_surface_init(IDirectDrawSurfaceImpl *surface, IDirectDrawImpl *ddraw,
-        DDSURFACEDESC2 *desc, UINT mip_level, UINT version) DECLSPEC_HIDDEN;
-ULONG ddraw_surface_release_iface(IDirectDrawSurfaceImpl *This) DECLSPEC_HIDDEN;
+        DDSURFACEDESC2 *desc, UINT mip_level, WINED3DSURFTYPE surface_type) DECLSPEC_HIDDEN;
 
-static inline IDirectDrawSurfaceImpl *impl_from_IDirect3DTexture(IDirect3DTexture *iface)
+static inline IDirectDrawSurfaceImpl *surface_from_texture1(IDirect3DTexture *iface)
 {
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirect3DTexture_iface);
+    return (IDirectDrawSurfaceImpl *)((char*)iface - FIELD_OFFSET(IDirectDrawSurfaceImpl, IDirect3DTexture_vtbl));
 }
 
-static inline IDirectDrawSurfaceImpl *impl_from_IDirect3DTexture2(IDirect3DTexture2 *iface)
+static inline IDirectDrawSurfaceImpl *surface_from_texture2(IDirect3DTexture2 *iface)
 {
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirect3DTexture2_iface);
+    return (IDirectDrawSurfaceImpl *)((char*)iface - FIELD_OFFSET(IDirectDrawSurfaceImpl, IDirect3DTexture2_vtbl));
 }
 
-static inline IDirectDrawSurfaceImpl *impl_from_IDirectDrawSurface(IDirectDrawSurface *iface)
+static inline IDirectDrawSurfaceImpl *surface_from_surface3(IDirectDrawSurface3 *iface)
 {
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirectDrawSurface_iface);
+    return (IDirectDrawSurfaceImpl *)((char*)iface - FIELD_OFFSET(IDirectDrawSurfaceImpl, IDirectDrawSurface3_vtbl));
 }
 
-static inline IDirectDrawSurfaceImpl *impl_from_IDirectDrawSurface2(IDirectDrawSurface2 *iface)
-{
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirectDrawSurface2_iface);
-}
-
-static inline IDirectDrawSurfaceImpl *impl_from_IDirectDrawSurface3(IDirectDrawSurface3 *iface)
-{
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirectDrawSurface3_iface);
-}
-
-static inline IDirectDrawSurfaceImpl *impl_from_IDirectDrawSurface4(IDirectDrawSurface4 *iface)
-{
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirectDrawSurface4_iface);
-}
-
-static inline IDirectDrawSurfaceImpl *impl_from_IDirectDrawSurface7(IDirectDrawSurface7 *iface)
-{
-    return CONTAINING_RECORD(iface, IDirectDrawSurfaceImpl, IDirectDrawSurface7_iface);
-}
-
-IDirectDrawSurfaceImpl *unsafe_impl_from_IDirectDrawSurface(IDirectDrawSurface *iface) DECLSPEC_HIDDEN;
-IDirectDrawSurfaceImpl *unsafe_impl_from_IDirectDrawSurface4(IDirectDrawSurface4 *iface) DECLSPEC_HIDDEN;
-IDirectDrawSurfaceImpl *unsafe_impl_from_IDirectDrawSurface7(IDirectDrawSurface7 *iface) DECLSPEC_HIDDEN;
-
-IDirectDrawSurfaceImpl *unsafe_impl_from_IDirect3DTexture(IDirect3DTexture *iface) DECLSPEC_HIDDEN;
-IDirectDrawSurfaceImpl *unsafe_impl_from_IDirect3DTexture2(IDirect3DTexture2 *iface) DECLSPEC_HIDDEN;
-
 /* Get the number of bytes per pixel for a given surface */
 #define PFGET_BPP(pf) (pf.dwFlags&DDPF_PALETTEINDEXED8?1:((pf.dwRGBBitCount+7)/8))
 #define GET_BPP(desc) PFGET_BPP(desc.ddpfPixelFormat)
@@ -297,13 +286,13 @@ struct IDirect3DDeviceImpl
     LONG                    ref;
 
     /* Other object connections */
-    struct wined3d_device *wined3d_device;
+    IWineD3DDevice          *wineD3DDevice;
     IDirectDrawImpl         *ddraw;
     struct wined3d_buffer *indexbuffer;
     IDirectDrawSurfaceImpl  *target;
 
     /* Viewport management */
-    struct list viewport_list;
+    IDirect3DViewportImpl *viewport_list;
     IDirect3DViewportImpl *current_viewport;
     D3DVIEWPORT7 active_viewport;
 
@@ -353,12 +342,28 @@ static inline IDirect3DDeviceImpl *device_from_device3(IDirect3DDevice3 *iface)
     return (IDirect3DDeviceImpl *)((char*)iface - FIELD_OFFSET(IDirect3DDeviceImpl, IDirect3DDevice3_vtbl));
 }
 
+/* Structures */
+struct EnumTextureFormatsCBS
+{
+    LPD3DENUMTEXTUREFORMATSCALLBACK cbv2;
+    LPD3DENUMPIXELFORMATSCALLBACK cbv7;
+    void *Context;
+};
+
+/* Structure for EnumZBufferFormats */
+struct EnumZBufferFormatsData
+{
+    LPD3DENUMPIXELFORMATSCALLBACK Callback;
+    void *Context;
+};
+
 /*****************************************************************************
  * IDirectDrawClipper implementation structure
  *****************************************************************************/
 struct IDirectDrawClipperImpl
 {
-    IDirectDrawClipper IDirectDrawClipper_iface;
+    /* IUnknown fields */
+    const IDirectDrawClipperVtbl *lpVtbl;
     LONG ref;
 
     struct wined3d_clipper *wineD3DClipper;
@@ -366,7 +371,6 @@ struct IDirectDrawClipperImpl
 };
 
 HRESULT ddraw_clipper_init(IDirectDrawClipperImpl *clipper) DECLSPEC_HIDDEN;
-IDirectDrawClipperImpl *unsafe_impl_from_IDirectDrawClipper(IDirectDrawClipper *iface) DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirectDrawPalette implementation structure
@@ -399,7 +403,7 @@ struct object_creation_info
  ******************************************************************************/
 struct IDirect3DLightImpl
 {
-    IDirect3DLight IDirect3DLight_iface;
+    const IDirect3DLightVtbl *lpVtbl;
     LONG ref;
 
     /* IDirect3DLight fields */
@@ -413,23 +417,23 @@ struct IDirect3DLightImpl
 
     DWORD dwLightIndex;
 
-    struct list entry;
+    /* Chained list used for adding / removing from viewports */
+    IDirect3DLightImpl        *next;
 };
 
 /* Helper functions */
 void light_activate(IDirect3DLightImpl *light) DECLSPEC_HIDDEN;
 void light_deactivate(IDirect3DLightImpl *light) DECLSPEC_HIDDEN;
 void d3d_light_init(IDirect3DLightImpl *light, IDirectDrawImpl *ddraw) DECLSPEC_HIDDEN;
-IDirect3DLightImpl *unsafe_impl_from_IDirect3DLight(IDirect3DLight *iface) DECLSPEC_HIDDEN;
 
 /******************************************************************************
  * IDirect3DMaterial implementation structure - Wraps to D3D7
  ******************************************************************************/
 struct IDirect3DMaterialImpl
 {
-    IDirect3DMaterial3 IDirect3DMaterial3_iface;
-    IDirect3DMaterial2 IDirect3DMaterial2_iface;
-    IDirect3DMaterial IDirect3DMaterial_iface;
+    const IDirect3DMaterial3Vtbl *lpVtbl;
+    const IDirect3DMaterial2Vtbl *IDirect3DMaterial2_vtbl;
+    const IDirect3DMaterialVtbl *IDirect3DMaterial_vtbl;
     LONG  ref;
 
     /* IDirect3DMaterial2 fields */
@@ -442,7 +446,7 @@ struct IDirect3DMaterialImpl
 
 /* Helper functions */
 void material_activate(IDirect3DMaterialImpl* This) DECLSPEC_HIDDEN;
-IDirect3DMaterialImpl *d3d_material_create(IDirectDrawImpl *ddraw) DECLSPEC_HIDDEN;
+void d3d_material_init(IDirect3DMaterialImpl *material, IDirectDrawImpl *ddraw) DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirect3DViewport - Wraps to D3D7
@@ -469,8 +473,11 @@ struct IDirect3DViewportImpl
         D3DVIEWPORT2 vp2;
     } viewports;
 
-    struct list entry;
-    struct list light_list;
+    /* Field used to chain viewports together */
+    IDirect3DViewportImpl     *next;
+
+    /* Lights list */
+    IDirect3DLightImpl        *lights;
 
     /* Background material */
     IDirect3DMaterialImpl     *background;
@@ -485,8 +492,10 @@ void d3d_viewport_init(IDirect3DViewportImpl *viewport, IDirectDrawImpl *ddraw)
  *****************************************************************************/
 struct IDirect3DExecuteBufferImpl
 {
-    IDirect3DExecuteBuffer IDirect3DExecuteBuffer_iface;
-    LONG ref;
+    /* IUnknown */
+    const IDirect3DExecuteBufferVtbl *lpVtbl;
+    LONG                 ref;
+
     /* IDirect3DExecuteBuffer fields */
     IDirectDrawImpl      *ddraw;
     IDirect3DDeviceImpl  *d3ddev;
@@ -507,20 +516,20 @@ struct IDirect3DExecuteBufferImpl
 
 HRESULT d3d_execute_buffer_init(IDirect3DExecuteBufferImpl *execute_buffer,
         IDirect3DDeviceImpl *device, D3DEXECUTEBUFFERDESC *desc) DECLSPEC_HIDDEN;
-IDirect3DExecuteBufferImpl *unsafe_impl_from_IDirect3DExecuteBuffer(IDirect3DExecuteBuffer *iface) DECLSPEC_HIDDEN;
 
 /* The execute function */
-HRESULT d3d_execute_buffer_execute(IDirect3DExecuteBufferImpl *execute_buffer,
-        IDirect3DDeviceImpl *device, IDirect3DViewportImpl *viewport) DECLSPEC_HIDDEN;
+void IDirect3DExecuteBufferImpl_Execute(IDirect3DExecuteBufferImpl *This,
+        IDirect3DDeviceImpl *Device, IDirect3DViewportImpl *ViewportImpl) DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirect3DVertexBuffer
  *****************************************************************************/
 struct IDirect3DVertexBufferImpl
 {
-    IDirect3DVertexBuffer7 IDirect3DVertexBuffer7_iface;
-    IDirect3DVertexBuffer IDirect3DVertexBuffer_iface;
-    LONG ref;
+    /*** IUnknown Methods ***/
+    const IDirect3DVertexBuffer7Vtbl *lpVtbl;
+    const IDirect3DVertexBufferVtbl *IDirect3DVertexBuffer_vtbl;
+    LONG                 ref;
 
     /*** WineD3D and ddraw links ***/
     struct wined3d_buffer *wineD3DVertexBuffer;
@@ -532,10 +541,14 @@ struct IDirect3DVertexBufferImpl
     DWORD                fvf;
 };
 
-HRESULT d3d_vertex_buffer_create(IDirect3DVertexBufferImpl **vertex_buf, IDirectDrawImpl *ddraw,
-        D3DVERTEXBUFFERDESC *desc) DECLSPEC_HIDDEN;
-IDirect3DVertexBufferImpl *unsafe_impl_from_IDirect3DVertexBuffer(IDirect3DVertexBuffer *iface) DECLSPEC_HIDDEN;
-IDirect3DVertexBufferImpl *unsafe_impl_from_IDirect3DVertexBuffer7(IDirect3DVertexBuffer7 *iface) DECLSPEC_HIDDEN;
+HRESULT d3d_vertex_buffer_init(IDirect3DVertexBufferImpl *buffer,
+        IDirectDrawImpl *ddraw, D3DVERTEXBUFFERDESC *desc) DECLSPEC_HIDDEN;
+
+static inline IDirect3DVertexBufferImpl *vb_from_vb1(IDirect3DVertexBuffer *iface)
+{
+    return (IDirect3DVertexBufferImpl *)((char*)iface
+            - FIELD_OFFSET(IDirect3DVertexBufferImpl, IDirect3DVertexBuffer_vtbl));
+}
 
 /*****************************************************************************
  * Helper functions from utils.c
@@ -555,13 +568,14 @@ void DDRAW_dump_DDCAPS(const DDCAPS *lpcaps) DECLSPEC_HIDDEN;
 DWORD get_flexible_vertex_size(DWORD d3dvtVertexType) DECLSPEC_HIDDEN;
 void DDRAW_dump_DDSCAPS2(const DDSCAPS2 *in) DECLSPEC_HIDDEN;
 void DDRAW_dump_cooperativelevel(DWORD cooplevel) DECLSPEC_HIDDEN;
-void DDSD_to_DDSD2(const DDSURFACEDESC *in, DDSURFACEDESC2 *out) DECLSPEC_HIDDEN;
-void DDSD2_to_DDSD(const DDSURFACEDESC2 *in, DDSURFACEDESC *out) DECLSPEC_HIDDEN;
 
 /* This only needs to be here as long the processvertices functionality of
  * IDirect3DExecuteBuffer isn't in WineD3D */
 void multiply_matrix(LPD3DMATRIX dest, const D3DMATRIX *src1, const D3DMATRIX *src2) DECLSPEC_HIDDEN;
 
+/* Helper function in main.c */
+BOOL LoadWineD3D(void) DECLSPEC_HIDDEN;
+
 /* Used for generic dumping */
 typedef struct
 {
@@ -582,31 +596,22 @@ typedef struct
 /* Structure copy */
 #define ME(x,f,e) { x, #x, (void (*)(const void *))(f), offsetof(STRUCT, e) }
 
-#define DD_STRUCT_COPY_BYSIZE_(to,from,from_size)                 \
-    do {                                                          \
-        DWORD __size = (to)->dwSize;                              \
-        DWORD __resetsize = min(__size, sizeof(*to));             \
-        DWORD __copysize = min(__resetsize, from_size);           \
-        assert(to != from);                                       \
-        memcpy(to, from, __copysize);                             \
-        memset((char*)(to) + __copysize, 0, __resetsize - __copysize); \
-        (to)->dwSize = __size; /* restore size */                 \
-    } while (0)
+#define DD_STRUCT_COPY_BYSIZE(to,from)                  \
+        do {                                            \
+                DWORD __size = (to)->dwSize;            \
+                DWORD __copysize = __size;              \
+                DWORD __resetsize = __size;             \
+                assert(to != from);                     \
+                if (__resetsize > sizeof(*to))          \
+                    __resetsize = sizeof(*to);          \
+                memset(to,0,__resetsize);               \
+                if ((from)->dwSize < __size)            \
+                    __copysize = (from)->dwSize;        \
+                memcpy(to,from,__copysize);             \
+                (to)->dwSize = __size;/*restore size*/  \
+        } while (0)
 
-#define DD_STRUCT_COPY_BYSIZE(to,from) DD_STRUCT_COPY_BYSIZE_(to,from,(from)->dwSize)
-
-#define SIZEOF_END_PADDING(type, last_field) \
-    (sizeof(type) - offsetof(type, last_field) - sizeof(((type *)0)->last_field))
-
-static inline void copy_to_surfacedesc2(DDSURFACEDESC2 *to, DDSURFACEDESC2 *from)
-{
-    DWORD from_size = from->dwSize;
-    if (from_size == sizeof(DDSURFACEDESC))
-        from_size -= SIZEOF_END_PADDING(DDSURFACEDESC, ddsCaps);
-    to->dwSize = sizeof(DDSURFACEDESC2); /* for struct copy */
-    DD_STRUCT_COPY_BYSIZE_(to, from, from_size);
-}
-
-HRESULT hr_ddraw_from_wined3d(HRESULT hr) DECLSPEC_HIDDEN;
 
 #endif
+
+HRESULT hr_ddraw_from_wined3d(HRESULT hr) DECLSPEC_HIDDEN;
