@@ -306,13 +306,13 @@ static HRESULT GAMEUX_updateStatisticsFile(struct GAMEUX_STATS *stats)
 }
 /*******************************************************************************
  * GAMEUX_buildStatisticsFilePath
- * Creates path to file containing statistics of game with given id.
+ * Creates path to file contaning statistics of game with given id.
  *
  * Parameters:
  *  lpApplicationId                         [I]     application id of game,
  *                                                  as string
  *  lpStatisticsFile                        [O]     array where path will be
- *                                                  stored. Its size must be
+ *                                                  stored. It's size must be
  *                                                  at least MAX_PATH
  */
 static HRESULT GAMEUX_buildStatisticsFilePath(
@@ -635,7 +635,7 @@ static HRESULT GAMEUX_loadGameStatistics(struct GAMEUX_STATS *pStats,
     }
     else if(hr == S_FALSE && openType == GAMESTATS_OPEN_OPENORCREATE) /* file does not exist */
     {
-        /* create new statistics, not yet connected with file */
+        /* create new statitics, not yet connected with file */
         ZeroMemory(pStats->categories, sizeof(pStats->categories));
         *pOpenResult = GAMESTATS_OPEN_CREATED;
         hr = S_OK;
@@ -651,15 +651,20 @@ static HRESULT GAMEUX_loadGameStatistics(struct GAMEUX_STATS *pStats,
  */
 typedef struct _GameStatisticsImpl
 {
-    IGameStatistics IGameStatistics_iface;
+    const struct IGameStatisticsVtbl *lpVtbl;
     LONG ref;
     struct GAMEUX_STATS stats;
 } GameStatisticsImpl;
 
 static inline GameStatisticsImpl *impl_from_IGameStatistics( IGameStatistics *iface )
 {
-    return CONTAINING_RECORD(iface, GameStatisticsImpl, IGameStatistics_iface);
+    return (GameStatisticsImpl *)((char*)iface - FIELD_OFFSET(GameStatisticsImpl, lpVtbl));
 }
+static inline IGameStatistics *IGameStatistics_from_impl( GameStatisticsImpl* This )
+{
+    return (struct IGameStatistics*)&This->lpVtbl;
+}
+
 
 static HRESULT WINAPI GameStatisticsImpl_QueryInterface(
         IGameStatistics *iface,
@@ -814,13 +819,12 @@ static HRESULT WINAPI GameStatisticsImpl_GetCategoryTitle(
 
     TRACE("%p, %d, %p\n", This, categoryIndex, pTitle);
 
-    if(!pTitle)
-        return E_INVALIDARG;
     *pTitle = NULL;
 
-    if (categoryIndex >= MAX_CATEGORIES)
+    if(!pTitle || categoryIndex >= MAX_CATEGORIES)
         hr = E_INVALIDARG;
 
+
     if(SUCCEEDED(hr))
     {
         nLength = lstrlenW(This->stats.categories[categoryIndex].sName);
@@ -990,15 +994,15 @@ static const struct IGameStatisticsVtbl GameStatisticsImplVtbl =
 };
 
 
-static HRESULT create_IGameStatistics(GameStatisticsImpl** ppStats)
+HRESULT create_IGameStatistics(GameStatisticsImpl** ppStats)
 {
     TRACE("(%p)\n", ppStats);
 
-    *ppStats = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(**ppStats));
+    *ppStats = HeapAlloc( GetProcessHeap(), 0, sizeof(**ppStats));
     if(!(*ppStats))
         return E_OUTOFMEMORY;
 
-    (*ppStats)->IGameStatistics_iface.lpVtbl = &GameStatisticsImplVtbl;
+    (*ppStats)->lpVtbl = &GameStatisticsImplVtbl;
     (*ppStats)->ref = 1;
 
     TRACE("returning coclass: %p\n", *ppStats);
@@ -1010,13 +1014,13 @@ static HRESULT create_IGameStatistics(GameStatisticsImpl** ppStats)
  */
 typedef struct _GameStatisticsMgrImpl
 {
-    IGameStatisticsMgr IGameStatisticsMgr_iface;
+    const struct IGameStatisticsMgrVtbl *lpVtbl;
     LONG ref;
 } GameStatisticsMgrImpl;
 
 static inline GameStatisticsMgrImpl *impl_from_IGameStatisticsMgr( IGameStatisticsMgr *iface )
 {
-    return CONTAINING_RECORD(iface, GameStatisticsMgrImpl, IGameStatisticsMgr_iface);
+    return (GameStatisticsMgrImpl *)((char*)iface - FIELD_OFFSET(GameStatisticsMgrImpl, lpVtbl));
 }
 
 
@@ -1095,7 +1099,7 @@ static HRESULT STDMETHODCALLTYPE GameStatisticsMgrImpl_GetGameStatistics(
 
     if(SUCCEEDED(hr))
     {
-        output_iface = &statisticsImpl->IGameStatistics_iface;
+        output_iface = IGameStatistics_from_impl(statisticsImpl);
         hr = GAMEUX_buildStatisticsFilePath(lpApplicationId, statisticsImpl->stats.sStatsFile);
     }
 
@@ -1156,10 +1160,10 @@ HRESULT GameStatistics_create(
     if( !pGameStatistics )
         return E_OUTOFMEMORY;
 
-    pGameStatistics->IGameStatisticsMgr_iface.lpVtbl = &GameStatisticsMgrImplVtbl;
+    pGameStatistics->lpVtbl = &GameStatisticsMgrImplVtbl;
     pGameStatistics->ref = 1;
 
-    *ppObj = (IUnknown*)&pGameStatistics->IGameStatisticsMgr_iface;
+    *ppObj = (IUnknown*)(&pGameStatistics->lpVtbl);
 
     TRACE("returning iface %p\n", *ppObj);
     return S_OK;
