@@ -79,7 +79,6 @@ DC *alloc_dc_ptr( WORD magic )
 
     dc->nulldrv.funcs       = &null_driver;
     dc->nulldrv.next        = NULL;
-    dc->dibdrv              = NULL;
     dc->physDev             = &dc->nulldrv;
     dc->thread              = GetCurrentThreadId();
     dc->refcount            = 1;
@@ -180,13 +179,7 @@ void free_dc_ptr( DC *dc )
 {
     assert( dc->refcount == 1 );
 
-    while (dc->physDev != &dc->nulldrv)
-    {
-        PHYSDEV physdev = pop_dc_driver( &dc->physDev );
-        physdev->funcs->pDeleteDC( physdev );
-        if (physdev == dc->dibdrv) dc->dibdrv = NULL;
-    }
-    if (dc->dibdrv) dc->dibdrv->funcs->pDeleteDC( dc->dibdrv );
+    while (dc->physDev != &dc->nulldrv) pop_dc_driver( dc, dc->physDev );
     free_gdi_handle( dc->hSelf );
     free_dc_state( dc );
 }
@@ -247,6 +240,34 @@ void update_dc( DC *dc )
 }
 
 
+/***********************************************************************
+ *           push_dc_driver
+ *
+ * Push a driver on top of the DC driver stack.
+ */
+void push_dc_driver( DC * dc, PHYSDEV physdev, const DC_FUNCTIONS *funcs )
+{
+    physdev->funcs = funcs;
+    physdev->next = dc->physDev;
+    physdev->hdc = dc->hSelf;
+    dc->physDev = physdev;
+}
+
+
+/***********************************************************************
+ *           pop_dc_driver
+ *
+ * Pop the top driver from the DC driver stack.
+ */
+void pop_dc_driver( DC * dc, PHYSDEV physdev )
+{
+    assert( physdev == dc->physDev );
+    assert( physdev != &dc->nulldrv );
+    dc->physDev = physdev->next;
+    physdev->funcs->pDeleteDC( physdev );
+}
+
+
 /***********************************************************************
  *           DC_DeleteObject
  */
@@ -359,7 +380,7 @@ void DC_UpdateXforms( DC *dc )
 /***********************************************************************
  *           nulldrv_SaveDC
  */
-INT nulldrv_SaveDC( PHYSDEV dev )
+INT CDECL nulldrv_SaveDC( PHYSDEV dev )
 {
     DC *newdc, *dc = get_nulldrv_dc( dev );
 
@@ -447,7 +468,7 @@ INT nulldrv_SaveDC( PHYSDEV dev )
 /***********************************************************************
  *           restore_dc_state
  */
-BOOL nulldrv_RestoreDC( PHYSDEV dev, INT level )
+BOOL CDECL nulldrv_RestoreDC( PHYSDEV dev, INT level )
 {
     DC *dcs, *first_dcs, *dc = get_nulldrv_dc( dev );
     INT save_level;
@@ -629,11 +650,13 @@ HDC WINAPI CreateDCW( LPCWSTR driver, LPCWSTR device, LPCWSTR output,
 
     if (funcs->pCreateDC)
     {
-        if (!funcs->pCreateDC( &dc->physDev, buf, device, output, initData ))
+        PHYSDEV physdev;
+        if (!funcs->pCreateDC( hdc, &physdev, buf, device, output, initData ))
         {
             WARN("creation aborted by device\n" );
             goto error;
         }
+        push_dc_driver( dc, physdev, funcs );
     }
 
     dc->vis_rect.left   = 0;
@@ -718,7 +741,7 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
 {
     DC *dc, *origDC;
     HDC ret;
-    const struct gdi_dc_funcs *funcs = &null_driver;
+    const DC_FUNCTIONS *funcs = NULL;
     PHYSDEV physDev = NULL;
 
     GDI_CheckNotLock();
@@ -726,11 +749,14 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
     if (hdc)
     {
         if (!(origDC = get_dc_ptr( hdc ))) return 0;
-        physDev = GET_DC_PHYSDEV( origDC, pCreateCompatibleDC );
-        funcs = physDev->funcs;
+        physDev = GET_DC_PHYSDEV( origDC, pCreateDC );
+        if (physDev != &origDC->nulldrv) funcs = physDev->funcs;
+        else physDev = NULL;
         release_dc_ptr( origDC );
     }
 
+    if (!funcs && !(funcs = DRIVER_get_display_driver())) return 0;
+
     if (!(dc = alloc_dc_ptr( OBJ_MEMDC ))) goto error;
 
     TRACE("(%p): returning %p\n", hdc, dc->hSelf );
@@ -744,10 +770,17 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
 
     ret = dc->hSelf;
 
-    if (!funcs->pCreateCompatibleDC( physDev, &dc->physDev ))
+    /* Pass the driver-specific physical device info into
+     * the new DC. The driver may use this read-only info
+     * while creating the compatible DC. */
+    if (funcs->pCreateDC)
     {
-        WARN("creation aborted by device\n");
-        goto error;
+        if (!funcs->pCreateDC( dc->hSelf, &physDev, NULL, NULL, NULL, NULL ))
+        {
+            WARN("creation aborted by device\n");
+            goto error;
+        }
+        push_dc_driver( dc, physDev, funcs );
     }
     DC_InitDC( dc );
     release_dc_ptr( dc );
@@ -1364,7 +1397,7 @@ HCOLORSPACE WINAPI SetColorSpace( HDC hDC, HCOLORSPACE hColorSpace )
  */
 UINT WINAPI GetBoundsRect(HDC hdc, LPRECT rect, UINT flags)
 {
-    UINT ret = 0;
+    UINT ret;
     DC *dc = get_dc_ptr( hdc );
 
     if ( !dc ) return 0;
@@ -1372,15 +1405,18 @@ UINT WINAPI GetBoundsRect(HDC hdc, LPRECT rect, UINT flags)
     if (rect)
     {
         *rect = dc->BoundsRect;
-        ret = is_rect_empty( rect ) ? DCB_RESET : DCB_SET;
-        DPtoLP( hdc, (POINT *)rect, 2 );
+        ret = ((dc->flags & DC_BOUNDS_SET) ? DCB_SET : DCB_RESET);
     }
+    else
+        ret = 0;
+
     if (flags & DCB_RESET)
     {
         dc->BoundsRect.left   = 0;
         dc->BoundsRect.top    = 0;
         dc->BoundsRect.right  = 0;
         dc->BoundsRect.bottom = 0;
+        dc->flags &= ~DC_BOUNDS_SET;
     }
     release_dc_ptr( dc );
     return ret;
@@ -1399,7 +1435,7 @@ UINT WINAPI SetBoundsRect(HDC hdc, const RECT* rect, UINT flags)
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
     ret = ((dc->flags & DC_BOUNDS_ENABLE) ? DCB_ENABLE : DCB_DISABLE) |
-           (is_rect_empty( &dc->BoundsRect ) ? DCB_RESET : DCB_SET);
+          ((dc->flags & DC_BOUNDS_SET) ? DCB_SET : DCB_RESET);
 
     if (flags & DCB_RESET)
     {
@@ -1407,23 +1443,22 @@ UINT WINAPI SetBoundsRect(HDC hdc, const RECT* rect, UINT flags)
         dc->BoundsRect.top    = 0;
         dc->BoundsRect.right  = 0;
         dc->BoundsRect.bottom = 0;
+        dc->flags &= ~DC_BOUNDS_SET;
     }
 
-    if ((flags & DCB_ACCUMULATE) && rect)
+    if ((flags & DCB_ACCUMULATE) && rect && rect->left < rect->right && rect->top < rect->bottom)
     {
-        RECT rc = *rect;
-
-        LPtoDP( hdc, (POINT *)&rc, 2 );
-        if (!is_rect_empty( &rc ))
+        if (dc->flags & DC_BOUNDS_SET)
         {
-            if (!is_rect_empty( &dc->BoundsRect))
-            {
-                dc->BoundsRect.left   = min( dc->BoundsRect.left, rc.left );
-                dc->BoundsRect.top    = min( dc->BoundsRect.top, rc.top );
-                dc->BoundsRect.right  = max( dc->BoundsRect.right, rc.right );
-                dc->BoundsRect.bottom = max( dc->BoundsRect.bottom, rc.bottom );
-            }
-            else dc->BoundsRect = rc;
+            dc->BoundsRect.left   = min( dc->BoundsRect.left, rect->left );
+            dc->BoundsRect.top    = min( dc->BoundsRect.top, rect->top );
+            dc->BoundsRect.right  = max( dc->BoundsRect.right, rect->right );
+            dc->BoundsRect.bottom = max( dc->BoundsRect.bottom, rect->bottom );
+        }
+        else
+        {
+            dc->BoundsRect = *rect;
+            dc->flags |= DC_BOUNDS_SET;
         }
     }
 
