@@ -178,141 +178,96 @@ int DIB_GetBitmapInfo( const BITMAPINFOHEADER *header, LONG *width,
     return -1;
 }
 
+/* nulldrv fallback implementation using SetDIBits/StretchBlt */
+INT CDECL nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst, INT heightDst,
+                                 INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, const void *bits,
+                                 const BITMAPINFO *info, UINT coloruse, DWORD rop )
+{
+    DC *dc = get_nulldrv_dc( dev );
+    INT ret;
+    LONG width, height;
+    WORD planes, bpp;
+    DWORD compr, size;
+    HBITMAP hBitmap;
+    HDC hdcMem;
+
+    /* make sure we have a real implementation for StretchBlt and SetDIBits */
+    if (GET_DC_PHYSDEV( dc, pStretchBlt ) == dev || GET_DC_PHYSDEV( dc, pSetDIBits ) == dev)
+        return 0;
+
+    if (DIB_GetBitmapInfo( &info->bmiHeader, &width, &height, &planes, &bpp, &compr, &size ) == -1)
+        return 0;
+
+    if (width < 0) return 0;
+
+    if (xSrc == 0 && ySrc == 0 && widthDst == widthSrc && heightDst == heightSrc &&
+        info->bmiHeader.biCompression == BI_RGB)
+    {
+        /* Windows appears to have a fast case optimization
+         * that uses the wrong origin for top-down DIBs */
+        if (height < 0 && heightSrc < abs(height)) ySrc = abs(height) - heightSrc;
+
+        if (xDst == 0 && yDst == 0 && info->bmiHeader.biCompression == BI_RGB && rop == SRCCOPY)
+        {
+            BITMAP bm;
+            hBitmap = GetCurrentObject( dev->hdc, OBJ_BITMAP );
+            if (GetObjectW( hBitmap, sizeof(bm), &bm ) &&
+                bm.bmWidth == widthSrc && bm.bmHeight == heightSrc &&
+                bm.bmBitsPixel == bpp && bm.bmPlanes == planes)
+            {
+                /* fast path */
+                return SetDIBits( dev->hdc, hBitmap, 0, height, bits, info, coloruse );
+            }
+        }
+    }
+
+    hdcMem = CreateCompatibleDC( dev->hdc );
+    hBitmap = CreateCompatibleBitmap( dev->hdc, width, height );
+    SelectObject( hdcMem, hBitmap );
+    if (coloruse == DIB_PAL_COLORS)
+        SelectPalette( hdcMem, GetCurrentObject( dev->hdc, OBJ_PAL ), FALSE );
+
+    if (info->bmiHeader.biCompression == BI_RLE4 || info->bmiHeader.biCompression == BI_RLE8)
+    {
+        /* when RLE compression is used, there may be some gaps (ie the DIB doesn't
+         * contain all the rectangle described in bmiHeader, but only part of it.
+         * This mean that those undescribed pixels must be left untouched.
+         * So, we first copy on a memory bitmap the current content of the
+         * destination rectangle, blit the DIB bits on top of it - hence leaving
+         * the gaps untouched -, and blitting the rectangle back.
+         * This insure that gaps are untouched on the destination rectangle
+         */
+        StretchBlt( hdcMem, xSrc, abs(height) - heightSrc - ySrc, widthSrc, heightSrc,
+                    dev->hdc, xDst, yDst, widthDst, heightDst, rop );
+    }
+    ret = SetDIBits( hdcMem, hBitmap, 0, height, bits, info, coloruse );
+    if (ret) StretchBlt( dev->hdc, xDst, yDst, widthDst, heightDst,
+                         hdcMem, xSrc, abs(height) - heightSrc - ySrc, widthSrc, heightSrc, rop );
+    DeleteDC( hdcMem );
+    DeleteObject( hBitmap );
+    return ret;
+}
 
 /***********************************************************************
  *           StretchDIBits   (GDI32.@)
  */
-INT WINAPI StretchDIBits(HDC hdc, INT xDst, INT yDst, INT widthDst,
-                       INT heightDst, INT xSrc, INT ySrc, INT widthSrc,
-                       INT heightSrc, const void *bits,
-                       const BITMAPINFO *info, UINT wUsage, DWORD dwRop )
+INT WINAPI StretchDIBits(HDC hdc, INT xDst, INT yDst, INT widthDst, INT heightDst,
+                         INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, const void *bits,
+                         const BITMAPINFO *info, UINT coloruse, DWORD rop )
 {
     DC *dc;
-    INT ret;
+    INT ret = 0;
 
-    if (!bits || !info)
-	return 0;
+    if (!bits || !info) return 0;
 
-    if (!(dc = get_dc_ptr( hdc ))) return 0;
-
-    if(dc->funcs->pStretchDIBits)
+    if ((dc = get_dc_ptr( hdc )))
     {
+        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pStretchDIBits );
         update_dc( dc );
-        ret = dc->funcs->pStretchDIBits(dc->physDev, xDst, yDst, widthDst,
-                                        heightDst, xSrc, ySrc, widthSrc,
-                                        heightSrc, bits, info, wUsage, dwRop);
+        ret = physdev->funcs->pStretchDIBits( physdev, xDst, yDst, widthDst, heightDst,
+                                              xSrc, ySrc, widthSrc, heightSrc, bits, info, coloruse, rop );
         release_dc_ptr( dc );
     }
-    else /* use StretchBlt */
-    {
-        LONG height;
-        LONG width;
-        WORD planes, bpp;
-        DWORD compr, size;
-        HBITMAP hBitmap;
-        BOOL fastpath = FALSE;
-
-        release_dc_ptr( dc );
-
-        if (DIB_GetBitmapInfo( &info->bmiHeader, &width, &height, &planes, &bpp, &compr, &size ) == -1)
-        {
-            ERR("Invalid bitmap\n");
-            return 0;
-        }
-
-        if (width < 0)
-        {
-            ERR("Bitmap has a negative width\n");
-            return 0;
-        }
-
-        if (xSrc == 0 && ySrc == 0 && widthDst == widthSrc && heightDst == heightSrc &&
-            info->bmiHeader.biCompression == BI_RGB)
-        {
-            /* Windows appears to have a fast case optimization
-             * that uses the wrong origin for top-down DIBs */
-            if (height < 0 && heightSrc < abs(height)) ySrc = abs(height) - heightSrc;
-        }
-
-        hBitmap = GetCurrentObject(hdc, OBJ_BITMAP);
-
-        if (xDst == 0 && yDst == 0 && xSrc == 0 && ySrc == 0 &&
-            widthDst == widthSrc && heightDst == heightSrc &&
-            info->bmiHeader.biCompression == BI_RGB &&
-            dwRop == SRCCOPY)
-        {
-            BITMAPOBJ *bmp;
-            if ((bmp = GDI_GetObjPtr( hBitmap, OBJ_BITMAP )))
-            {
-                if (bmp->bitmap.bmBitsPixel == bpp &&
-                    bmp->bitmap.bmWidth == widthSrc &&
-                    bmp->bitmap.bmHeight == heightSrc &&
-                    bmp->bitmap.bmPlanes == planes)
-                    fastpath = TRUE;
-                GDI_ReleaseObj( hBitmap );
-            }
-        }
-
-        if (fastpath)
-        {
-            /* fast path */
-            TRACE("using fast path\n");
-            ret = SetDIBits( hdc, hBitmap, 0, height, bits, info, wUsage);
-        }
-        else
-        {
-            /* slow path - need to use StretchBlt */
-            HBITMAP hOldBitmap;
-            HPALETTE hpal = NULL;
-            HDC hdcMem;
-
-            hdcMem = CreateCompatibleDC( hdc );
-            hBitmap = CreateCompatibleBitmap(hdc, width, height);
-            hOldBitmap = SelectObject( hdcMem, hBitmap );
-            if(wUsage == DIB_PAL_COLORS)
-            {
-                hpal = GetCurrentObject(hdc, OBJ_PAL);
-                hpal = SelectPalette(hdcMem, hpal, FALSE);
-            }
-
-            if (info->bmiHeader.biCompression == BI_RLE4 ||
-	            info->bmiHeader.biCompression == BI_RLE8) {
-
-                /* when RLE compression is used, there may be some gaps (ie the DIB doesn't
-                 * contain all the rectangle described in bmiHeader, but only part of it.
-                 * This mean that those undescribed pixels must be left untouched.
-                 * So, we first copy on a memory bitmap the current content of the
-                 * destination rectangle, blit the DIB bits on top of it - hence leaving
-                 * the gaps untouched -, and blitting the rectangle back.
-                 * This insure that gaps are untouched on the destination rectangle
-                 * Not doing so leads to trashed images (the gaps contain what was on the
-                 * memory bitmap => generally black or garbage)
-                 * Unfortunately, RLE DIBs without gaps will be slowed down. But this is
-                 * another speed vs correctness issue. Anyway, if speed is needed, then the
-                 * pStretchDIBits function shall be implemented.
-                 * ericP (2000/09/09)
-                 */
-
-                /* copy existing bitmap from destination dc */
-                StretchBlt( hdcMem, xSrc, abs(height) - heightSrc - ySrc,
-                            widthSrc, heightSrc, hdc, xDst, yDst, widthDst, heightDst,
-                            dwRop );
-            }
-
-            ret = SetDIBits(hdcMem, hBitmap, 0, height, bits, info, wUsage);
-
-            /* Origin for DIBitmap may be bottom left (positive biHeight) or top
-               left (negative biHeight) */
-            if (ret) StretchBlt( hdc, xDst, yDst, widthDst, heightDst,
-                                 hdcMem, xSrc, abs(height) - heightSrc - ySrc,
-                                 widthSrc, heightSrc, dwRop );
-            if(hpal)
-                SelectPalette(hdcMem, hpal, FALSE);
-            SelectObject( hdcMem, hOldBitmap );
-            DeleteDC( hdcMem );
-            DeleteObject( hBitmap );
-        }
-    }
     return ret;
 }
 
@@ -341,6 +296,7 @@ INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
 {
     DC *dc = get_dc_ptr( hdc );
     BOOL delete_hdc = FALSE;
+    PHYSDEV physdev;
     BITMAPOBJ *bitmap;
     INT result = 0;
 
@@ -362,19 +318,10 @@ INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
         return 0;
     }
 
-    if (!bitmap->funcs && !BITMAP_SetOwnerDC( hbitmap, dc )) goto done;
+    physdev = GET_DC_PHYSDEV( dc, pSetDIBits );
+    if (BITMAP_SetOwnerDC( hbitmap, physdev ))
+        result = physdev->funcs->pSetDIBits( physdev, hbitmap, startscan, lines, bits, info, coloruse );
 
-    result = lines;
-    if (bitmap->funcs)
-    {
-        if (bitmap->funcs != dc->funcs)
-            ERR( "not supported: DDB bitmap %p not belonging to device %p\n", hbitmap, hdc );
-        else if (dc->funcs->pSetDIBits)
-            result = dc->funcs->pSetDIBits( dc->physDev, hbitmap, startscan, lines,
-                                            bits, info, coloruse );
-    }
-
- done:
     GDI_ReleaseObj( hbitmap );
     release_dc_ptr( dc );
     if (delete_hdc) DeleteDC(hdc);
@@ -390,26 +337,19 @@ INT WINAPI SetDIBitsToDevice(HDC hdc, INT xDest, INT yDest, DWORD cx,
                            UINT lines, LPCVOID bits, const BITMAPINFO *info,
                            UINT coloruse )
 {
-    INT ret;
+    INT ret = 0;
     DC *dc;
 
     if (!bits) return 0;
 
-    if (!(dc = get_dc_ptr( hdc ))) return 0;
-
-    if(dc->funcs->pSetDIBitsToDevice)
+    if ((dc = get_dc_ptr( hdc )))
     {
+        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pSetDIBitsToDevice );
         update_dc( dc );
-        ret = dc->funcs->pSetDIBitsToDevice( dc->physDev, xDest, yDest, cx, cy, xSrc,
-					     ySrc, startscan, lines, bits,
-					     info, coloruse );
+        ret = physdev->funcs->pSetDIBitsToDevice( physdev, xDest, yDest, cx, cy, xSrc,
+                                                  ySrc, startscan, lines, bits, info, coloruse );
+        release_dc_ptr( dc );
     }
-    else {
-        FIXME("unimplemented on hdc %p\n", hdc);
-	ret = 0;
-    }
-
-    release_dc_ptr( dc );
     return ret;
 }
 
@@ -426,6 +366,8 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, CONST RGBQUA
 
     if ((bitmap = GDI_GetObjPtr( dc->hBitmap, OBJ_BITMAP )))
     {
+        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pSetDIBColorTable );
+
         /* Check if currently selected bitmap is a DIB */
         if (bitmap->color_table)
         {
@@ -437,11 +379,8 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, CONST RGBQUA
             }
         }
         GDI_ReleaseObj( dc->hBitmap );
+        physdev->funcs->pSetDIBColorTable( physdev, startpos, entries, colors );
     }
-
-    if (dc->funcs->pSetDIBColorTable)
-        dc->funcs->pSetDIBColorTable(dc->physDev, startpos, entries, colors);
-
     release_dc_ptr( dc );
     return result;
 }
@@ -453,29 +392,24 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, CONST RGBQUA
 UINT WINAPI GetDIBColorTable( HDC hdc, UINT startpos, UINT entries, RGBQUAD *colors )
 {
     DC * dc;
+    BITMAPOBJ *bitmap;
     UINT result = 0;
 
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
-    if (dc->funcs->pGetDIBColorTable)
-        result = dc->funcs->pGetDIBColorTable(dc->physDev, startpos, entries, colors);
-    else
+    if ((bitmap = GDI_GetObjPtr( dc->hBitmap, OBJ_BITMAP )))
     {
-        BITMAPOBJ *bitmap = GDI_GetObjPtr( dc->hBitmap, OBJ_BITMAP );
-        if (bitmap)
+        /* Check if currently selected bitmap is a DIB */
+        if (bitmap->color_table)
         {
-            /* Check if currently selected bitmap is a DIB */
-            if (bitmap->color_table)
+            if (startpos < bitmap->nb_colors)
             {
-                if (startpos < bitmap->nb_colors)
-                {
-                    if (startpos + entries > bitmap->nb_colors) entries = bitmap->nb_colors - startpos;
-                    memcpy(colors, bitmap->color_table + startpos, entries * sizeof(RGBQUAD));
-                    result = entries;
-                }
+                if (startpos + entries > bitmap->nb_colors) entries = bitmap->nb_colors - startpos;
+                memcpy(colors, bitmap->color_table + startpos, entries * sizeof(RGBQUAD));
+                result = entries;
             }
-            GDI_ReleaseObj( dc->hBitmap );
         }
+        GDI_ReleaseObj( dc->hBitmap );
     }
     release_dc_ptr( dc );
     return result;
@@ -1086,15 +1020,10 @@ INT WINAPI GetDIBits(
         /* Otherwise, get bits from the XImage */
         else
         {
-            if (!bmp->funcs && !BITMAP_SetOwnerDC( hbitmap, dc )) lines = 0;
-            else
-            {
-                if (bmp->funcs && bmp->funcs->pGetDIBits)
-                    lines = bmp->funcs->pGetDIBits( dc->physDev, hbitmap, startscan,
-                                                    lines, bits, info, coloruse );
-                else
-                    lines = 0;  /* FIXME: should copy from bmp->bitmap.bmBits */
-            }
+            PHYSDEV physdev = GET_DC_PHYSDEV( dc, pGetDIBits );
+            if (!BITMAP_SetOwnerDC( hbitmap, physdev )) lines = 0;
+            else lines = physdev->funcs->pGetDIBits( physdev, hbitmap, startscan,
+                                                     lines, bits, info, coloruse );
         }
     }
     else lines = abs(height);
@@ -1133,7 +1062,6 @@ HBITMAP WINAPI CreateDIBitmap( HDC hdc, const BITMAPINFOHEADER *header,
     LONG height;
     WORD planes, bpp;
     DWORD compr, size;
-    DC *dc;
 
     if (!header) return 0;
 
@@ -1166,16 +1094,6 @@ HBITMAP WINAPI CreateDIBitmap( HDC hdc, const BITMAPINFOHEADER *header,
                 handle = 0;
             }
         }
-
-        else if (hdc && ((dc = get_dc_ptr( hdc )) != NULL) )
-        {
-            if (!BITMAP_SetOwnerDC( handle, dc ))
-            {
-                DeleteObject( handle );
-                handle = 0;
-            }
-            release_dc_ptr( dc );
-        }
     }
 
     return handle;
@@ -1390,19 +1308,17 @@ HBITMAP WINAPI CreateDIBSection(HDC hdc, CONST BITMAPINFO *bmi, UINT usage,
 
     if (ret && ((bmp = GDI_GetObjPtr(ret, OBJ_BITMAP))))
     {
+        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pCreateDIBSection );
         bmp->dib = dib;
-        bmp->funcs = dc->funcs;
+        bmp->funcs = physdev->funcs;
         /* create local copy of DIB palette */
         if (bpp <= 8) DIB_CopyColorTable( dc, bmp, usage, bmi );
         GDI_ReleaseObj( ret );
 
-        if (dc->funcs->pCreateDIBSection)
+        if (!physdev->funcs->pCreateDIBSection( physdev, ret, bmi, usage ))
         {
-            if (!dc->funcs->pCreateDIBSection(dc->physDev, ret, bmi, usage))
-            {
-                DeleteObject( ret );
-                ret = 0;
-            }
+            DeleteObject( ret );
+            ret = 0;
         }
     }
 
