@@ -41,9 +41,9 @@ WINE_DEFAULT_DEBUG_CHANNEL(driver);
 
 struct graphics_driver
 {
-    struct list                entry;
-    HMODULE                    module;  /* module handle */
-    const struct gdi_dc_funcs *funcs;
+    struct list             entry;
+    HMODULE                 module;  /* module handle */
+    DC_FUNCTIONS            funcs;
 };
 
 static struct list drivers = LIST_INIT( drivers );
@@ -65,39 +65,166 @@ static CRITICAL_SECTION driver_section = { &critsect_debug, -1, 0, 0, 0, 0 };
  */
 static struct graphics_driver *create_driver( HMODULE module )
 {
-    static const struct gdi_dc_funcs empty_funcs;
-    const struct gdi_dc_funcs *funcs = NULL;
     struct graphics_driver *driver;
 
     if (!(driver = HeapAlloc( GetProcessHeap(), 0, sizeof(*driver)))) return NULL;
     driver->module = module;
 
+    /* fill the function table */
     if (module)
     {
-        const struct gdi_dc_funcs * (CDECL *wine_get_gdi_driver)( unsigned int version );
+#define GET_FUNC(name) driver->funcs.p##name = (void*)GetProcAddress( module, #name )
+        GET_FUNC(AbortDoc);
+        GET_FUNC(AbortPath);
+        GET_FUNC(AlphaBlend);
+        GET_FUNC(AngleArc);
+        GET_FUNC(Arc);
+        GET_FUNC(ArcTo);
+        GET_FUNC(BeginPath);
+        GET_FUNC(ChoosePixelFormat);
+        GET_FUNC(Chord);
+        GET_FUNC(CloseFigure);
+        GET_FUNC(CreateBitmap);
+        GET_FUNC(CreateDC);
+        GET_FUNC(CreateDIBSection);
+        GET_FUNC(DeleteBitmap);
+        GET_FUNC(DeleteDC);
+        GET_FUNC(DescribePixelFormat);
+        GET_FUNC(DeviceCapabilities);
+        GET_FUNC(Ellipse);
+        GET_FUNC(EndDoc);
+        GET_FUNC(EndPage);
+        GET_FUNC(EndPath);
+        GET_FUNC(EnumDeviceFonts);
+        GET_FUNC(EnumICMProfiles);
+        GET_FUNC(ExcludeClipRect);
+        GET_FUNC(ExtDeviceMode);
+        GET_FUNC(ExtEscape);
+        GET_FUNC(ExtFloodFill);
+        GET_FUNC(ExtSelectClipRgn);
+        GET_FUNC(ExtTextOut);
+        GET_FUNC(FillPath);
+        GET_FUNC(FillRgn);
+        GET_FUNC(FlattenPath);
+        GET_FUNC(FrameRgn);
+        GET_FUNC(GdiComment);
+        GET_FUNC(GetBitmapBits);
+        GET_FUNC(GetCharWidth);
+        GET_FUNC(GetDIBits);
+        GET_FUNC(GetDeviceCaps);
+        GET_FUNC(GetDeviceGammaRamp);
+        GET_FUNC(GetICMProfile);
+        GET_FUNC(GetNearestColor);
+        GET_FUNC(GetPixel);
+        GET_FUNC(GetPixelFormat);
+        GET_FUNC(GetSystemPaletteEntries);
+        GET_FUNC(GetTextExtentExPoint);
+        GET_FUNC(GetTextMetrics);
+        GET_FUNC(IntersectClipRect);
+        GET_FUNC(InvertRgn);
+        GET_FUNC(LineTo);
+        GET_FUNC(MoveTo);
+        GET_FUNC(ModifyWorldTransform);
+        GET_FUNC(OffsetClipRgn);
+        GET_FUNC(OffsetViewportOrgEx);
+        GET_FUNC(OffsetWindowOrgEx);
+        GET_FUNC(PaintRgn);
+        GET_FUNC(PatBlt);
+        GET_FUNC(Pie);
+        GET_FUNC(PolyBezier);
+        GET_FUNC(PolyBezierTo);
+        GET_FUNC(PolyDraw);
+        GET_FUNC(PolyPolygon);
+        GET_FUNC(PolyPolyline);
+        GET_FUNC(Polygon);
+        GET_FUNC(Polyline);
+        GET_FUNC(PolylineTo);
+        GET_FUNC(RealizeDefaultPalette);
+        GET_FUNC(RealizePalette);
+        GET_FUNC(Rectangle);
+        GET_FUNC(ResetDC);
+        GET_FUNC(RestoreDC);
+        GET_FUNC(RoundRect);
+        GET_FUNC(SaveDC);
+        GET_FUNC(ScaleViewportExtEx);
+        GET_FUNC(ScaleWindowExtEx);
+        GET_FUNC(SelectBitmap);
+        GET_FUNC(SelectBrush);
+        GET_FUNC(SelectClipPath);
+        GET_FUNC(SelectFont);
+        GET_FUNC(SelectPalette);
+        GET_FUNC(SelectPen);
+        GET_FUNC(SetArcDirection);
+        GET_FUNC(SetBitmapBits);
+        GET_FUNC(SetBkColor);
+        GET_FUNC(SetBkMode);
+        GET_FUNC(SetDCBrushColor);
+        GET_FUNC(SetDCPenColor);
+        GET_FUNC(SetDIBColorTable);
+        GET_FUNC(SetDIBits);
+        GET_FUNC(SetDIBitsToDevice);
+        GET_FUNC(SetDeviceClipping);
+        GET_FUNC(SetDeviceGammaRamp);
+        GET_FUNC(SetMapMode);
+        GET_FUNC(SetMapperFlags);
+        GET_FUNC(SetPixel);
+        GET_FUNC(SetPixelFormat);
+        GET_FUNC(SetPolyFillMode);
+        GET_FUNC(SetROP2);
+        GET_FUNC(SetRelAbs);
+        GET_FUNC(SetStretchBltMode);
+        GET_FUNC(SetTextAlign);
+        GET_FUNC(SetTextCharacterExtra);
+        GET_FUNC(SetTextColor);
+        GET_FUNC(SetTextJustification);
+        GET_FUNC(SetViewportExtEx);
+        GET_FUNC(SetViewportOrgEx);
+        GET_FUNC(SetWindowExtEx);
+        GET_FUNC(SetWindowOrgEx);
+        GET_FUNC(SetWorldTransform);
+        GET_FUNC(StartDoc);
+        GET_FUNC(StartPage);
+        GET_FUNC(StretchBlt);
+        GET_FUNC(StretchDIBits);
+        GET_FUNC(StrokeAndFillPath);
+        GET_FUNC(StrokePath);
+        GET_FUNC(SwapBuffers);
+        GET_FUNC(UnrealizePalette);
+        GET_FUNC(WidenPath);
 
-        if ((wine_get_gdi_driver = (void *)GetProcAddress( module, "wine_get_gdi_driver" )))
-            funcs = wine_get_gdi_driver( WINE_GDI_DRIVER_VERSION );
+        /* OpenGL32 */
+        GET_FUNC(wglCreateContext);
+        GET_FUNC(wglCreateContextAttribsARB);
+        GET_FUNC(wglDeleteContext);
+        GET_FUNC(wglGetProcAddress);
+        GET_FUNC(wglGetPbufferDCARB);
+        GET_FUNC(wglMakeContextCurrentARB);
+        GET_FUNC(wglMakeCurrent);
+        GET_FUNC(wglSetPixelFormatWINE);
+        GET_FUNC(wglShareLists);
+        GET_FUNC(wglUseFontBitmapsA);
+        GET_FUNC(wglUseFontBitmapsW);
+#undef GET_FUNC
     }
-    if (!funcs) funcs = &empty_funcs;
-    driver->funcs = funcs;
+    else memset( &driver->funcs, 0, sizeof(driver->funcs) );
+
     return driver;
 }
 
 
 /**********************************************************************
- *	     get_display_driver
+ *	     DRIVER_get_display_driver
  *
  * Special case for loading the display driver: get the name from the config file
  */
-static const struct gdi_dc_funcs *get_display_driver(void)
+const DC_FUNCTIONS *DRIVER_get_display_driver(void)
 {
     struct graphics_driver *driver;
     char buffer[MAX_PATH], libname[32], *name, *next;
     HMODULE module = 0;
     HKEY hkey;
 
-    if (display_driver) return display_driver->funcs;  /* already loaded */
+    if (display_driver) return &display_driver->funcs;  /* already loaded */
 
     strcpy( buffer, "x11" );  /* default value */
     /* @@ Wine registry key: HKCU\Software\Wine\Drivers */
@@ -131,7 +258,7 @@ static const struct gdi_dc_funcs *get_display_driver(void)
         FreeLibrary( driver->module );
         HeapFree( GetProcessHeap(), 0, driver );
     }
-    return display_driver->funcs;
+    return &display_driver->funcs;
 }
 
 
@@ -146,11 +273,11 @@ const DC_FUNCTIONS *DRIVER_load_driver( LPCWSTR name )
     static const WCHAR display1W[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y','1',0};
 
     /* display driver is a special case */
-    if (!strcmpiW( name, displayW ) || !strcmpiW( name, display1W )) return get_display_driver();
+    if (!strcmpiW( name, displayW ) || !strcmpiW( name, display1W )) return DRIVER_get_display_driver();
 
     if ((module = GetModuleHandleW( name )))
     {
-        if (display_driver && display_driver->module == module) return display_driver->funcs;
+        if (display_driver && display_driver->module == module) return &display_driver->funcs;
         EnterCriticalSection( &driver_section );
         LIST_FOR_EACH_ENTRY( driver, &drivers, struct graphics_driver, entry )
         {
@@ -181,147 +308,145 @@ const DC_FUNCTIONS *DRIVER_load_driver( LPCWSTR name )
     TRACE( "loaded driver %p for %s\n", driver, debugstr_w(name) );
 done:
     LeaveCriticalSection( &driver_section );
-    return driver->funcs;
+    return &driver->funcs;
 }
 
 
-static INT nulldrv_AbortDoc( PHYSDEV dev )
+static INT CDECL nulldrv_AbortDoc( PHYSDEV dev )
 {
     return 0;
 }
 
-static BOOL nulldrv_AlphaBlend( PHYSDEV dst_dev, struct bitblt_coords *dst,
-                                PHYSDEV src_dev, struct bitblt_coords *src, BLENDFUNCTION func)
+static BOOL CDECL nulldrv_AlphaBlend( PHYSDEV dst_dev, INT x_dst, INT y_dst, INT width_dst, INT height_dst,
+                                      PHYSDEV src_dev, INT x_src, INT y_src, INT width_src, INT height_src,
+                                      BLENDFUNCTION func)
 {
     return TRUE;
 }
 
-static BOOL nulldrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                         INT xstart, INT ystart, INT xend, INT yend )
+static BOOL CDECL nulldrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                               INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static INT nulldrv_ChoosePixelFormat( PHYSDEV dev, const PIXELFORMATDESCRIPTOR *descr )
+static INT CDECL nulldrv_ChoosePixelFormat( PHYSDEV dev, const PIXELFORMATDESCRIPTOR *descr )
 {
     return 0;
 }
 
-static BOOL nulldrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                           INT xstart, INT ystart, INT xend, INT yend )
+static BOOL CDECL nulldrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                                 INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_CreateBitmap( PHYSDEV dev, HBITMAP bitmap )
+static BOOL CDECL nulldrv_CreateBitmap( PHYSDEV dev, HBITMAP bitmap, LPVOID bits )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
-{
-    if (!display_driver || !display_driver->funcs->pCreateCompatibleDC) return TRUE;
-    return display_driver->funcs->pCreateCompatibleDC( NULL, pdev );
-}
-
-static BOOL nulldrv_CreateDC( PHYSDEV *dev, LPCWSTR driver, LPCWSTR device,
-                              LPCWSTR output, const DEVMODEW *devmode )
+static BOOL CDECL nulldrv_CreateDC( HDC hdc, PHYSDEV *dev, LPCWSTR driver, LPCWSTR device,
+                                    LPCWSTR output, const DEVMODEW *devmode )
 {
     assert(0);  /* should never be called */
     return FALSE;
 }
 
-static HBITMAP nulldrv_CreateDIBSection( PHYSDEV dev, HBITMAP bitmap, BITMAPINFO *info, UINT usage )
+static HBITMAP CDECL nulldrv_CreateDIBSection( PHYSDEV dev, HBITMAP bitmap,
+                                               const BITMAPINFO *info, UINT usage )
 {
     return bitmap;
 }
 
-static BOOL nulldrv_DeleteBitmap( HBITMAP bitmap )
+static BOOL CDECL nulldrv_DeleteBitmap( HBITMAP bitmap )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_DeleteDC( PHYSDEV dev )
+static BOOL CDECL nulldrv_DeleteDC( PHYSDEV dev )
 {
     assert(0);  /* should never be called */
     return TRUE;
 }
 
-static BOOL nulldrv_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
+static BOOL CDECL nulldrv_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
 {
     return TRUE;
 }
 
-static INT nulldrv_DescribePixelFormat( PHYSDEV dev, INT format, UINT size, PIXELFORMATDESCRIPTOR * descr )
+static INT CDECL nulldrv_DescribePixelFormat( PHYSDEV dev, INT format,
+                                              UINT size, PIXELFORMATDESCRIPTOR * descr )
 {
     return 0;
 }
 
-static DWORD nulldrv_DeviceCapabilities( LPSTR buffer, LPCSTR device, LPCSTR port,
-                                         WORD cap, LPSTR output, DEVMODEA *devmode )
+static DWORD CDECL nulldrv_DeviceCapabilities( LPSTR buffer, LPCSTR device, LPCSTR port,
+                                               WORD cap, LPSTR output, DEVMODEA *devmode )
 {
     return -1;
 }
 
-static BOOL nulldrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
+static BOOL CDECL nulldrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
 {
     return TRUE;
 }
 
-static INT nulldrv_EndDoc( PHYSDEV dev )
+static INT CDECL nulldrv_EndDoc( PHYSDEV dev )
 {
     return 0;
 }
 
-static INT nulldrv_EndPage( PHYSDEV dev )
+static INT CDECL nulldrv_EndPage( PHYSDEV dev )
 {
     return 0;
 }
 
-static BOOL nulldrv_EnumDeviceFonts( PHYSDEV dev, LOGFONTW *logfont, FONTENUMPROCW proc, LPARAM lParam )
+static BOOL CDECL nulldrv_EnumDeviceFonts( PHYSDEV dev, LOGFONTW *logfont,
+                                           FONTENUMPROCW proc, LPARAM lParam )
 {
     return FALSE;
 }
 
-static INT nulldrv_EnumICMProfiles( PHYSDEV dev, ICMENUMPROCW func, LPARAM lparam )
+static INT CDECL nulldrv_EnumICMProfiles( PHYSDEV dev, ICMENUMPROCW func, LPARAM lparam )
 {
     return -1;
 }
 
-static INT nulldrv_ExtDeviceMode( LPSTR buffer, HWND hwnd, DEVMODEA *output, LPSTR device,
-                                  LPSTR port, DEVMODEA *input, LPSTR profile, DWORD mode )
+static INT CDECL nulldrv_ExtDeviceMode( LPSTR buffer, HWND hwnd, DEVMODEA *output, LPSTR device,
+                                        LPSTR port, DEVMODEA *input, LPSTR profile, DWORD mode )
 {
     return -1;
 }
 
-static INT nulldrv_ExtEscape( PHYSDEV dev, INT escape, INT in_size, const void *in_data,
+static INT CDECL nulldrv_ExtEscape( PHYSDEV dev, INT escape, INT in_size, const void *in_data,
                                     INT out_size, void *out_data )
 {
     return 0;
 }
 
-static BOOL nulldrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type )
+static BOOL CDECL nulldrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags, const RECT *rect,
-                                LPCWSTR str, UINT count, const INT *dx )
+static BOOL CDECL nulldrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags, const RECT *rect,
+                                      LPCWSTR str, UINT count, const INT *dx )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_GdiComment( PHYSDEV dev, UINT size, const BYTE *data )
+static BOOL CDECL nulldrv_GdiComment( PHYSDEV dev, UINT size, const BYTE *data )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT last, INT *buffer )
+static BOOL CDECL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT last, INT *buffer )
 {
     return FALSE;
 }
 
-static INT nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
+static INT CDECL nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
 {
     switch (cap)  /* return meaningful values for some entries */
     {
@@ -343,316 +468,325 @@ static INT nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
     }
 }
 
-static BOOL nulldrv_GetDeviceGammaRamp( PHYSDEV dev, void *ramp )
+static BOOL CDECL nulldrv_GetDeviceGammaRamp( PHYSDEV dev, void *ramp )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
+static BOOL CDECL nulldrv_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
 {
     return FALSE;
 }
 
-static COLORREF nulldrv_GetPixel( PHYSDEV dev, INT x, INT y )
+static COLORREF CDECL nulldrv_GetPixel( PHYSDEV dev, INT x, INT y )
 {
     return 0;
 }
 
-static INT nulldrv_GetPixelFormat( PHYSDEV dev )
+static INT CDECL nulldrv_GetPixelFormat( PHYSDEV dev )
 {
     return 0;
 }
 
-static UINT nulldrv_GetSystemPaletteEntries( PHYSDEV dev, UINT start, UINT count, PALETTEENTRY *entries )
+static UINT CDECL nulldrv_GetSystemPaletteEntries( PHYSDEV dev, UINT start,
+                                                   UINT count, PALETTEENTRY *entries )
 {
     return 0;
 }
 
-static BOOL nulldrv_GetTextExtentExPoint( PHYSDEV dev, LPCWSTR str, INT count, INT max_ext,
-                                          INT *fit, INT *dx, SIZE *size )
+static BOOL CDECL nulldrv_GetTextExtentExPoint( PHYSDEV dev, LPCWSTR str, INT count, INT max_ext,
+                                                INT *fit, INT *dx, SIZE *size )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_GetTextMetrics( PHYSDEV dev, TEXTMETRICW *metrics )
+static BOOL CDECL nulldrv_GetTextMetrics( PHYSDEV dev, TEXTMETRICW *metrics )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_LineTo( PHYSDEV dev, INT x, INT y )
+static BOOL CDECL nulldrv_LineTo( PHYSDEV dev, INT x, INT y )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_MoveTo( PHYSDEV dev, INT x, INT y )
+static BOOL CDECL nulldrv_MoveTo( PHYSDEV dev, INT x, INT y )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PaintRgn( PHYSDEV dev, HRGN rgn )
+static BOOL CDECL nulldrv_PaintRgn( PHYSDEV dev, HRGN rgn )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop )
+static BOOL CDECL nulldrv_PatBlt( PHYSDEV dev, INT x, INT y, INT width, INT height, DWORD rop )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                         INT xstart, INT ystart, INT xend, INT yend )
+static BOOL CDECL nulldrv_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                               INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_PolyPolygon( PHYSDEV dev, const POINT *points, const INT *counts, UINT polygons )
+static BOOL CDECL nulldrv_PolyPolygon( PHYSDEV dev, const POINT *points, const INT *counts, UINT polygons )
 {
     /* FIXME: could be implemented with Polygon */
     return TRUE;
 }
 
-static BOOL nulldrv_PolyPolyline( PHYSDEV dev, const POINT *points, const DWORD *counts, DWORD lines )
+static BOOL CDECL nulldrv_PolyPolyline( PHYSDEV dev, const POINT *points, const DWORD *counts, DWORD lines )
 {
     /* FIXME: could be implemented with Polyline */
     return TRUE;
 }
 
-static BOOL nulldrv_Polygon( PHYSDEV dev, const POINT *points, INT count )
+static BOOL CDECL nulldrv_Polygon( PHYSDEV dev, const POINT *points, INT count )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
+static BOOL CDECL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
 {
     return TRUE;
 }
 
-static UINT nulldrv_RealizeDefaultPalette( PHYSDEV dev )
+static UINT CDECL nulldrv_RealizeDefaultPalette( PHYSDEV dev )
 {
     return 0;
 }
 
-static UINT nulldrv_RealizePalette( PHYSDEV dev, HPALETTE palette, BOOL primary )
+static UINT CDECL nulldrv_RealizePalette( PHYSDEV dev, HPALETTE palette, BOOL primary )
 {
     return 0;
 }
 
-static BOOL nulldrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
+static BOOL CDECL nulldrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
 {
     return TRUE;
 }
 
-static HDC nulldrv_ResetDC( PHYSDEV dev, const DEVMODEW *devmode )
+static HDC CDECL nulldrv_ResetDC( PHYSDEV dev, const DEVMODEW *devmode )
 {
     return 0;
 }
 
-static BOOL nulldrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                               INT ell_width, INT ell_height )
+static BOOL CDECL nulldrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                                     INT ell_width, INT ell_height )
 {
     return TRUE;
 }
 
-static HBITMAP nulldrv_SelectBitmap( PHYSDEV dev, HBITMAP bitmap )
+static HBITMAP CDECL nulldrv_SelectBitmap( PHYSDEV dev, HBITMAP bitmap )
 {
     return bitmap;
 }
 
-static HBRUSH nulldrv_SelectBrush( PHYSDEV dev, HBRUSH brush )
+static HBRUSH CDECL nulldrv_SelectBrush( PHYSDEV dev, HBRUSH brush )
 {
     return brush;
 }
 
-static HFONT nulldrv_SelectFont( PHYSDEV dev, HFONT font, HANDLE gdi_font )
+static HFONT CDECL nulldrv_SelectFont( PHYSDEV dev, HFONT font, HANDLE gdi_font )
 {
     return 0;
 }
 
-static HPALETTE nulldrv_SelectPalette( PHYSDEV dev, HPALETTE palette, BOOL bkgnd )
+static HPALETTE CDECL nulldrv_SelectPalette( PHYSDEV dev, HPALETTE palette, BOOL bkgnd )
 {
     return palette;
 }
 
-static HPEN nulldrv_SelectPen( PHYSDEV dev, HPEN pen )
+static HPEN CDECL nulldrv_SelectPen( PHYSDEV dev, HPEN pen )
 {
     return pen;
 }
 
-static INT nulldrv_SetArcDirection( PHYSDEV dev, INT dir )
+static INT CDECL nulldrv_SetArcDirection( PHYSDEV dev, INT dir )
 {
     return dir;
 }
 
-static COLORREF nulldrv_SetBkColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetBkColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static INT nulldrv_SetBkMode( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetBkMode( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static COLORREF nulldrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static COLORREF nulldrv_SetDCPenColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetDCPenColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static UINT nulldrv_SetDIBColorTable( PHYSDEV dev, UINT pos, UINT count, const RGBQUAD *colors )
+static UINT CDECL nulldrv_SetDIBColorTable( PHYSDEV dev, UINT pos, UINT count, const RGBQUAD *colors )
 {
     return 0;
 }
 
-static void nulldrv_SetDeviceClipping( PHYSDEV dev, HRGN vis_rgn, HRGN clip_rgn )
+static INT CDECL nulldrv_SetDIBitsToDevice( PHYSDEV dev, INT x_dst, INT y_dst, DWORD width, DWORD height,
+                                            INT x_src, INT y_src, UINT start, UINT lines,
+                                            const void *bits, const BITMAPINFO *info, UINT coloruse )
+{
+    return 0;
+}
+
+static void CDECL nulldrv_SetDeviceClipping( PHYSDEV dev, HRGN vis_rgn, HRGN clip_rgn )
 {
 }
 
-static DWORD nulldrv_SetLayout( PHYSDEV dev, DWORD layout )
+static DWORD CDECL nulldrv_SetLayout( PHYSDEV dev, DWORD layout )
 {
     return layout;
 }
 
-static BOOL nulldrv_SetDeviceGammaRamp( PHYSDEV dev, void *ramp )
+static BOOL CDECL nulldrv_SetDeviceGammaRamp( PHYSDEV dev, void *ramp )
 {
     return FALSE;
 }
 
-static DWORD nulldrv_SetMapperFlags( PHYSDEV dev, DWORD flags )
+static DWORD CDECL nulldrv_SetMapperFlags( PHYSDEV dev, DWORD flags )
 {
     return flags;
 }
 
-static COLORREF nulldrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
+static COLORREF CDECL nulldrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
 {
     return color;
 }
 
-static BOOL nulldrv_SetPixelFormat( PHYSDEV dev, INT format, const PIXELFORMATDESCRIPTOR *descr )
+static BOOL CDECL nulldrv_SetPixelFormat( PHYSDEV dev, INT format, const PIXELFORMATDESCRIPTOR *descr )
 {
     return FALSE;
 }
 
-static INT nulldrv_SetPolyFillMode( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetPolyFillMode( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static INT nulldrv_SetROP2( PHYSDEV dev, INT rop )
+static INT CDECL nulldrv_SetROP2( PHYSDEV dev, INT rop )
 {
     return rop;
 }
 
-static INT nulldrv_SetRelAbs( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetRelAbs( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static INT nulldrv_SetStretchBltMode( PHYSDEV dev, INT mode )
+static INT CDECL nulldrv_SetStretchBltMode( PHYSDEV dev, INT mode )
 {
     return mode;
 }
 
-static UINT nulldrv_SetTextAlign( PHYSDEV dev, UINT align )
+static UINT CDECL nulldrv_SetTextAlign( PHYSDEV dev, UINT align )
 {
     return align;
 }
 
-static INT nulldrv_SetTextCharacterExtra( PHYSDEV dev, INT extra )
+static INT CDECL nulldrv_SetTextCharacterExtra( PHYSDEV dev, INT extra )
 {
     return extra;
 }
 
-static COLORREF nulldrv_SetTextColor( PHYSDEV dev, COLORREF color )
+static COLORREF CDECL nulldrv_SetTextColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static BOOL nulldrv_SetTextJustification( PHYSDEV dev, INT extra, INT breaks )
+static BOOL CDECL nulldrv_SetTextJustification( PHYSDEV dev, INT extra, INT breaks )
 {
     return TRUE;
 }
 
-static INT nulldrv_StartDoc( PHYSDEV dev, const DOCINFOW *info )
+static INT CDECL nulldrv_StartDoc( PHYSDEV dev, const DOCINFOW *info )
 {
     return 0;
 }
 
-static INT nulldrv_StartPage( PHYSDEV dev )
+static INT CDECL nulldrv_StartPage( PHYSDEV dev )
 {
     return 1;
 }
 
-static BOOL nulldrv_SwapBuffers( PHYSDEV dev )
+static BOOL CDECL nulldrv_SwapBuffers( PHYSDEV dev )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_UnrealizePalette( HPALETTE palette )
+static BOOL CDECL nulldrv_UnrealizePalette( HPALETTE palette )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglCopyContext( HGLRC ctx_src, HGLRC ctx_dst, UINT mask )
+static BOOL CDECL nulldrv_wglCopyContext( HGLRC ctx_src, HGLRC ctx_dst, UINT mask )
 {
     return FALSE;
 }
 
-static HGLRC nulldrv_wglCreateContext( PHYSDEV dev )
+static HGLRC CDECL nulldrv_wglCreateContext( PHYSDEV dev )
 {
     return 0;
 }
 
-static HGLRC nulldrv_wglCreateContextAttribsARB( PHYSDEV dev, HGLRC share_ctx, const int *attribs )
+static HGLRC CDECL nulldrv_wglCreateContextAttribsARB( PHYSDEV dev, HGLRC share_ctx, const int *attribs )
 {
     return 0;
 }
 
-static BOOL nulldrv_wglDeleteContext( HGLRC ctx )
+static BOOL CDECL nulldrv_wglDeleteContext( HGLRC ctx )
 {
     return FALSE;
 }
 
-static PROC nulldrv_wglGetProcAddress( LPCSTR name )
+static PROC CDECL nulldrv_wglGetProcAddress( LPCSTR name )
 {
     return NULL;
 }
 
-static HDC nulldrv_wglGetPbufferDCARB( PHYSDEV dev, void *pbuffer )
+static HDC CDECL nulldrv_wglGetPbufferDCARB( PHYSDEV dev, void *pbuffer )
 {
     return 0;
 }
 
-static BOOL nulldrv_wglMakeCurrent( PHYSDEV dev, HGLRC ctx )
+static BOOL CDECL nulldrv_wglMakeCurrent( PHYSDEV dev, HGLRC ctx )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglMakeContextCurrentARB( PHYSDEV dev_draw, PHYSDEV dev_read, HGLRC ctx )
+static BOOL CDECL nulldrv_wglMakeContextCurrentARB( PHYSDEV dev_draw, PHYSDEV dev_read, HGLRC ctx )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglSetPixelFormatWINE( PHYSDEV dev, INT format, const PIXELFORMATDESCRIPTOR *descr )
+static BOOL CDECL nulldrv_wglSetPixelFormatWINE( PHYSDEV dev, INT format,
+                                                 const PIXELFORMATDESCRIPTOR *descr )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglShareLists( HGLRC ctx1, HGLRC ctx2 )
+static BOOL CDECL nulldrv_wglShareLists( HGLRC ctx1, HGLRC ctx2 )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglUseFontBitmapsA( PHYSDEV dev, DWORD start, DWORD count, DWORD base )
+static BOOL CDECL nulldrv_wglUseFontBitmapsA( PHYSDEV dev, DWORD start, DWORD count, DWORD base )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglUseFontBitmapsW( PHYSDEV dev, DWORD start, DWORD count, DWORD base )
+static BOOL CDECL nulldrv_wglUseFontBitmapsW( PHYSDEV dev, DWORD start, DWORD count, DWORD base )
 {
     return FALSE;
 }
@@ -670,7 +804,6 @@ const DC_FUNCTIONS null_driver =
     nulldrv_Chord,                      /* pChord */
     nulldrv_CloseFigure,                /* pCloseFigure */
     nulldrv_CreateBitmap,               /* pCreateBitmap */
-    nulldrv_CreateCompatibleDC,         /* pCreateCompatibleDC */
     nulldrv_CreateDC,                   /* pCreateDC */
     nulldrv_CreateDIBSection,           /* pCreateDIBSection */
     nulldrv_DeleteBitmap,               /* pDeleteBitmap */
@@ -695,11 +828,12 @@ const DC_FUNCTIONS null_driver =
     nulldrv_FlattenPath,                /* pFlattenPath */
     nulldrv_FrameRgn,                   /* pFrameRgn */
     nulldrv_GdiComment,                 /* pGdiComment */
+    nulldrv_GetBitmapBits,              /* pGetBitmapBits */
     nulldrv_GetCharWidth,               /* pGetCharWidth */
+    nulldrv_GetDIBits,                  /* pGetDIBits */
     nulldrv_GetDeviceCaps,              /* pGetDeviceCaps */
     nulldrv_GetDeviceGammaRamp,         /* pGetDeviceGammaRamp */
     nulldrv_GetICMProfile,              /* pGetICMProfile */
-    nulldrv_GetImage,                   /* pGetImage */
     nulldrv_GetNearestColor,            /* pGetNearestColor */
     nulldrv_GetPixel,                   /* pGetPixel */
     nulldrv_GetPixelFormat,             /* pGetPixelFormat */
@@ -725,7 +859,6 @@ const DC_FUNCTIONS null_driver =
     nulldrv_Polygon,                    /* pPolygon */
     nulldrv_Polyline,                   /* pPolyline */
     nulldrv_PolylineTo,                 /* pPolylineTo */
-    nulldrv_PutImage,                   /* pPutImage */
     nulldrv_RealizeDefaultPalette,      /* pRealizeDefaultPalette */
     nulldrv_RealizePalette,             /* pRealizePalette */
     nulldrv_Rectangle,                  /* pRectangle */
@@ -742,11 +875,13 @@ const DC_FUNCTIONS null_driver =
     nulldrv_SelectPalette,              /* pSelectPalette */
     nulldrv_SelectPen,                  /* pSelectPen */
     nulldrv_SetArcDirection,            /* pSetArcDirection */
+    nulldrv_SetBitmapBits,              /* pSetBitmapBits */
     nulldrv_SetBkColor,                 /* pSetBkColor */
     nulldrv_SetBkMode,                  /* pSetBkMode */
     nulldrv_SetDCBrushColor,            /* pSetDCBrushColor */
     nulldrv_SetDCPenColor,              /* pSetDCPenColor */
     nulldrv_SetDIBColorTable,           /* pSetDIBColorTable */
+    nulldrv_SetDIBits,                  /* pSetDIBits */
     nulldrv_SetDIBitsToDevice,          /* pSetDIBitsToDevice */
     nulldrv_SetDeviceClipping,          /* pSetDeviceClipping */
     nulldrv_SetDeviceGammaRamp,         /* pSetDeviceGammaRamp */
@@ -781,10 +916,10 @@ const DC_FUNCTIONS null_driver =
     nulldrv_wglCreateContext,           /* pwglCreateContext */
     nulldrv_wglCreateContextAttribsARB, /* pwglCreateContextAttribsARB */
     nulldrv_wglDeleteContext,           /* pwglDeleteContext */
-    nulldrv_wglGetPbufferDCARB,         /* pwglGetPbufferDCARB */
     nulldrv_wglGetProcAddress,          /* pwglGetProcAddress */
-    nulldrv_wglMakeContextCurrentARB,   /* pwglMakeContextCurrentARB */
+    nulldrv_wglGetPbufferDCARB,         /* pwglGetPbufferDCARB */
     nulldrv_wglMakeCurrent,             /* pwglMakeCurrent */
+    nulldrv_wglMakeContextCurrentARB,   /* pwglMakeContextCurrentARB */
     nulldrv_wglSetPixelFormatWINE,      /* pwglSetPixelFormatWINE */
     nulldrv_wglShareLists,              /* pwglShareLists */
     nulldrv_wglUseFontBitmapsA,         /* pwglUseFontBitmapsA */
