@@ -32,7 +32,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(enhmetafile);
 
-static BOOL EMFDRV_DeleteDC( PHYSDEV dev );
+static BOOL CDECL EMFDRV_DeleteDC( PHYSDEV dev );
 
 static const DC_FUNCTIONS EMFDRV_Funcs =
 {
@@ -47,7 +47,6 @@ static const DC_FUNCTIONS EMFDRV_Funcs =
     EMFDRV_Chord,                    /* pChord */
     EMFDRV_CloseFigure,              /* pCloseFigure */
     NULL,                            /* pCreateBitmap */
-    NULL,                            /* pCreateCompatibleDC */
     NULL,                            /* pCreateDC */
     NULL,                            /* pCreateDIBSection */
     NULL,                            /* pDeleteBitmap */
@@ -72,11 +71,12 @@ static const DC_FUNCTIONS EMFDRV_Funcs =
     EMFDRV_FlattenPath,              /* pFlattenPath */
     EMFDRV_FrameRgn,                 /* pFrameRgn */
     EMFDRV_GdiComment,               /* pGdiComment */
+    NULL,                            /* pGetBitmapBits */
     NULL,                            /* pGetCharWidth */
+    NULL,                            /* pGetDIBits */
     EMFDRV_GetDeviceCaps,            /* pGetDeviceCaps */
     NULL,                            /* pGetDeviceGammaRamp */
     NULL,                            /* pGetICMProfile */
-    NULL,                            /* pGetImage */
     NULL,                            /* pGetNearestColor */
     NULL,                            /* pGetPixel */
     NULL,                            /* pGetPixelFormat */
@@ -102,7 +102,6 @@ static const DC_FUNCTIONS EMFDRV_Funcs =
     EMFDRV_Polygon,                  /* pPolygon */
     EMFDRV_Polyline,                 /* pPolyline */
     NULL,                            /* pPolylineTo */
-    NULL,                            /* pPutImage */
     NULL,                            /* pRealizeDefaultPalette */
     NULL,                            /* pRealizePalette */
     EMFDRV_Rectangle,                /* pRectangle */
@@ -119,11 +118,13 @@ static const DC_FUNCTIONS EMFDRV_Funcs =
     EMFDRV_SelectPalette,            /* pSelectPalette */
     EMFDRV_SelectPen,                /* pSelectPen */
     EMFDRV_SetArcDirection,          /* pSetArcDirection */
+    NULL,                            /* pSetBitmapBits */
     EMFDRV_SetBkColor,               /* pSetBkColor */
     EMFDRV_SetBkMode,                /* pSetBkMode */
     EMFDRV_SetDCBrushColor,          /* pSetDCBrushColor*/
     EMFDRV_SetDCPenColor,            /* pSetDCPenColor*/
     NULL,                            /* pSetDIBColorTable */
+    NULL,                            /* pSetDIBits */
     EMFDRV_SetDIBitsToDevice,        /* pSetDIBitsToDevice */
     NULL,                            /* pSetDeviceClipping */
     NULL,                            /* pSetDeviceGammaRamp */
@@ -160,7 +161,7 @@ static const DC_FUNCTIONS EMFDRV_Funcs =
 /**********************************************************************
  *	     EMFDRV_DeleteDC
  */
-static BOOL EMFDRV_DeleteDC( PHYSDEV dev )
+static BOOL CDECL EMFDRV_DeleteDC( PHYSDEV dev )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE *)dev;
     UINT index;
@@ -168,7 +169,7 @@ static BOOL EMFDRV_DeleteDC( PHYSDEV dev )
     if (physDev->emh) HeapFree( GetProcessHeap(), 0, physDev->emh );
     for(index = 0; index < physDev->handles_size; index++)
         if(physDev->handles[index])
-	    GDI_hdc_not_using_object(physDev->handles[index], dev->hdc);
+	    GDI_hdc_not_using_object(physDev->handles[index], physDev->hdc);
     HeapFree( GetProcessHeap(), 0, physDev->handles );
     HeapFree( GetProcessHeap(), 0, physDev );
     return TRUE;
@@ -222,8 +223,8 @@ void EMFDRV_UpdateBBox( PHYSDEV dev, RECTL *rect )
     RECTL *bounds = &physDev->emh->rclBounds;
     RECTL vportRect = *rect;
 
-    LPtoDP( dev->hdc, (LPPOINT)&vportRect, 2 );
-
+    LPtoDP(physDev->hdc, (LPPOINT)&vportRect, 2);
+    
     /* The coordinate systems may be mirrored
        (LPtoDP handles points, not rectangles) */
     if (vportRect.left > vportRect.right)
@@ -332,7 +333,8 @@ HDC WINAPI CreateEnhMetaFileW(
         return 0;
     }
 
-    push_dc_driver( &dc->physDev, &physDev->dev, &EMFDRV_Funcs );
+    push_dc_driver( dc, &physDev->dev, &EMFDRV_Funcs );
+    physDev->hdc = dc->hSelf;
 
     physDev->handles = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, HANDLE_LIST_INC * sizeof(physDev->handles[0]));
     physDev->handles_size = HANDLE_LIST_INC;
@@ -354,7 +356,7 @@ HDC WINAPI CreateEnhMetaFileW(
     physDev->numcolors = GetDeviceCaps(hRefDC, NUMCOLORS);
     physDev->restoring = 0;
 
-    SetVirtualResolution(physDev->dev.hdc, 0, 0, 0, 0);
+    SetVirtualResolution(dc->hSelf, 0, 0, 0, 0);
 
     physDev->emh->iType = EMR_HEADER;
     physDev->emh->nSize = size;
@@ -413,8 +415,8 @@ HDC WINAPI CreateEnhMetaFileW(
 	physDev->hFile = hFile;
     }
 
-    TRACE("returning %p\n", physDev->dev.hdc);
-    ret = physDev->dev.hdc;
+    TRACE("returning %p\n", dc->hSelf);
+    ret = dc->hSelf;
     release_dc_ptr( dc );
 
     if( !hdc )
@@ -444,7 +446,7 @@ HENHMETAFILE WINAPI CloseEnhMetaFile(HDC hdc) /* [in] metafile DC */
     }
     if (dc->refcount != 1)
     {
-        FIXME( "not deleting busy DC %p refcount %u\n", hdc, dc->refcount );
+        FIXME( "not deleting busy DC %p refcount %u\n", dc->hSelf, dc->refcount );
         release_dc_ptr( dc );
         return NULL;
     }
