@@ -74,7 +74,7 @@ static UINT EMFDRV_FindObject( PHYSDEV dev, HGDIOBJ obj )
 /******************************************************************
  *         EMFDRV_DeleteObject
  */
-BOOL EMFDRV_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
+BOOL CDECL EMFDRV_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
 {
     EMRDELETEOBJECT emr;
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*) dev;
@@ -99,12 +99,42 @@ BOOL EMFDRV_DeleteObject( PHYSDEV dev, HGDIOBJ obj )
 /***********************************************************************
  *           EMFDRV_SelectBitmap
  */
-HBITMAP EMFDRV_SelectBitmap( PHYSDEV dev, HBITMAP hbitmap )
+HBITMAP CDECL EMFDRV_SelectBitmap( PHYSDEV dev, HBITMAP hbitmap )
 {
     return 0;
 }
 
 
+/* Internal helper for EMFDRV_CreateBrushIndirect():
+ * Change the padding of a bitmap from 16 (BMP) to 32 (DIB) bits.
+ */
+static inline void EMFDRV_PadTo32(LPBYTE lpRows, int height, int width)
+{
+    int bytes16 = 2 * ((width + 15) / 16);
+    int bytes32 = 4 * ((width + 31) / 32);
+    LPBYTE lpSrc, lpDst;
+    int i;
+
+    if (!height)
+        return;
+
+    height = abs(height) - 1;
+    lpSrc = lpRows + height * bytes16;
+    lpDst = lpRows + height * bytes32;
+
+    /* Note that we work backwards so we can re-pad in place */
+    while (height >= 0)
+    {
+        for (i = bytes32; i > bytes16; i--)
+            lpDst[i - 1] = 0; /* Zero the padding bytes */
+        for (; i > 0; i--)
+            lpDst[i - 1] = lpSrc[i - 1]; /* Move image bytes into alignment */
+        lpSrc -= bytes16;
+        lpDst -= bytes32;
+        height--;
+    }
+}
+
 /***********************************************************************
  *           EMFDRV_CreateBrushIndirect
  */
@@ -141,7 +171,9 @@ DWORD EMFDRV_CreateBrushIndirect( PHYSDEV dev, HBRUSH hBrush )
 	if (info->bmiHeader.biCompression)
             bmSize = info->bmiHeader.biSizeImage;
         else
-	    bmSize = get_dib_image_size( info );
+	    bmSize = DIB_GetDIBImageBytes(info->bmiHeader.biWidth,
+					  info->bmiHeader.biHeight,
+					  info->bmiHeader.biBitCount);
 	biSize = bitmap_info_size(info, LOWORD(logbrush.lbColor));
 	size = sizeof(EMRCREATEDIBPATTERNBRUSHPT) + biSize + bmSize;
 	emr = HeapAlloc( GetProcessHeap(), 0, size );
@@ -167,36 +199,46 @@ DWORD EMFDRV_CreateBrushIndirect( PHYSDEV dev, HBRUSH hBrush )
     case BS_PATTERN:
       {
         EMRCREATEDIBPATTERNBRUSHPT *emr;
-        char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
-        BITMAPINFO *dst_info, *src_info = (BITMAPINFO *)buffer;
-        struct gdi_image_bits bits;
-        DWORD size;
+        BITMAPINFOHEADER *info;
+        BITMAP bm;
+        DWORD bmSize, biSize, size;
 
-        if (!get_bitmap_image( (HANDLE)logbrush.lbHatch, src_info, &bits )) break;
-        if (src_info->bmiHeader.biBitCount != 1)
+        GetObjectA((HANDLE)logbrush.lbHatch, sizeof(bm), &bm);
+
+        if (bm.bmBitsPixel != 1 || bm.bmPlanes != 1)
         {
             FIXME("Trying to create a color pattern brush\n");
-            if (bits.free) bits.free( &bits );
             break;
         }
 
+        /* BMP will be aligned to 32 bits, not 16 */
+        bmSize = DIB_GetDIBImageBytes(bm.bmWidth, bm.bmHeight, bm.bmBitsPixel);
+
+        biSize = sizeof(BITMAPINFOHEADER);
         /* FIXME: There is an extra DWORD written by native before the BMI.
          *        Not sure what its meant to contain.
          */
-        size = sizeof(EMRCREATEDIBPATTERNBRUSHPT) + sizeof(DWORD) +
-            sizeof(BITMAPINFOHEADER) + src_info->bmiHeader.biSizeImage;
+        size = sizeof(EMRCREATEDIBPATTERNBRUSHPT) + biSize + bmSize + sizeof(DWORD);
 
         emr = HeapAlloc( GetProcessHeap(), 0, size );
         if(!emr)
-        {
-            if (bits.free) bits.free( &bits );
-            break;
-        }
+          break;
 
-        dst_info = (BITMAPINFO *)((LPBYTE)(emr + 1) + sizeof(DWORD));
-        dst_info->bmiHeader = src_info->bmiHeader;
-        memcpy( &dst_info->bmiHeader + 1, bits.ptr, dst_info->bmiHeader.biSizeImage );
-        if (bits.free) bits.free( &bits );
+        info = (BITMAPINFOHEADER *)((LPBYTE)emr +
+                sizeof(EMRCREATEDIBPATTERNBRUSHPT) + sizeof(DWORD));
+        info->biSize = sizeof(BITMAPINFOHEADER);
+        info->biWidth = bm.bmWidth;
+        info->biHeight = bm.bmHeight;
+        info->biPlanes = bm.bmPlanes;
+        info->biBitCount = bm.bmBitsPixel;
+        info->biSizeImage = bmSize;
+        GetBitmapBits((HANDLE)logbrush.lbHatch,
+                      bm.bmHeight * BITMAP_GetWidthBytes(bm.bmWidth, bm.bmBitsPixel),
+                      (LPBYTE)info + sizeof(BITMAPINFOHEADER));
+
+        /* Change the padding to be DIB compatible if needed */
+        if (bm.bmWidth & 31)
+            EMFDRV_PadTo32((LPBYTE)info + sizeof(BITMAPINFOHEADER), bm.bmWidth, bm.bmHeight);
 
         emr->emr.iType = EMR_CREATEMONOBRUSH;
         emr->emr.nSize = size;
@@ -209,10 +251,10 @@ DWORD EMFDRV_CreateBrushIndirect( PHYSDEV dev, HBRUSH hBrush )
          * FIXME: It may be that the DIB functions themselves accept this value.
          */
         emr->iUsage = DIB_PAL_MONO;
-        emr->offBmi = (LPBYTE)dst_info - (LPBYTE)emr;
-        emr->cbBmi = sizeof( BITMAPINFOHEADER );
-        emr->offBits = emr->offBmi + emr->cbBmi;
-        emr->cbBits = dst_info->bmiHeader.biSizeImage;
+        emr->offBmi = (LPBYTE)info - (LPBYTE)emr;
+        emr->cbBmi = biSize;
+        emr->offBits = emr->offBmi + biSize;
+        emr->cbBits = bmSize;
 
         if(!EMFDRV_WriteRecord( dev, &emr->emr ))
             index = 0;
@@ -231,7 +273,7 @@ DWORD EMFDRV_CreateBrushIndirect( PHYSDEV dev, HBRUSH hBrush )
 /***********************************************************************
  *           EMFDRV_SelectBrush
  */
-HBRUSH EMFDRV_SelectBrush(PHYSDEV dev, HBRUSH hBrush )
+HBRUSH CDECL EMFDRV_SelectBrush(PHYSDEV dev, HBRUSH hBrush )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*)dev;
     EMRSELECTOBJECT emr;
@@ -257,7 +299,7 @@ HBRUSH EMFDRV_SelectBrush(PHYSDEV dev, HBRUSH hBrush )
         goto found;
 
     if (!(index = EMFDRV_CreateBrushIndirect(dev, hBrush ))) return 0;
-    GDI_hdc_using_object(hBrush, dev->hdc);
+    GDI_hdc_using_object(hBrush, physDev->hdc);
 
  found:
     emr.emr.iType = EMR_SELECTOBJECT;
@@ -310,7 +352,7 @@ static BOOL EMFDRV_CreateFontIndirect(PHYSDEV dev, HFONT hFont )
 /***********************************************************************
  *           EMFDRV_SelectFont
  */
-HFONT EMFDRV_SelectFont( PHYSDEV dev, HFONT hFont, HANDLE gdiFont )
+HFONT CDECL EMFDRV_SelectFont( PHYSDEV dev, HFONT hFont, HANDLE gdiFont )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*)dev;
     EMRSELECTOBJECT emr;
@@ -338,7 +380,7 @@ HFONT EMFDRV_SelectFont( PHYSDEV dev, HFONT hFont, HANDLE gdiFont )
         goto found;
 
     if (!(index = EMFDRV_CreateFontIndirect(dev, hFont ))) return HGDI_ERROR;
-    GDI_hdc_using_object(hFont, dev->hdc);
+    GDI_hdc_using_object(hFont, physDev->hdc);
 
  found:
     emr.emr.iType = EMR_SELECTOBJECT;
@@ -391,7 +433,7 @@ static DWORD EMFDRV_CreatePenIndirect(PHYSDEV dev, HPEN hPen)
 /******************************************************************
  *         EMFDRV_SelectPen
  */
-HPEN EMFDRV_SelectPen(PHYSDEV dev, HPEN hPen )
+HPEN CDECL EMFDRV_SelectPen(PHYSDEV dev, HPEN hPen )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*)dev;
     EMRSELECTOBJECT emr;
@@ -418,7 +460,7 @@ HPEN EMFDRV_SelectPen(PHYSDEV dev, HPEN hPen )
         goto found;
 
     if (!(index = EMFDRV_CreatePenIndirect(dev, hPen))) return 0;
-    GDI_hdc_using_object(hPen, dev->hdc);
+    GDI_hdc_using_object(hPen, physDev->hdc);
 
  found:
     emr.emr.iType = EMR_SELECTOBJECT;
@@ -459,7 +501,7 @@ static DWORD EMFDRV_CreatePalette(PHYSDEV dev, HPALETTE hPal)
 /******************************************************************
  *         EMFDRV_SelectPalette
  */
-HPALETTE EMFDRV_SelectPalette( PHYSDEV dev, HPALETTE hPal, BOOL force )
+HPALETTE CDECL EMFDRV_SelectPalette( PHYSDEV dev, HPALETTE hPal, BOOL force )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*)dev;
     EMRSELECTPALETTE emr;
@@ -477,7 +519,7 @@ HPALETTE EMFDRV_SelectPalette( PHYSDEV dev, HPALETTE hPal, BOOL force )
         goto found;
 
     if (!(index = EMFDRV_CreatePalette( dev, hPal ))) return 0;
-    GDI_hdc_using_object( hPal, dev->hdc );
+    GDI_hdc_using_object( hPal, physDev->hdc );
 
 found:
     emr.emr.iType = EMR_SELECTPALETTE;
@@ -489,18 +531,18 @@ found:
 /******************************************************************
  *         EMFDRV_SetDCBrushColor
  */
-COLORREF EMFDRV_SetDCBrushColor( PHYSDEV dev, COLORREF color )
+COLORREF CDECL EMFDRV_SetDCBrushColor( PHYSDEV dev, COLORREF color )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*)dev;
     EMRSELECTOBJECT emr;
     DWORD index;
 
-    if (GetCurrentObject( dev->hdc, OBJ_BRUSH ) != GetStockObject( DC_BRUSH )) return color;
+    if (GetCurrentObject( physDev->hdc, OBJ_BRUSH ) != GetStockObject( DC_BRUSH )) return color;
 
     if (physDev->dc_brush) DeleteObject( physDev->dc_brush );
     if (!(physDev->dc_brush = CreateSolidBrush( color ))) return CLR_INVALID;
     if (!(index = EMFDRV_CreateBrushIndirect(dev, physDev->dc_brush ))) return CLR_INVALID;
-    GDI_hdc_using_object( physDev->dc_brush, dev->hdc );
+    GDI_hdc_using_object( physDev->dc_brush, physDev->hdc );
     emr.emr.iType = EMR_SELECTOBJECT;
     emr.emr.nSize = sizeof(emr);
     emr.ihObject = index;
@@ -510,19 +552,19 @@ COLORREF EMFDRV_SetDCBrushColor( PHYSDEV dev, COLORREF color )
 /******************************************************************
  *         EMFDRV_SetDCPenColor
  */
-COLORREF EMFDRV_SetDCPenColor( PHYSDEV dev, COLORREF color )
+COLORREF CDECL EMFDRV_SetDCPenColor( PHYSDEV dev, COLORREF color )
 {
     EMFDRV_PDEVICE *physDev = (EMFDRV_PDEVICE*)dev;
     EMRSELECTOBJECT emr;
     DWORD index;
     LOGPEN logpen = { PS_SOLID, { 0, 0 }, color };
 
-    if (GetCurrentObject( dev->hdc, OBJ_PEN ) != GetStockObject( DC_PEN )) return color;
+    if (GetCurrentObject( physDev->hdc, OBJ_PEN ) != GetStockObject( DC_PEN )) return color;
 
     if (physDev->dc_pen) DeleteObject( physDev->dc_pen );
     if (!(physDev->dc_pen = CreatePenIndirect( &logpen ))) return CLR_INVALID;
     if (!(index = EMFDRV_CreatePenIndirect(dev, physDev->dc_pen))) return CLR_INVALID;
-    GDI_hdc_using_object( physDev->dc_pen, dev->hdc );
+    GDI_hdc_using_object( physDev->dc_pen, physDev->hdc );
     emr.emr.iType = EMR_SELECTOBJECT;
     emr.emr.nSize = sizeof(emr);
     emr.ihObject = index;
@@ -532,7 +574,7 @@ COLORREF EMFDRV_SetDCPenColor( PHYSDEV dev, COLORREF color )
 /******************************************************************
  *         EMFDRV_GdiComment
  */
-BOOL EMFDRV_GdiComment(PHYSDEV dev, UINT bytes, CONST BYTE *buffer)
+BOOL CDECL EMFDRV_GdiComment(PHYSDEV dev, UINT bytes, CONST BYTE *buffer)
 {
     EMRGDICOMMENT *emr;
     UINT total, rounded_size;
