@@ -30,54 +30,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(metafile);
 
-static BOOL MFDRV_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev );
-static BOOL MFDRV_DeleteDC( PHYSDEV dev );
-
-
-/**********************************************************************
- *           MFDRV_ExtEscape
- */
-static INT MFDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_data,
-                            INT cbOutput, LPVOID out_data )
-{
-    METARECORD *mr;
-    DWORD len;
-    INT ret;
-
-    if (cbOutput) return 0;  /* escapes that require output cannot work in metafiles */
-
-    len = sizeof(*mr) + sizeof(WORD) + ((cbInput + 1) & ~1);
-    mr = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, len);
-    mr->rdSize = len / 2;
-    mr->rdFunction = META_ESCAPE;
-    mr->rdParm[0] = nEscape;
-    mr->rdParm[1] = cbInput;
-    memcpy(&(mr->rdParm[2]), in_data, cbInput);
-    ret = MFDRV_WriteRecord( dev, mr, len);
-    HeapFree(GetProcessHeap(), 0, mr);
-    return ret;
-}
-
-
-/******************************************************************
- *         MFDRV_GetDeviceCaps
- *
- *A very simple implementation that returns DT_METAFILE
- */
-static INT MFDRV_GetDeviceCaps(PHYSDEV dev, INT cap)
-{
-    switch(cap)
-    {
-    case TECHNOLOGY:
-        return DT_METAFILE;
-    case TEXTCAPS:
-        return 0;
-    default:
-        TRACE(" unsupported capability %d, will return 0\n", cap );
-    }
-    return 0;
-}
-
+static BOOL CDECL MFDRV_DeleteDC( PHYSDEV dev );
 
 static const DC_FUNCTIONS MFDRV_Funcs =
 {
@@ -92,7 +45,6 @@ static const DC_FUNCTIONS MFDRV_Funcs =
     MFDRV_Chord,                     /* pChord */
     MFDRV_CloseFigure,               /* pCloseFigure */
     NULL,                            /* pCreateBitmap */
-    MFDRV_CreateCompatibleDC,        /* pCreateCompatibleDC */
     NULL,                            /* pCreateDC */
     NULL,                            /* pCreateDIBSection */
     NULL,                            /* pDeleteBitmap */
@@ -117,11 +69,12 @@ static const DC_FUNCTIONS MFDRV_Funcs =
     MFDRV_FlattenPath,               /* pFlattenPath */
     MFDRV_FrameRgn,                  /* pFrameRgn */
     NULL,                            /* pGdiComment */
+    NULL,                            /* pGetBitmapBits */
     NULL,                            /* pGetCharWidth */
+    NULL,                            /* pGetDIBits */
     MFDRV_GetDeviceCaps,             /* pGetDeviceCaps */
     NULL,                            /* pGetDeviceGammaRamp */
     NULL,                            /* pGetICMProfile */
-    NULL,                            /* pGetImage */
     NULL,                            /* pGetNearestColor */
     NULL,                            /* pGetPixel */
     NULL,                            /* pGetPixelFormat */
@@ -147,7 +100,6 @@ static const DC_FUNCTIONS MFDRV_Funcs =
     MFDRV_Polygon,                   /* pPolygon */
     MFDRV_Polyline,                  /* pPolyline */
     NULL,                            /* pPolylineTo */
-    NULL,                            /* pPutImage */
     NULL,                            /* pRealizeDefaultPalette */
     MFDRV_RealizePalette,            /* pRealizePalette */
     MFDRV_Rectangle,                 /* pRectangle */
@@ -164,11 +116,13 @@ static const DC_FUNCTIONS MFDRV_Funcs =
     MFDRV_SelectPalette,             /* pSelectPalette */
     MFDRV_SelectPen,                 /* pSelectPen */
     NULL,                            /* pSetArcDirection */
+    NULL,                            /* pSetBitmapBits */
     MFDRV_SetBkColor,                /* pSetBkColor */
     MFDRV_SetBkMode,                 /* pSetBkMode */
     MFDRV_SetDCBrushColor,           /* pSetDCBrushColor*/
     MFDRV_SetDCPenColor,             /* pSetDCPenColor*/
     NULL,                            /* pSetDIBColorTable */
+    NULL,                            /* pSetDIBits */
     MFDRV_SetDIBitsToDevice,         /* pSetDIBitsToDevice */
     NULL,                            /* pSetDeviceClipping */
     NULL,                            /* pSetDeviceGammaRamp */
@@ -226,7 +180,8 @@ static DC *MFDRV_AllocMetaFile(void)
         return NULL;
     }
 
-    push_dc_driver( &dc->physDev, &physDev->dev, &MFDRV_Funcs );
+    push_dc_driver( dc, &physDev->dev, &MFDRV_Funcs );
+    physDev->hdc = dc->hSelf;
 
     physDev->handles = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, HANDLE_LIST_INC * sizeof(physDev->handles[0]));
     physDev->handles_size = HANDLE_LIST_INC;
@@ -241,26 +196,16 @@ static DC *MFDRV_AllocMetaFile(void)
     physDev->mh->mtMaxRecord    = 0;
     physDev->mh->mtNoParameters = 0;
 
-    SetVirtualResolution( physDev->dev.hdc, 0, 0, 0, 0);
+    SetVirtualResolution(dc->hSelf, 0, 0, 0, 0);
 
     return dc;
 }
 
 
-/**********************************************************************
- *	     MFDRV_CreateCompatibleDC
- */
-static BOOL MFDRV_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
-{
-    /* not supported on metafile DCs */
-    return FALSE;
-}
-
-
 /**********************************************************************
  *	     MFDRV_DeleteDC
  */
-static BOOL MFDRV_DeleteDC( PHYSDEV dev )
+static BOOL CDECL MFDRV_DeleteDC( PHYSDEV dev )
 {
     METAFILEDRV_PDEVICE *physDev = (METAFILEDRV_PDEVICE *)dev;
     DWORD index;
@@ -268,7 +213,7 @@ static BOOL MFDRV_DeleteDC( PHYSDEV dev )
     HeapFree( GetProcessHeap(), 0, physDev->mh );
     for(index = 0; index < physDev->handles_size; index++)
         if(physDev->handles[index])
-            GDI_hdc_not_using_object(physDev->handles[index], dev->hdc);
+            GDI_hdc_not_using_object(physDev->handles[index], physDev->hdc);
     HeapFree( GetProcessHeap(), 0, physDev->handles );
     HeapFree( GetProcessHeap(), 0, physDev );
     return TRUE;
@@ -320,8 +265,8 @@ HDC WINAPI CreateMetaFileW( LPCWSTR filename )
     else  /* memory based metafile */
 	physDev->mh->mtType = METAFILE_MEMORY;
 
-    TRACE("returning %p\n", physDev->dev.hdc);
-    ret = physDev->dev.hdc;
+    TRACE("returning %p\n", dc->hSelf);
+    ret = dc->hSelf;
     release_dc_ptr( dc );
     return ret;
 }
@@ -369,7 +314,7 @@ static DC *MFDRV_CloseMetaFile( HDC hdc )
     }
     if (dc->refcount != 1)
     {
-        FIXME( "not deleting busy DC %p refcount %u\n", hdc, dc->refcount );
+        FIXME( "not deleting busy DC %p refcount %u\n", dc->hSelf, dc->refcount );
         release_dc_ptr( dc );
         return NULL;
     }
@@ -589,3 +534,48 @@ BOOL MFDRV_MetaParam8(PHYSDEV dev, short func, short param1, short param2,
     *(mr->rdParm + 7) = param1;
     return MFDRV_WriteRecord( dev, mr, mr->rdSize * 2);
 }
+
+
+/**********************************************************************
+ *           MFDRV_ExtEscape
+ */
+INT CDECL MFDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_data,
+                           INT cbOutput, LPVOID out_data )
+{
+    METARECORD *mr;
+    DWORD len;
+    INT ret;
+
+    if (cbOutput) return 0;  /* escapes that require output cannot work in metafiles */
+
+    len = sizeof(*mr) + sizeof(WORD) + ((cbInput + 1) & ~1);
+    mr = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, len);
+    mr->rdSize = len / 2;
+    mr->rdFunction = META_ESCAPE;
+    mr->rdParm[0] = nEscape;
+    mr->rdParm[1] = cbInput;
+    memcpy(&(mr->rdParm[2]), in_data, cbInput);
+    ret = MFDRV_WriteRecord( dev, mr, len);
+    HeapFree(GetProcessHeap(), 0, mr);
+    return ret;
+}
+
+
+/******************************************************************
+ *         MFDRV_GetDeviceCaps
+ *
+ *A very simple implementation that returns DT_METAFILE
+ */
+INT CDECL MFDRV_GetDeviceCaps(PHYSDEV dev, INT cap)
+{
+    switch(cap)
+    {
+    case TECHNOLOGY:
+        return DT_METAFILE;
+    case TEXTCAPS:
+        return 0;
+    default:
+        TRACE(" unsupported capability %d, will return 0\n", cap );
+    }
+    return 0;
+}
