@@ -1,6 +1,6 @@
 /*
  * GDI region objects. Shamelessly ripped out from the X11 distribution
- * Thanks for the nice license.
+ * Thanks for the nice licence.
  *
  * Copyright 1993, 1994, 1995 Alexandre Julliard
  * Modifications and additions: Copyright 1998 Huw Davies
@@ -105,6 +105,13 @@ SOFTWARE.
 
 WINE_DEFAULT_DEBUG_CHANNEL(region);
 
+typedef struct {
+    INT size;
+    INT numRects;
+    RECT *rects;
+    RECT extents;
+} WINEREGION;
+
   /* GDI logical region object */
 typedef struct
 {
@@ -917,20 +924,6 @@ HRGN WINAPI CreateEllipticRgnIndirect( const RECT *rect )
 				 rect->bottom - rect->top );
 }
 
-/*********************************************************************
- *   get_wine_region
- *
- * Return the region data without making a copy.  The caller
- * must not alter anything and must call GDI_ReleaseObj() when
- * they have finished with the data.
- */
-const WINEREGION *get_wine_region(HRGN rgn)
-{
-    RGNOBJ *obj = GDI_GetObjPtr( rgn, OBJ_REGION );
-    if(!obj) return NULL;
-    return &obj->rgn;
-}
-
 /***********************************************************************
  *           GetRegionData   (GDI32.@)
  *
