@@ -940,22 +940,14 @@ GpStatus WINGDIPAPI GdipCreateTextureIAI(GpImage *image, GDIPCONST GpImageAttrib
 GpStatus WINGDIPAPI GdipCreateTexture2I(GpImage *image, GpWrapMode wrapmode,
         INT x, INT y, INT width, INT height, GpTexture **texture)
 {
-    GpImageAttributes *imageattr;
-    GpStatus stat;
+    GpImageAttributes imageattr;
 
     TRACE("%p %d %d %d %d %d %p\n", image, wrapmode, x, y, width, height,
             texture);
 
-    stat = GdipCreateImageAttributes(&imageattr);
+    imageattr.wrap = wrapmode;
 
-    if (stat == Ok)
-    {
-        imageattr->wrap = wrapmode;
-
-        stat = GdipCreateTextureIA(image, imageattr, x, y, width, height, texture);
-    }
-
-    return stat;
+    return GdipCreateTextureIA(image, &imageattr, x, y, width, height, texture);
 }
 
 GpStatus WINGDIPAPI GdipGetBrushType(GpBrush *brush, GpBrushType *type)
@@ -1253,16 +1245,11 @@ GpStatus WINGDIPAPI GdipGetPathGradientSurroundColorsWithCount(GpPathGradient
 
 GpStatus WINGDIPAPI GdipGetPathGradientSurroundColorCount(GpPathGradient *brush, INT *count)
 {
-    static int calls;
-
     TRACE("(%p, %p)\n", brush, count);
 
     if (!brush || !count)
        return InvalidParameter;
 
-    if(!(calls++))
-        FIXME("not implemented\n");
-
     return NotImplemented;
 }
 
