@@ -943,7 +943,7 @@ static HRESULT WINAPI JScriptSafety_SetInterfaceSafetyOptions(IObjectSafety *ifa
     if(dwOptionSetMask & ~SUPPORTED_OPTIONS)
         return E_FAIL;
 
-    This->safeopt = (dwEnabledOptions & dwOptionSetMask) | (This->safeopt & ~dwOptionSetMask) | INTERFACE_USES_DISPEX;
+    This->safeopt = dwEnabledOptions & dwEnabledOptions;
     return S_OK;
 }
 
@@ -963,11 +963,6 @@ HRESULT WINAPI JScriptFactory_CreateInstance(IClassFactory *iface, IUnknown *pUn
 
     TRACE("(%p %s %p)\n", pUnkOuter, debugstr_guid(riid), ppv);
 
-    if(pUnkOuter) {
-        *ppv = NULL;
-        return CLASS_E_NOAGGREGATION;
-    }
-
     lock_module();
 
     ret = heap_alloc_zero(sizeof(*ret));
