@@ -1557,64 +1557,6 @@ static inline HTMLDocument *impl_from_IDispatchEx(IDispatchEx *iface)
     return CONTAINING_RECORD(iface, HTMLDocument, IDispatchEx_iface);
 }
 
-static HRESULT dispid_from_elem_name(HTMLDocumentNode *This, BSTR name, DISPID *dispid)
-{
-    nsIDOMNodeList *node_list;
-    nsAString name_str;
-    PRUint32 len;
-    unsigned i;
-    nsresult nsres;
-
-    if(!This->nsdoc)
-        return DISP_E_UNKNOWNNAME;
-
-    nsAString_InitDepend(&name_str, name);
-    nsres = nsIDOMHTMLDocument_GetElementsByName(This->nsdoc, &name_str, &node_list);
-    nsAString_Finish(&name_str);
-    if(NS_FAILED(nsres))
-        return E_FAIL;
-
-    nsres = nsIDOMNodeList_GetLength(node_list, &len);
-    nsIDOMNodeList_Release(node_list);
-    if(NS_FAILED(nsres))
-        return E_FAIL;
-
-    if(!len)
-        return DISP_E_UNKNOWNNAME;
-
-    for(i=0; i < This->elem_vars_cnt; i++) {
-        if(!strcmpW(name, This->elem_vars[i])) {
-            *dispid = MSHTML_DISPID_CUSTOM_MIN+i;
-            return S_OK;
-        }
-    }
-
-    if(This->elem_vars_cnt == This->elem_vars_size) {
-        WCHAR **new_vars;
-
-        if(This->elem_vars_size) {
-            new_vars = heap_realloc(This->elem_vars, This->elem_vars_size*2*sizeof(WCHAR*));
-            if(!new_vars)
-                return E_OUTOFMEMORY;
-            This->elem_vars_size *= 2;
-        }else {
-            new_vars = heap_alloc(16*sizeof(WCHAR*));
-            if(!new_vars)
-                return E_OUTOFMEMORY;
-            This->elem_vars_size = 16;
-        }
-
-        This->elem_vars = new_vars;
-    }
-
-    This->elem_vars[This->elem_vars_cnt] = heap_strdupW(name);
-    if(!This->elem_vars[This->elem_vars_cnt])
-        return E_OUTOFMEMORY;
-
-    *dispid = MSHTML_DISPID_CUSTOM_MIN+This->elem_vars_cnt++;
-    return S_OK;
-}
-
 static HRESULT WINAPI DocDispatchEx_QueryInterface(IDispatchEx *iface, REFIID riid, void **ppv)
 {
     HTMLDocument *This = impl_from_IDispatchEx(iface);
@@ -1688,13 +1630,8 @@ static HRESULT WINAPI DocDispatchEx_Invoke(IDispatchEx *iface, DISPID dispIdMemb
 static HRESULT WINAPI DocDispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DWORD grfdex, DISPID *pid)
 {
     HTMLDocument *This = impl_from_IDispatchEx(iface);
-    HRESULT hres;
 
-    hres = IDispatchEx_GetDispID(This->dispex, bstrName, grfdex, pid);
-    if(hres != DISP_E_UNKNOWNNAME)
-        return hres;
-
-    return  dispid_from_elem_name(This->doc_node, bstrName, pid);
+    return IDispatchEx_GetDispID(This->dispex, bstrName, grfdex, pid);
 }
 
 static HRESULT WINAPI DocDispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lcid, WORD wFlags, DISPPARAMS *pdp,
@@ -1770,45 +1707,6 @@ static const IDispatchExVtbl DocDispatchExVtbl = {
     DocDispatchEx_GetNameSpaceParent
 };
 
-static inline HTMLDocument *impl_from_IProvideClassInfo(IProvideClassInfo *iface)
-{
-    return CONTAINING_RECORD(iface, HTMLDocument, IProvideClassInfo_iface);
-}
-
-static HRESULT WINAPI ProvideClassInfo_QueryInterface(IProvideClassInfo *iface,
-        REFIID riid, void **ppv)
-{
-    HTMLDocument *This = impl_from_IProvideClassInfo(iface);
-    return htmldoc_query_interface(This, riid, ppv);
-}
-
-static ULONG WINAPI ProvideClassInfo_AddRef(IProvideClassInfo *iface)
-{
-    HTMLDocument *This = impl_from_IProvideClassInfo(iface);
-    return htmldoc_addref(This);
-}
-
-static ULONG WINAPI ProvideClassInfo_Release(IProvideClassInfo *iface)
-{
-    HTMLDocument *This = impl_from_IProvideClassInfo(iface);
-    return htmldoc_release(This);
-}
-
-static HRESULT WINAPI ProvideClassInfo_GetClassInfo(IProvideClassInfo* iface,
-        ITypeInfo **ppTI)
-{
-    HTMLDocument *This = impl_from_IProvideClassInfo(iface);
-    TRACE("(%p)->(%p)\n", This, ppTI);
-    return get_htmldoc_classinfo(ppTI);
-}
-
-static const IProvideClassInfoVtbl ProvideClassInfoVtbl = {
-    ProvideClassInfo_QueryInterface,
-    ProvideClassInfo_AddRef,
-    ProvideClassInfo_Release,
-    ProvideClassInfo_GetClassInfo
-};
-
 static BOOL htmldoc_qi(HTMLDocument *This, REFIID riid, void **ppv)
 {
     *ppv = NULL;
@@ -1936,9 +1834,6 @@ static BOOL htmldoc_qi(HTMLDocument *This, REFIID riid, void **ppv)
     }else if(IsEqualGUID(&IID_IObjectSafety, riid)) {
         TRACE("(%p)->(IID_IObjectSafety %p)\n", This, ppv);
         *ppv = &This->IObjectSafety_iface;
-    }else if(IsEqualGUID(&IID_IProvideClassInfo, riid)) {
-        TRACE("(%p)->(IID_IProvideClassInfo, %p)\n", This, ppv);
-        *ppv = &This->IProvideClassInfo_iface;
     }else {
         return FALSE;
     }
@@ -1955,7 +1850,6 @@ static void init_doc(HTMLDocument *doc, IUnknown *unk_impl, IDispatchEx *dispex)
     doc->IHTMLDocument2_iface.lpVtbl = &HTMLDocumentVtbl;
     doc->IDispatchEx_iface.lpVtbl = &DocDispatchExVtbl;
     doc->ISupportErrorInfo_iface.lpVtbl = &SupportErrorInfoVtbl;
-    doc->IProvideClassInfo_iface.lpVtbl = &ProvideClassInfoVtbl;
 
     doc->unk_impl = unk_impl;
     doc->dispex = dispex;
@@ -1972,7 +1866,7 @@ static void init_doc(HTMLDocument *doc, IUnknown *unk_impl, IDispatchEx *dispex)
     HTMLDocument_Hlink_Init(doc);
 
     ConnectionPointContainer_Init(&doc->cp_container, (IUnknown*)&doc->IHTMLDocument2_iface);
-    ConnectionPoint_Init(&doc->cp_dispatch, &doc->cp_container, &IID_IDispatch, &HTMLDocumentEvents_data);
+    ConnectionPoint_Init(&doc->cp_dispatch, &doc->cp_container, &IID_IDispatch, NULL);
     ConnectionPoint_Init(&doc->cp_propnotif, &doc->cp_container, &IID_IPropertyNotifySink, NULL);
     ConnectionPoint_Init(&doc->cp_htmldocevents, &doc->cp_container, &DIID_HTMLDocumentEvents, &HTMLDocumentEvents_data);
     ConnectionPoint_Init(&doc->cp_htmldocevents2, &doc->cp_container, &DIID_HTMLDocumentEvents2, NULL);
@@ -2011,11 +1905,6 @@ static HRESULT HTMLDocumentNode_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
 static void HTMLDocumentNode_destructor(HTMLDOMNode *iface)
 {
     HTMLDocumentNode *This = impl_from_HTMLDOMNode(iface);
-    unsigned i;
-
-    for(i=0; i < This->elem_vars_cnt; i++)
-        heap_free(This->elem_vars[i]);
-    heap_free(This->elem_vars);
 
     if(This->body_event_target)
         release_event_target(This->body_event_target);
@@ -2023,6 +1912,8 @@ static void HTMLDocumentNode_destructor(HTMLDOMNode *iface)
         release_nsevents(This);
     if(This->catmgr)
         ICatInformation_Release(This->catmgr);
+    if(This->secmgr)
+        IInternetSecurityManager_Release(This->secmgr);
 
     detach_selection(This);
     detach_ranges(This);
@@ -2067,62 +1958,6 @@ static HRESULT HTMLDocumentFragment_clone(HTMLDOMNode *iface, nsIDOMNode *nsnode
     return S_OK;
 }
 
-static inline HTMLDocumentNode *impl_from_DispatchEx(DispatchEx *iface)
-{
-    return CONTAINING_RECORD(iface, HTMLDocumentNode, node.dispex);
-}
-
-static HRESULT HTMLDocumentNode_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
-        VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
-{
-    HTMLDocumentNode *This = impl_from_DispatchEx(dispex);
-    nsIDOMNodeList *node_list;
-    nsAString name_str;
-    nsIDOMNode *nsnode;
-    HTMLDOMNode *node;
-    unsigned i;
-    nsresult nsres;
-    HRESULT hres;
-
-    if(flags != DISPATCH_PROPERTYGET) {
-        FIXME("unsupported flags %x\n", flags);
-        return E_NOTIMPL;
-    }
-
-    i = id - MSHTML_DISPID_CUSTOM_MIN;
-
-    if(!This->nsdoc || i >= This->elem_vars_cnt)
-        return DISP_E_UNKNOWNNAME;
-
-    nsAString_InitDepend(&name_str, This->elem_vars[i]);
-    nsres = nsIDOMHTMLDocument_GetElementsByName(This->nsdoc, &name_str, &node_list);
-    nsAString_Finish(&name_str);
-    if(NS_FAILED(nsres))
-        return E_FAIL;
-
-    nsres = nsIDOMNodeList_Item(node_list, 0, &nsnode);
-    nsIDOMNodeList_Release(node_list);
-    if(NS_FAILED(nsres) || !nsnode)
-        return DISP_E_UNKNOWNNAME;
-
-    hres = get_node(This, nsnode, TRUE, &node);
-    if(FAILED(hres))
-        return hres;
-
-    IHTMLDOMNode_AddRef(&node->IHTMLDOMNode_iface);
-    V_VT(res) = VT_DISPATCH;
-    V_DISPATCH(res) = (IDispatch*)&node->IHTMLDOMNode_iface;
-    return S_OK;
-}
-
-
-static const dispex_static_data_vtbl_t HTMLDocumentNode_dispex_vtbl = {
-    NULL,
-    NULL,
-    HTMLDocumentNode_invoke,
-    NULL
-};
-
 static const NodeImplVtbl HTMLDocumentFragmentImplVtbl = {
     HTMLDocumentNode_QI,
     HTMLDocumentNode_destructor,
@@ -2140,7 +1975,7 @@ static const tid_t HTMLDocumentNode_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLDocumentNode_dispex = {
-    &HTMLDocumentNode_dispex_vtbl,
+    NULL,
     DispHTMLDocument_tid,
     NULL,
     HTMLDocumentNode_iface_tids
@@ -2165,6 +2000,8 @@ static HTMLDocumentNode *alloc_doc_node(HTMLDocumentObj *doc_obj, HTMLWindow *wi
             &doc->node.dispex.IDispatchEx_iface);
     HTMLDocumentNode_SecMgr_Init(doc);
 
+    init_nsevents(doc);
+
     list_init(&doc->bindings);
     list_init(&doc->selection_list);
     list_init(&doc->range_list);
@@ -2176,6 +2013,7 @@ static HTMLDocumentNode *alloc_doc_node(HTMLDocumentObj *doc_obj, HTMLWindow *wi
 HRESULT create_doc_from_nsdoc(nsIDOMHTMLDocument *nsdoc, HTMLDocumentObj *doc_obj, HTMLWindow *window, HTMLDocumentNode **ret)
 {
     HTMLDocumentNode *doc;
+    HRESULT hres;
 
     doc = alloc_doc_node(doc_obj, window);
     if(!doc)
@@ -2187,12 +2025,17 @@ HRESULT create_doc_from_nsdoc(nsIDOMHTMLDocument *nsdoc, HTMLDocumentObj *doc_ob
     nsIDOMHTMLDocument_AddRef(nsdoc);
     doc->nsdoc = nsdoc;
     init_document_mutation(doc);
-    doc_init_events(doc);
 
     HTMLDOMNode_Init(doc, &doc->node, (nsIDOMNode*)nsdoc);
     doc->node.vtbl = &HTMLDocumentNodeImplVtbl;
     doc->node.cp_container = &doc->basedoc.cp_container;
 
+    hres = CoInternetCreateSecurityManager(NULL, &doc->secmgr, 0);
+    if(FAILED(hres)) {
+        htmldoc_release(&doc->basedoc);
+        return hres;
+    }
+
     *ret = doc;
     return S_OK;
 }
