@@ -141,7 +141,6 @@ typedef struct {
 #define EVENT_FORWARDBODY        0x0004
 #define EVENT_NODEHANDLER        0x0008
 #define EVENT_CANCELABLE         0x0010
-#define EVENT_HASDEFAULTHANDLERS 0x0020
 
 static const event_info_t event_info[] = {
     {beforeunloadW,      onbeforeunloadW,      EVENTT_NONE,   DISPID_EVMETH_ONBEFOREUNLOAD,
@@ -151,7 +150,7 @@ static const event_info_t event_info[] = {
     {changeW,            onchangeW,            EVENTT_HTML,   DISPID_EVMETH_ONCHANGE,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {clickW,             onclickW,             EVENTT_MOUSE,  DISPID_EVMETH_ONCLICK,
-        EVENT_DEFAULTLISTENER|EVENT_BUBBLE|EVENT_CANCELABLE|EVENT_HASDEFAULTHANDLERS},
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE|EVENT_CANCELABLE},
     {contextmenuW,       oncontextmenuW,       EVENTT_MOUSE,  DISPID_EVMETH_ONCONTEXTMENU,
         EVENT_BUBBLE|EVENT_CANCELABLE},
     {dblclickW,          ondblclickW,          EVENTT_MOUSE,  DISPID_EVMETH_ONDBLCLICK,
@@ -290,8 +289,6 @@ static ULONG WINAPI HTMLEventObj_Release(IHTMLEventObj *iface)
     TRACE("(%p) ref=%d\n", This, ref);
 
     if(!ref) {
-        if(This->target)
-            IHTMLDOMNode_Release(&This->target->IHTMLDOMNode_iface);
         if(This->nsevent)
             nsIDOMEvent_Release(This->nsevent);
         release_dispex(&This->dispex);
@@ -435,17 +432,8 @@ static HRESULT WINAPI HTMLEventObj_get_shiftKey(IHTMLEventObj *iface, VARIANT_BO
 static HRESULT WINAPI HTMLEventObj_put_returnValue(IHTMLEventObj *iface, VARIANT v)
 {
     HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
-
-    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
-
-    if(V_VT(&v) != VT_BOOL) {
-        FIXME("unsupported vt %d\n", V_VT(&v));
-        return DISP_E_BADVARTYPE;
-    }
-
-    if(!V_BOOL(&v))
-        This->prevent_default = TRUE;
-    return S_OK;
+    FIXME("(%p)->()\n", This);
+    return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLEventObj_get_returnValue(IHTMLEventObj *iface, VARIANT *p)
@@ -777,12 +765,19 @@ static HTMLEventObj *create_event(HTMLDOMNode *target, eventid_t eid, nsIDOMEven
     if(nsevent) {
         nsIDOMEvent_AddRef(nsevent);
     }else if(event_types[event_info[eid].type]) {
-        nsAString type_str;
+        nsIDOMDocumentEvent *doc_event;
         nsresult nsres;
 
-        nsAString_InitDepend(&type_str, event_types[event_info[eid].type]);
-        nsres = nsIDOMHTMLDocument_CreateEvent(target->doc->nsdoc, &type_str, &ret->nsevent);
-        nsAString_Finish(&type_str);
+        nsres = nsIDOMHTMLDocument_QueryInterface(target->doc->nsdoc, &IID_nsIDOMDocumentEvent,
+                 (void**)&doc_event);
+        if(NS_SUCCEEDED(nsres)) {
+            nsAString type_str;
+
+            nsAString_InitDepend(&type_str, event_types[event_info[eid].type]);
+            nsres = nsIDOMDocumentEvent_CreateEvent(doc_event, &type_str, &ret->nsevent);
+            nsAString_Finish(&type_str);
+            nsIDOMDocumentEvent_Release(doc_event);
+        }
         if(NS_FAILED(nsres)) {
             ERR("Could not create event: %08x\n", nsres);
             IHTMLEventObj_Release(&ret->IHTMLEventObj_iface);
@@ -974,10 +969,8 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     HTMLEventObj *event_obj = NULL;
     IHTMLEventObj *prev_event;
     nsIDOMNode *parent, *nsnode;
-    BOOL prevent_default = FALSE;
     HTMLDOMNode *node;
     PRUint16 node_type;
-    nsresult nsres;
     HRESULT hres;
 
     TRACE("(%p) %s\n", doc, debugstr_w(event_info[eid].name));
@@ -1049,42 +1042,14 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     if(nsnode)
         nsIDOMNode_Release(nsnode);
 
-    if(event_obj && event_obj->prevent_default)
-        prevent_default = TRUE;
     doc->basedoc.window->event = prev_event;
-    if(event_obj)
+
+    if(event_obj) {
+        if(event_obj->prevent_default && nsevent) {
+            TRACE("calling PreventDefault\n");
+            nsIDOMEvent_PreventDefault(nsevent);
+        }
         IHTMLEventObj_Release(&event_obj->IHTMLEventObj_iface);
-
-    if(!prevent_default && (event_info[eid].flags & EVENT_HASDEFAULTHANDLERS)) {
-        nsIDOMNode_AddRef(target);
-        nsnode = target;
-
-        do {
-            hres = get_node(doc, nsnode, TRUE, &node);
-            if(FAILED(hres))
-                break;
-
-            if(node && node->vtbl->handle_event) {
-                hres = node->vtbl->handle_event(node, eid, &prevent_default);
-                if(FAILED(hres) || prevent_default)
-                    break;
-            }
-
-            nsres = nsIDOMNode_GetParentNode(nsnode, &parent);
-            if(NS_FAILED(nsres))
-                break;
-
-            nsIDOMNode_Release(nsnode);
-            nsnode = parent;
-        } while(nsnode);
-
-        if(nsnode)
-            nsIDOMNode_Release(nsnode);
-    }
-
-    if(prevent_default && nsevent) {
-        TRACE("calling PreventDefault\n");
-        nsIDOMEvent_PreventDefault(nsevent);
     }
 }
 
@@ -1112,14 +1077,14 @@ HRESULT dispatch_event(HTMLDOMNode *node, const WCHAR *event_name, VARIANT *even
     return S_OK;
 }
 
-HRESULT call_fire_event(HTMLDOMNode *node, eventid_t eid)
+HRESULT call_event(HTMLDOMNode *node, eventid_t eid)
 {
     HRESULT hres;
 
-    if(node->vtbl->fire_event) {
+    if(node->vtbl->call_event) {
         BOOL handled = FALSE;
 
-        hres = node->vtbl->fire_event(node, eid, &handled);
+        hres = node->vtbl->call_event(node, eid, &handled);
         if(handled)
             return hres;
     }
@@ -1176,6 +1141,12 @@ static HRESULT ensure_nsevent_handler(HTMLDocumentNode *doc, event_target_t *eve
     if(!(event_info[eid].flags & EVENT_DEFAULTLISTENER))
         return S_OK;
 
+    if(!doc->event_vector) {
+        doc->event_vector = heap_alloc_zero(EVENTID_LAST*sizeof(BOOL));
+        if(!doc->event_vector)
+            return E_OUTOFMEMORY;
+    }
+
     if(!doc->event_vector[eid]) {
         doc->event_vector[eid] = TRUE;
         add_nsevent_listener(doc, NULL, event_info[eid].name);
@@ -1363,28 +1334,6 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMElement *nselem)
     nsAString_Finish(&attr_name_str);
 }
 
-HRESULT doc_init_events(HTMLDocumentNode *doc)
-{
-    unsigned i;
-    HRESULT hres;
-
-    doc->event_vector = heap_alloc_zero(EVENTID_LAST*sizeof(BOOL));
-    if(!doc->event_vector)
-        return E_OUTOFMEMORY;
-
-    init_nsevents(doc);
-
-    for(i=0; i < EVENTID_LAST; i++) {
-        if(event_info[i].flags & EVENT_HASDEFAULTHANDLERS) {
-            hres = ensure_nsevent_handler(doc, NULL, NULL, i);
-            if(FAILED(hres))
-                return hres;
-        }
-    }
-
-    return S_OK;
-}
-
 void release_event_target(event_target_t *event_target)
 {
     int i, j;
@@ -1394,8 +1343,7 @@ void release_event_target(event_target_t *event_target)
             if(event_target->event_table[i]->handler_prop)
                 IDispatch_Release(event_target->event_table[i]->handler_prop);
             for(j=0; j < event_target->event_table[i]->handler_cnt; j++)
-                if(event_target->event_table[i]->handlers[j])
-                    IDispatch_Release(event_target->event_table[i]->handlers[j]);
+                IDispatch_Release(event_target->event_table[i]->handlers[j]);
         }
     }
 
