@@ -27,14 +27,10 @@
 #include "mshtmdid.h"
 #include "shlguid.h"
 
-#define NO_SHLWAPI_REG
-#include "shlwapi.h"
-
 #include "wine/debug.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
-#include "binding.h"
 #include "resource.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
@@ -83,6 +79,37 @@ static void window_set_docnode(HTMLWindow *window, HTMLDocumentNode *doc_node)
     }
 }
 
+nsIDOMWindow *get_nsdoc_window(nsIDOMDocument *nsdoc)
+{
+    nsIDOMDocumentView *nsdocview;
+    nsIDOMAbstractView *nsview;
+    nsIDOMWindow *nswindow;
+    nsresult nsres;
+
+    nsres = nsIDOMDocument_QueryInterface(nsdoc, &IID_nsIDOMDocumentView, (void**)&nsdocview);
+    nsIDOMDocument_Release(nsdoc);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMDocumentView iface: %08x\n", nsres);
+        return NULL;
+    }
+
+    nsres = nsIDOMDocumentView_GetDefaultView(nsdocview, &nsview);
+    nsIDOMDocumentView_Release(nsdocview);
+    if(NS_FAILED(nsres)) {
+        ERR("GetDefaultView failed: %08x\n", nsres);
+        return NULL;
+    }
+
+    nsres = nsIDOMAbstractView_QueryInterface(nsview, &IID_nsIDOMWindow, (void**)&nswindow);
+    nsIDOMAbstractView_Release(nsview);
+    if(NS_FAILED(nsres)) {
+        ERR("Coult not get nsIDOMWindow iface: %08x\n", nsres);
+        return NULL;
+    }
+
+    return nswindow;
+}
+
 static void release_children(HTMLWindow *This)
 {
     HTMLWindow *child;
@@ -209,9 +236,6 @@ static ULONG WINAPI HTMLWindow2_Release(IHTMLWindow2 *iface)
         window_set_docnode(This, NULL);
         release_children(This);
 
-        if(This->secmgr)
-            IInternetSecurityManager_Release(This->secmgr);
-
         if(This->frame_element)
             This->frame_element->content_window = NULL;
 
@@ -460,25 +484,15 @@ static HRESULT WINAPI HTMLWindow2_get_defaultStatus(IHTMLWindow2 *iface, BSTR *p
 static HRESULT WINAPI HTMLWindow2_put_status(IHTMLWindow2 *iface, BSTR v)
 {
     HTMLWindow *This = impl_from_IHTMLWindow2(iface);
-
-    WARN("(%p)->(%s)\n", This, debugstr_w(v));
-
-    /*
-     * FIXME: Since IE7, setting status is blocked, but still possible in certain circumstances.
-     * Ignoring the call should be enough for us.
-     */
-    return S_OK;
+    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
+    return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLWindow2_get_status(IHTMLWindow2 *iface, BSTR *p)
 {
     HTMLWindow *This = impl_from_IHTMLWindow2(iface);
-
-    TRACE("(%p)->(%p)\n", This, p);
-
-    /* See put_status */
-    *p = NULL;
-    return S_OK;
+    FIXME("(%p)->(%p)\n", This, p);
+    return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLWindow2_setTimeout(IHTMLWindow2 *iface, BSTR expression,
@@ -1648,79 +1662,6 @@ static ULONG WINAPI HTMLPrivateWindow_Release(IHTMLPrivateWindow *iface)
     return IHTMLWindow2_Release(&This->IHTMLWindow2_iface);
 }
 
-typedef struct {
-    task_t header;
-    HTMLWindow *window;
-    IUri *uri;
-} navigate_javascript_task_t;
-
-static void navigate_javascript_proc(task_t *_task)
-{
-    navigate_javascript_task_t *task = (navigate_javascript_task_t*)_task;
-    HTMLWindow *window = task->window;
-    VARIANT v;
-    BSTR code;
-    HRESULT hres;
-
-    static const WCHAR jscriptW[] = {'j','s','c','r','i','p','t',0};
-
-    task->window->readystate = READYSTATE_COMPLETE;
-
-    hres = IUri_GetPath(task->uri, &code);
-    if(FAILED(hres))
-        return;
-
-    set_download_state(window->doc_obj, 1);
-
-    V_VT(&v) = VT_EMPTY;
-    hres = exec_script(window, code, jscriptW, &v);
-    SysFreeString(code);
-    if(SUCCEEDED(hres) && V_VT(&v) != VT_EMPTY) {
-        FIXME("javascirpt URL returned %s\n", debugstr_variant(&v));
-        VariantClear(&v);
-    }
-
-    if(window->doc_obj->view_sink)
-        IAdviseSink_OnViewChange(window->doc_obj->view_sink, DVASPECT_CONTENT, -1);
-
-    set_download_state(window->doc_obj, 0);
-}
-
-static void navigate_javascript_task_destr(task_t *_task)
-{
-    navigate_javascript_task_t *task = (navigate_javascript_task_t*)_task;
-
-    IUri_Release(task->uri);
-    heap_free(task);
-}
-
-typedef struct {
-    task_t header;
-    HTMLWindow *window;
-    nsChannelBSC *bscallback;
-    IMoniker *mon;
-} navigate_task_t;
-
-static void navigate_proc(task_t *_task)
-{
-    navigate_task_t *task = (navigate_task_t*)_task;
-    HRESULT hres;
-
-    hres = set_moniker(&task->window->doc_obj->basedoc, task->mon, NULL, task->bscallback, TRUE);
-    if(SUCCEEDED(hres))
-        hres = start_binding(task->window, NULL, (BSCallback*)task->bscallback, NULL);
-
-}
-
-static void navigate_task_destr(task_t *_task)
-{
-    navigate_task_t *task = (navigate_task_t*)_task;
-
-    IUnknown_Release((IUnknown*)task->bscallback);
-    IMoniker_Release(task->mon);
-    heap_free(task);
-}
-
 static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface, BSTR url, BSTR arg2, BSTR arg3,
         BSTR arg4, VARIANT *post_data_var, VARIANT *headers_var, ULONG flags)
 {
@@ -1730,9 +1671,7 @@ static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface,
     WCHAR *headers = NULL;
     nsChannelBSC *bsc;
     IMoniker *mon;
-    DWORD scheme;
     BSTR new_url;
-    IUri *uri;
     HRESULT hres;
 
     TRACE("(%p)->(%s %s %s %s %s %s %x)\n", This, debugstr_w(url), debugstr_w(arg2), debugstr_w(arg3), debugstr_w(arg4),
@@ -1767,19 +1706,15 @@ static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface,
         }
     }
 
-    hres = CreateUri(new_url, 0, 0, &uri);
+    /* FIXME: Why not set_ready_state? */
+    This->readystate = READYSTATE_UNINITIALIZED;
+
+    hres = CreateURLMoniker(NULL, new_url, &mon);
     if(new_url != url)
         SysFreeString(new_url);
     if(FAILED(hres))
         return hres;
 
-    hres = CreateURLMonikerEx2(NULL, uri, &mon, URL_MK_UNIFORM);
-    if(FAILED(hres))
-        return hres;
-
-    /* FIXME: Why not set_ready_state? */
-    This->readystate = READYSTATE_UNINITIALIZED;
-
     if(post_data_var) {
         if(V_VT(post_data_var) == (VT_ARRAY|VT_UI1)) {
             SafeArrayAccessData(V_ARRAY(post_data_var), (void**)&post_data);
@@ -1802,50 +1737,13 @@ static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface,
         return hres;
     }
 
-    prepare_for_binding(&This->doc_obj->basedoc, mon, NULL, TRUE);
+    hres = set_moniker(&This->doc_obj->basedoc, mon, NULL, bsc, TRUE);
+    if(SUCCEEDED(hres))
+        hres = async_start_doc_binding(This, bsc);
 
-    hres = IUri_GetScheme(uri, &scheme);
-
-    if(scheme != URL_SCHEME_JAVASCRIPT) {
-        navigate_task_t *task;
-
-        IUri_Release(uri);
-
-        task = heap_alloc(sizeof(*task));
-        if(!task) {
-            IUnknown_Release((IUnknown*)bsc);
-            IMoniker_Release(mon);
-            return E_OUTOFMEMORY;
-        }
-
-        task->window = This;
-        task->bscallback = bsc;
-        task->mon = mon;
-        push_task(&task->header, navigate_proc, navigate_task_destr, This->task_magic);
-
-        /* Silently and repeated when real loading starts? */
-        This->readystate = READYSTATE_LOADING;
-    }else {
-        navigate_javascript_task_t *task;
-
-        IUnknown_Release((IUnknown*)bsc);
-        IMoniker_Release(mon);
-
-        task = heap_alloc(sizeof(*task));
-        if(!task) {
-            IUri_Release(uri);
-            return E_OUTOFMEMORY;
-        }
-
-        task->window = This;
-        task->uri = uri;
-        push_task(&task->header, navigate_javascript_proc, navigate_javascript_task_destr, This->task_magic);
-
-        /* Why silently? */
-        This->readystate = READYSTATE_COMPLETE;
-    }
-
-    return S_OK;
+    IUnknown_Release((IUnknown*)bsc);
+    IMoniker_Release(mon);
+    return hres;
 }
 
 static HRESULT WINAPI HTMLPrivateWindow_GetPendingUrl(IHTMLPrivateWindow *iface, BSTR *url)
@@ -2250,22 +2148,22 @@ static HRESULT HTMLWindow_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD
 
     switch(prop->type) {
     case GLOBAL_SCRIPTVAR: {
-        IDispatchEx *iface;
+        IDispatchEx *dispex;
         IDispatch *disp;
 
         disp = get_script_disp(prop->script_host);
         if(!disp)
             return E_UNEXPECTED;
 
-        hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&iface);
+        hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
         if(SUCCEEDED(hres)) {
             TRACE("%s >>>\n", debugstr_w(prop->name));
-            hres = IDispatchEx_InvokeEx(iface, prop->id, lcid, flags, params, res, ei, caller);
+            hres = IDispatchEx_InvokeEx(dispex, prop->id, lcid, flags, params, res, ei, caller);
             if(hres == S_OK)
                 TRACE("%s <<<\n", debugstr_w(prop->name));
             else
                 WARN("%s <<< %08x\n", debugstr_w(prop->name), hres);
-            IDispatchEx_Release(iface);
+            IDispatchEx_Release(dispex);
         }else {
             FIXME("No IDispatchEx\n");
         }
@@ -2299,8 +2197,7 @@ static HRESULT HTMLWindow_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD
 static const dispex_static_data_vtbl_t HTMLWindow_dispex_vtbl = {
     NULL,
     NULL,
-    HTMLWindow_invoke,
-    NULL
+    HTMLWindow_invoke
 };
 
 static const tid_t HTMLWindow_iface_tids[] = {
@@ -2320,7 +2217,6 @@ static dispex_static_data_t HTMLWindow_dispex = {
 HRESULT HTMLWindow_Create(HTMLDocumentObj *doc_obj, nsIDOMWindow *nswindow, HTMLWindow *parent, HTMLWindow **ret)
 {
     HTMLWindow *window;
-    HRESULT hres;
 
     window = heap_alloc_zero(sizeof(HTMLWindow));
     if(!window)
@@ -2355,12 +2251,6 @@ HRESULT HTMLWindow_Create(HTMLDocumentObj *doc_obj, nsIDOMWindow *nswindow, HTML
     window->readystate = READYSTATE_UNINITIALIZED;
     list_init(&window->script_hosts);
 
-    hres = CoInternetCreateSecurityManager(NULL, &window->secmgr, 0);
-    if(FAILED(hres)) {
-        IHTMLWindow2_Release(&window->IHTMLWindow2_iface);
-        return hres;
-    }
-
     window->task_magic = get_task_target_magic();
     update_window_doc(window);
 
