@@ -19,7 +19,6 @@
 #include "config.h"
 
 #include <stdarg.h>
-#include <assert.h>
 
 #define COBJMACROS
 
@@ -31,10 +30,10 @@
 #include "shlobj.h"
 
 #include "wine/debug.h"
+#include "wine/unicode.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
-#include "binding.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 WINE_DECLARE_DEBUG_CHANNEL(gecko);
@@ -47,6 +46,8 @@ WINE_DECLARE_DEBUG_CHANNEL(gecko);
 #define NS_EDITORCONTROLLER_CONTRACTID "@mozilla.org/editor/editorcontroller;1"
 #define NS_PREFERENCES_CONTRACTID "@mozilla.org/preferences;1"
 
+#define APPSTARTUP_TOPIC "app-startup"
+
 #define PR_UINT32_MAX 0xffffffff
 
 #define NS_STRING_CONTAINER_INIT_DEPEND  0x0002
@@ -65,7 +66,7 @@ static nsresult (CDECL *NS_NewLocalFile)(const nsAString*,PRBool,nsIFile**);
 static PRUint32 (CDECL *NS_StringGetData)(const nsAString*,const PRUnichar **,PRBool*);
 static PRUint32 (CDECL *NS_CStringGetData)(const nsACString*,const char**,PRBool*);
 
-static HINSTANCE xul_handle = NULL;
+static HINSTANCE hXPCOM = NULL;
 
 static nsIServiceManager *pServMgr = NULL;
 static nsIComponentManager *pCompMgr = NULL;
@@ -308,26 +309,26 @@ static void set_environment(LPCWSTR gre_path)
     }
 }
 
-static BOOL load_xul(const PRUnichar *gre_path)
+static BOOL load_xpcom(const PRUnichar *gre_path)
 {
-    static const WCHAR xul_dllW[] = {'\\','x','u','l','.','d','l','l',0};
+    static const WCHAR strXPCOM[] = {'\\','x','p','c','o','m','.','d','l','l',0};
     WCHAR file_name[MAX_PATH];
 
     strcpyW(file_name, gre_path);
-    strcatW(file_name, xul_dllW);
+    strcatW(file_name, strXPCOM);
 
     TRACE("(%s)\n", debugstr_w(file_name));
 
     set_environment(gre_path);
 
-    xul_handle = LoadLibraryExW(file_name, 0, LOAD_WITH_ALTERED_SEARCH_PATH);
-    if(!xul_handle) {
-        WARN("Could not load XUL: %d\n", GetLastError());
+    hXPCOM = LoadLibraryExW(file_name, 0, LOAD_WITH_ALTERED_SEARCH_PATH);
+    if(!hXPCOM) {
+        WARN("Could not load XPCOM: %d\n", GetLastError());
         return FALSE;
     }
 
 #define NS_DLSYM(func) \
-    func = (void *)GetProcAddress(xul_handle, #func "_P"); \
+    func = (void *)GetProcAddress(hXPCOM, #func); \
     if(!func) \
         ERR("Could not GetProcAddress(" #func ") failed\n")
 
@@ -408,7 +409,7 @@ static BOOL load_wine_gecko_v(PRUnichar *gre_path, HKEY mshtml_key,
     if(!check_version(gre_path, version_string))
         return FALSE;
 
-    return load_xul(gre_path);
+    return load_xpcom(gre_path);
 }
 
 static BOOL load_wine_gecko(PRUnichar *gre_path)
@@ -486,6 +487,55 @@ static void set_lang(nsIPrefBranch *pref)
     set_string_pref(pref, "intl.accept_languages", langs);
 }
 
+static void set_proxy(nsIPrefBranch *pref)
+{
+    char proxy[512];
+    char * proxy_port;
+    int proxy_port_num;
+    DWORD enabled = 0, res, size, type;
+    HKEY hkey;
+
+    static const WCHAR proxy_keyW[] =
+        {'S','o','f','t','w','a','r','e',
+         '\\','M','i','c','r','o','s','o','f','t',
+         '\\','W','i','n','d','o','w','s',
+         '\\','C','u','r','r','e','n','t','V','e','r','s','i','o','n',
+         '\\','I','n','t','e','r','n','e','t',' ','S','e','t','t','i','n','g','s',0};
+
+    res = RegOpenKeyW(HKEY_CURRENT_USER, proxy_keyW, &hkey);
+    if(res != ERROR_SUCCESS)
+        return;
+
+    size = sizeof(enabled);
+    res = RegQueryValueExA(hkey, "ProxyEnable", 0, &type, (LPBYTE)&enabled, &size);
+    if(res != ERROR_SUCCESS || type != REG_DWORD || enabled == 0)
+    {
+        RegCloseKey(hkey);
+        return;
+    }
+
+    size = sizeof(proxy);
+    res = RegQueryValueExA(hkey, "ProxyServer", 0, &type, (LPBYTE)proxy, &size);
+    RegCloseKey(hkey);
+    if(res != ERROR_SUCCESS || type != REG_SZ)
+        return;
+
+    proxy_port = strchr(proxy, ':');
+    if (!proxy_port)
+        return;
+
+    *proxy_port = 0;
+    proxy_port_num = atoi(proxy_port + 1);
+    TRACE("Setting proxy to %s, port %d\n", debugstr_a(proxy), proxy_port_num);
+
+    set_string_pref(pref, "network.proxy.http", proxy);
+    set_string_pref(pref, "network.proxy.ssl", proxy);
+
+    set_int_pref(pref, "network.proxy.type", 1);
+    set_int_pref(pref, "network.proxy.http_port", proxy_port_num);
+    set_int_pref(pref, "network.proxy.ssl_port", proxy_port_num);
+}
+
 static void set_preferences(void)
 {
     nsIPrefBranch *pref;
@@ -499,6 +549,7 @@ static void set_preferences(void)
     }
 
     set_lang(pref);
+    set_proxy(pref);
     set_bool_pref(pref, "security.warn_entering_secure", FALSE);
     set_bool_pref(pref, "security.warn_submit_insecure", FALSE);
     set_int_pref(pref, "layout.spellcheckDefault", 0);
@@ -508,25 +559,26 @@ static void set_preferences(void)
 
 static BOOL init_xpcom(const PRUnichar *gre_path)
 {
+    nsresult nsres;
+    nsIObserver *pStartNotif;
     nsIComponentRegistrar *registrar = NULL;
     nsAString path;
     nsIFile *gre_dir;
     WCHAR *ptr;
-    nsresult nsres;
 
     nsAString_InitDepend(&path, gre_path);
     nsres = NS_NewLocalFile(&path, FALSE, &gre_dir);
     nsAString_Finish(&path);
     if(NS_FAILED(nsres)) {
         ERR("NS_NewLocalFile failed: %08x\n", nsres);
-        FreeLibrary(xul_handle);
+        FreeLibrary(hXPCOM);
         return FALSE;
     }
 
     nsres = NS_InitXPCOM2(&pServMgr, gre_dir, &nsDirectoryServiceProvider);
     if(NS_FAILED(nsres)) {
         ERR("NS_InitXPCOM2 failed: %08x\n", nsres);
-        FreeLibrary(xul_handle);
+        FreeLibrary(hXPCOM);
         return FALSE;
     }
 
@@ -548,6 +600,19 @@ static BOOL init_xpcom(const PRUnichar *gre_path)
         ERR("NS_GetComponentRegistrar failed: %08x\n", nsres);
 
     init_mutation(pCompMgr);
+
+    nsres = nsIComponentManager_CreateInstanceByContractID(pCompMgr, NS_APPSTARTUPNOTIFIER_CONTRACTID,
+            NULL, &IID_nsIObserver, (void**)&pStartNotif);
+    if(NS_SUCCEEDED(nsres)) {
+        nsres = nsIObserver_Observe(pStartNotif, NULL, APPSTARTUP_TOPIC, NULL);
+        if(NS_FAILED(nsres))
+            ERR("Observe failed: %08x\n", nsres);
+
+        nsIObserver_Release(pStartNotif);
+    }else {
+        ERR("could not get appstartup-notifier: %08x\n", nsres);
+    }
+
     set_preferences();
 
     nsres = nsIComponentManager_CreateInstanceByContractID(pCompMgr, NS_MEMORY_CONTRACTID,
@@ -744,12 +809,10 @@ static HRESULT nsnode_to_nsstring_rec(nsIContentSerializer *serializer, nsIDOMNo
         return E_FAIL;
     }
 
-    if(type != DOCUMENT_NODE) {
-        nsres = nsIDOMNode_QueryInterface(nsnode, &IID_nsIContent, (void**)&nscontent);
-        if(NS_FAILED(nsres)) {
-            ERR("Could not get nsIContent interface: %08x\n", nsres);
-            return E_FAIL;
-        }
+    nsres = nsIDOMNode_QueryInterface(nsnode, &IID_nsIContent, (void**)&nscontent);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDontent interface: %08x\n", nsres);
+        return E_FAIL;
     }
 
     switch(type) {
@@ -801,8 +864,7 @@ static HRESULT nsnode_to_nsstring_rec(nsIContentSerializer *serializer, nsIDOMNo
     if(type == ELEMENT_NODE)
         nsIContentSerializer_AppendElementEnd(serializer, nscontent, str);
 
-    if(type != DOCUMENT_NODE)
-        nsIContent_Release(nscontent);
+    nsIContent_Release(nscontent);
     return S_OK;
 }
 
@@ -927,83 +989,6 @@ BOOL is_gecko_path(const char *path)
     return ret;
 }
 
-struct nsWeakReference {
-    nsIWeakReference nsIWeakReference_iface;
-
-    LONG ref;
-
-    NSContainer *nscontainer;
-};
-
-static inline nsWeakReference *impl_from_nsIWeakReference(nsIWeakReference *iface)
-{
-    return CONTAINING_RECORD(iface, nsWeakReference, nsIWeakReference_iface);
-}
-
-static nsresult NSAPI nsWeakReference_QueryInterface(nsIWeakReference *iface,
-        nsIIDRef riid, void **result)
-{
-    nsWeakReference *This = impl_from_nsIWeakReference(iface);
-
-    if(IsEqualGUID(&IID_nsISupports, riid)) {
-        TRACE("(%p)->(IID_nsISupports %p)\n", This, result);
-        *result = &This->nsIWeakReference_iface;
-    }else if(IsEqualGUID(&IID_nsIWeakReference, riid)) {
-        TRACE("(%p)->(IID_nsIWeakReference %p)\n", This, result);
-        *result = &This->nsIWeakReference_iface;
-    }else {
-        WARN("(%p)->(%s %p)\n", This, debugstr_guid(riid), result);
-        *result = NULL;
-        return NS_NOINTERFACE;
-    }
-
-    nsISupports_AddRef((nsISupports*)*result);
-    return NS_OK;
-}
-
-static nsrefcnt NSAPI nsWeakReference_AddRef(nsIWeakReference *iface)
-{
-    nsWeakReference *This = impl_from_nsIWeakReference(iface);
-    LONG ref = InterlockedIncrement(&This->ref);
-
-    TRACE("(%p) ref=%d\n", This, ref);
-
-    return ref;
-}
-
-static nsrefcnt NSAPI nsWeakReference_Release(nsIWeakReference *iface)
-{
-    nsWeakReference *This = impl_from_nsIWeakReference(iface);
-    LONG ref = InterlockedIncrement(&This->ref);
-
-    TRACE("(%p) ref=%d\n", This, ref);
-
-    if(!ref) {
-        assert(!This->nscontainer);
-        heap_free(This);
-    }
-
-    return ref;
-}
-
-static nsresult NSAPI nsWeakReference_QueryReferent(nsIWeakReference *iface,
-        const nsIID *riid, void **result)
-{
-    nsWeakReference *This = impl_from_nsIWeakReference(iface);
-
-    if(!This->nscontainer)
-        return NS_ERROR_NULL_POINTER;
-
-    return nsIWebBrowserChrome_QueryInterface(&This->nscontainer->nsIWebBrowserChrome_iface, riid, result);
-}
-
-static const nsIWeakReferenceVtbl nsWeakReferenceVtbl = {
-    nsWeakReference_QueryInterface,
-    nsWeakReference_AddRef,
-    nsWeakReference_Release,
-    nsWeakReference_QueryReferent
-};
-
 /**********************************************************
  *      nsIWebBrowserChrome interface
  */
@@ -1040,6 +1025,9 @@ static nsresult NSAPI nsWebBrowserChrome_QueryInterface(nsIWebBrowserChrome *ifa
     }else if(IsEqualGUID(&IID_nsIInterfaceRequestor, riid)) {
         TRACE("(%p)->(IID_nsIInterfaceRequestor %p)\n", This, result);
         *result = &This->nsIInterfaceRequestor_iface;
+    }else if(IsEqualGUID(&IID_nsIWeakReference, riid)) {
+        TRACE("(%p)->(IID_nsIWeakReference %p)\n", This, result);
+        *result = &This->nsIWeakReference_iface;
     }else if(IsEqualGUID(&IID_nsISupportsWeakReference, riid)) {
         TRACE("(%p)->(IID_nsISupportsWeakReference %p)\n", This, result);
         *result = &This->nsISupportsWeakReference_iface;
@@ -1074,10 +1062,6 @@ static nsrefcnt NSAPI nsWebBrowserChrome_Release(nsIWebBrowserChrome *iface)
     if(!ref) {
         if(This->parent)
             nsIWebBrowserChrome_Release(&This->parent->nsIWebBrowserChrome_iface);
-        if(This->weak_reference) {
-            This->weak_reference->nscontainer = NULL;
-            nsIWeakReference_Release(&This->weak_reference->nsIWeakReference_iface);
-        }
         heap_free(This);
     }
 
@@ -1661,6 +1645,44 @@ static const nsIInterfaceRequestorVtbl nsInterfaceRequestorVtbl = {
     nsInterfaceRequestor_GetInterface
 };
 
+static inline NSContainer *impl_from_nsIWeakReference(nsIWeakReference *iface)
+{
+    return CONTAINING_RECORD(iface, NSContainer, nsIWeakReference_iface);
+}
+
+static nsresult NSAPI nsWeakReference_QueryInterface(nsIWeakReference *iface,
+        nsIIDRef riid, void **result)
+{
+    NSContainer *This = impl_from_nsIWeakReference(iface);
+    return nsIWebBrowserChrome_QueryInterface(&This->nsIWebBrowserChrome_iface, riid, result);
+}
+
+static nsrefcnt NSAPI nsWeakReference_AddRef(nsIWeakReference *iface)
+{
+    NSContainer *This = impl_from_nsIWeakReference(iface);
+    return nsIWebBrowserChrome_AddRef(&This->nsIWebBrowserChrome_iface);
+}
+
+static nsrefcnt NSAPI nsWeakReference_Release(nsIWeakReference *iface)
+{
+    NSContainer *This = impl_from_nsIWeakReference(iface);
+    return nsIWebBrowserChrome_Release(&This->nsIWebBrowserChrome_iface);
+}
+
+static nsresult NSAPI nsWeakReference_QueryReferent(nsIWeakReference *iface,
+        const nsIID *riid, void **result)
+{
+    NSContainer *This = impl_from_nsIWeakReference(iface);
+    return nsIWebBrowserChrome_QueryInterface(&This->nsIWebBrowserChrome_iface, riid, result);
+}
+
+static const nsIWeakReferenceVtbl nsWeakReferenceVtbl = {
+    nsWeakReference_QueryInterface,
+    nsWeakReference_AddRef,
+    nsWeakReference_Release,
+    nsWeakReference_QueryReferent
+};
+
 static inline NSContainer *impl_from_nsISupportsWeakReference(nsISupportsWeakReference *iface)
 {
     return CONTAINING_RECORD(iface, NSContainer, nsISupportsWeakReference_iface);
@@ -1692,18 +1714,8 @@ static nsresult NSAPI nsSupportsWeakReference_GetWeakReference(nsISupportsWeakRe
 
     TRACE("(%p)->(%p)\n", This, _retval);
 
-    if(!This->weak_reference) {
-        This->weak_reference = heap_alloc(sizeof(nsWeakReference));
-        if(!This->weak_reference)
-            return NS_ERROR_OUT_OF_MEMORY;
-
-        This->weak_reference->nsIWeakReference_iface.lpVtbl = &nsWeakReferenceVtbl;
-        This->weak_reference->ref = 1;
-        This->weak_reference->nscontainer = This;
-    }
-
-    *_retval = &This->weak_reference->nsIWeakReference_iface;
-    nsIWeakReference_AddRef(*_retval);
+    nsIWeakReference_AddRef(&This->nsIWeakReference_iface);
+    *_retval = &This->nsIWeakReference_iface;
     return NS_OK;
 }
 
@@ -1744,6 +1756,7 @@ NSContainer *NSContainer_Create(HTMLDocumentObj *doc, NSContainer *parent)
     ret->nsIEmbeddingSiteWindow_iface.lpVtbl = &nsEmbeddingSiteWindowVtbl;
     ret->nsITooltipListener_iface.lpVtbl = &nsTooltipListenerVtbl;
     ret->nsIInterfaceRequestor_iface.lpVtbl = &nsInterfaceRequestorVtbl;
+    ret->nsIWeakReference_iface.lpVtbl = &nsWeakReferenceVtbl;
     ret->nsISupportsWeakReference_iface.lpVtbl = &nsSupportsWeakReferenceVtbl;
 
     ret->doc = doc;
