@@ -30,6 +30,7 @@
 #include "shlguid.h"
 
 #include "wine/debug.h"
+#include "wine/unicode.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
@@ -183,7 +184,6 @@ static nsresult NSAPI handle_keypress(nsIDOMEventListener *iface,
 
 static void handle_docobj_load(HTMLDocumentObj *doc)
 {
-    IOleCommandTarget *olecmd = NULL;
     HRESULT hres;
 
     if(doc->nscontainer->editor_controller) {
@@ -195,31 +195,33 @@ static void handle_docobj_load(HTMLDocumentObj *doc)
         handle_edit_load(&doc->basedoc);
 
     if(doc->client) {
+        IOleCommandTarget *olecmd = NULL;
+
         hres = IOleClientSite_QueryInterface(doc->client, &IID_IOleCommandTarget, (void**)&olecmd);
-        if(FAILED(hres))
-            olecmd = NULL;
-    }
+        if(SUCCEEDED(hres)) {
+            if(doc->download_state) {
+                VARIANT state, progress;
 
-    if(doc->download_state) {
-        if(olecmd) {
-            VARIANT progress;
+                V_VT(&progress) = VT_I4;
+                V_I4(&progress) = 0;
+                IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_SETPROGRESSPOS,
+                        OLECMDEXECOPT_DONTPROMPTUSER, &progress, NULL);
 
-            V_VT(&progress) = VT_I4;
-            V_I4(&progress) = 0;
-            IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_SETPROGRESSPOS,
-                    OLECMDEXECOPT_DONTPROMPTUSER, &progress, NULL);
+                V_VT(&state) = VT_I4;
+                V_I4(&state) = 0;
+                IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_SETDOWNLOADSTATE,
+                        OLECMDEXECOPT_DONTPROMPTUSER, &state, NULL);
+            }
+
+            IOleCommandTarget_Exec(olecmd, &CGID_ShellDocView, 103, 0, NULL, NULL);
+            IOleCommandTarget_Exec(olecmd, &CGID_MSHTML, IDM_PARSECOMPLETE, 0, NULL, NULL);
+            IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_HTTPEQUIV_DONE, 0, NULL, NULL);
+
+            IOleCommandTarget_Release(olecmd);
         }
-
-        set_download_state(doc, 0);
     }
 
-    if(olecmd) {
-        IOleCommandTarget_Exec(olecmd, &CGID_ShellDocView, 103, 0, NULL, NULL);
-        IOleCommandTarget_Exec(olecmd, &CGID_MSHTML, IDM_PARSECOMPLETE, 0, NULL, NULL);
-        IOleCommandTarget_Exec(olecmd, NULL, OLECMDID_HTTPEQUIV_DONE, 0, NULL, NULL);
-
-        IOleCommandTarget_Release(olecmd);
-    }
+    doc->download_state = 0;
 }
 
 static nsresult NSAPI handle_load(nsIDOMEventListener *iface, nsIDOMEvent *event)
@@ -252,7 +254,7 @@ static nsresult NSAPI handle_load(nsIDOMEventListener *iface, nsIDOMEvent *event
         update_title(doc_obj);
     }
 
-    if(doc_obj && doc_obj->usermode!=EDITMODE && doc_obj->doc_object_service)
+    if(doc_obj && doc_obj->doc_object_service)
         IDocObjectService_FireDocumentComplete(doc_obj->doc_object_service,
                 &doc->basedoc.window->IHTMLWindow2_iface, 0);
 
