@@ -2,7 +2,7 @@
  *    DOM Document implementation
  *
  * Copyright 2005 Mike McCormack
- * Copyright 2010-2011 Adam Martinson for CodeWeavers
+ * Copyright 2010 Adam Martinson for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -94,7 +94,7 @@ typedef struct domdoc domdoc;
 
 struct ConnectionPoint
 {
-    IConnectionPoint IConnectionPoint_iface;
+    const IConnectionPointVtbl  *lpVtblConnectionPoint;
     const IID *iid;
 
     ConnectionPoint *next;
@@ -120,12 +120,12 @@ typedef enum {
 struct domdoc
 {
     xmlnode node;
-    IXMLDOMDocument3          IXMLDOMDocument3_iface;
-    IPersistStreamInit        IPersistStreamInit_iface;
-    IObjectWithSite           IObjectWithSite_iface;
-    IObjectSafety             IObjectSafety_iface;
-    ISupportErrorInfo         ISupportErrorInfo_iface;
-    IConnectionPointContainer IConnectionPointContainer_iface;
+    const struct IXMLDOMDocument3Vtbl          *lpVtbl;
+    const struct IPersistStreamInitVtbl        *lpvtblIPersistStreamInit;
+    const struct IObjectWithSiteVtbl           *lpvtblIObjectWithSite;
+    const struct IObjectSafetyVtbl             *lpvtblIObjectSafety;
+    const struct ISupportErrorInfoVtbl         *lpvtblISupportErrorInfo;
+    const struct IConnectionPointContainerVtbl *lpVtblConnectionPointContainer;
     LONG ref;
     VARIANT_BOOL async;
     VARIANT_BOOL validating;
@@ -181,7 +181,7 @@ static HRESULT set_doc_event(domdoc *doc, eventid_t eid, const VARIANT *v)
 
 static inline ConnectionPoint *impl_from_IConnectionPoint(IConnectionPoint *iface)
 {
-    return CONTAINING_RECORD(iface, ConnectionPoint, IConnectionPoint_iface);
+    return (ConnectionPoint *)((char*)iface - FIELD_OFFSET(ConnectionPoint, lpVtblConnectionPoint));
 }
 
 /*
@@ -287,7 +287,7 @@ static xmldoc_priv * create_priv(void)
     return priv;
 }
 
-static domdoc_properties * create_properties(MSXML_VERSION version)
+static domdoc_properties * create_properties(const GUID *clsid)
 {
     domdoc_properties *properties = heap_alloc(sizeof(domdoc_properties));
 
@@ -298,8 +298,26 @@ static domdoc_properties * create_properties(MSXML_VERSION version)
     properties->selectNsStr_len = 0;
 
     /* properties that are dependent on object versions */
-    properties->version = version;
-    properties->XPath = (version == MSXML4 || version == MSXML6);
+    if (IsEqualCLSID(clsid, &CLSID_DOMDocument30))
+    {
+        properties->version = MSXML3;
+        properties->XPath = FALSE;
+    }
+    else if (IsEqualCLSID(clsid, &CLSID_DOMDocument40))
+    {
+        properties->version = MSXML4;
+        properties->XPath = TRUE;
+    }
+    else if (IsEqualCLSID(clsid, &CLSID_DOMDocument60))
+    {
+        properties->version = MSXML6;
+        properties->XPath = TRUE;
+    }
+    else
+    {
+        properties->version = MSXML_DEFAULT;
+        properties->XPath = FALSE;
+    }
 
     return properties;
 }
@@ -352,6 +370,11 @@ static void free_properties(domdoc_properties* properties)
     }
 }
 
+static BOOL xmldoc_has_decl(xmlDocPtr doc)
+{
+    return doc->children && (xmlStrEqual(doc->children->name, (xmlChar*)"xml") == 1);
+}
+
 /* links a "<?xml" node as a first child */
 void xmldoc_link_xmldecl(xmlDocPtr doc, xmlNodePtr node)
 {
@@ -437,7 +460,7 @@ static void sax_serror(void* ctx, xmlErrorPtr err)
     LIBXML2_CALLBACK_SERROR(doparse, err);
 }
 
-static xmlDocPtr doparse(domdoc* This, char const* ptr, int len, xmlCharEncoding encoding)
+static xmlDocPtr doparse(domdoc* This, char *ptr, int len, xmlChar const* encoding)
 {
     xmlDocPtr doc = NULL;
     xmlParserCtxtPtr pctx;
@@ -488,10 +511,7 @@ static xmlDocPtr doparse(domdoc* This, char const* ptr, int len, xmlCharEncoding
     pctx->sax = &sax_handler;
     pctx->_private = This;
     pctx->recovery = 0;
-
-    if (encoding != XML_CHAR_ENCODING_NONE)
-        xmlSwitchEncoding(pctx, encoding);
-
+    pctx->encoding = xmlStrdup(encoding);
     xmlParseDocument(pctx);
 
     if (pctx->wellFormed)
@@ -538,10 +558,10 @@ static xmlDocPtr doparse(domdoc* This, char const* ptr, int len, xmlCharEncoding
     return doc;
 }
 
-void xmldoc_init(xmlDocPtr doc, MSXML_VERSION version)
+void xmldoc_init(xmlDocPtr doc, const GUID *clsid)
 {
     doc->_private = create_priv();
-    priv_from_xmlDocPtr(doc)->properties = create_properties(version);
+    priv_from_xmlDocPtr(doc)->properties = create_properties(clsid);
 }
 
 LONG xmldoc_add_ref(xmlDocPtr doc)
@@ -635,32 +655,32 @@ static HRESULT attach_xmldoc(domdoc *This, xmlDocPtr xml )
 
 static inline domdoc *impl_from_IXMLDOMDocument3( IXMLDOMDocument3 *iface )
 {
-    return CONTAINING_RECORD(iface, domdoc, IXMLDOMDocument3_iface);
+    return (domdoc *)((char*)iface - FIELD_OFFSET(domdoc, lpVtbl));
 }
 
 static inline domdoc *impl_from_IPersistStreamInit(IPersistStreamInit *iface)
 {
-    return CONTAINING_RECORD(iface, domdoc, IPersistStreamInit_iface);
+    return (domdoc *)((char*)iface - FIELD_OFFSET(domdoc, lpvtblIPersistStreamInit));
 }
 
 static inline domdoc *impl_from_IObjectWithSite(IObjectWithSite *iface)
 {
-    return CONTAINING_RECORD(iface, domdoc, IObjectWithSite_iface);
+    return (domdoc *)((char*)iface - FIELD_OFFSET(domdoc, lpvtblIObjectWithSite));
 }
 
 static inline domdoc *impl_from_IObjectSafety(IObjectSafety *iface)
 {
-    return CONTAINING_RECORD(iface, domdoc, IObjectSafety_iface);
+    return (domdoc *)((char*)iface - FIELD_OFFSET(domdoc, lpvtblIObjectSafety));
 }
 
 static inline domdoc *impl_from_ISupportErrorInfo(ISupportErrorInfo *iface)
 {
-    return CONTAINING_RECORD(iface, domdoc, ISupportErrorInfo_iface);
+    return (domdoc *)((char*)iface - FIELD_OFFSET(domdoc, lpvtblISupportErrorInfo));
 }
 
 static inline domdoc *impl_from_IConnectionPointContainer(IConnectionPointContainer *iface)
 {
-    return CONTAINING_RECORD(iface, domdoc, IConnectionPointContainer_iface);
+    return (domdoc *)((char*)iface - FIELD_OFFSET(domdoc, lpVtblConnectionPointContainer));
 }
 
 /************************************************************************
@@ -670,21 +690,21 @@ static HRESULT WINAPI domdoc_IPersistStreamInit_QueryInterface(
     IPersistStreamInit *iface, REFIID riid, void **ppvObj)
 {
     domdoc* This = impl_from_IPersistStreamInit(iface);
-    return IXMLDOMDocument3_QueryInterface(&This->IXMLDOMDocument3_iface, riid, ppvObj);
+    return IXMLDOMDocument3_QueryInterface((IXMLDOMDocument3*)&This->lpVtbl, riid, ppvObj);
 }
 
 static ULONG WINAPI domdoc_IPersistStreamInit_AddRef(
     IPersistStreamInit *iface)
 {
     domdoc* This = impl_from_IPersistStreamInit(iface);
-    return IXMLDOMDocument3_AddRef(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_AddRef((IXMLDOMDocument3*)&This->lpVtbl);
 }
 
 static ULONG WINAPI domdoc_IPersistStreamInit_Release(
     IPersistStreamInit *iface)
 {
     domdoc* This = impl_from_IPersistStreamInit(iface);
-    return IXMLDOMDocument3_Release(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_Release((IXMLDOMDocument3*)&This->lpVtbl);
 }
 
 static HRESULT WINAPI domdoc_IPersistStreamInit_GetClassID(
@@ -748,7 +768,7 @@ static HRESULT WINAPI domdoc_IPersistStreamInit_Load(
     len = GlobalSize(hglobal);
     ptr = GlobalLock(hglobal);
     if (len != 0)
-        xmldoc = doparse(This, ptr, len, XML_CHAR_ENCODING_NONE);
+        xmldoc = doparse(This, ptr, len, NULL);
     GlobalUnlock(hglobal);
 
     if (!xmldoc)
@@ -771,7 +791,7 @@ static HRESULT WINAPI domdoc_IPersistStreamInit_Save(
 
     TRACE("(%p)->(%p %d)\n", This, stream, clr_dirty);
 
-    hr = IXMLDOMDocument3_get_xml(&This->IXMLDOMDocument3_iface, &xmlString);
+    hr = IXMLDOMDocument3_get_xml( (IXMLDOMDocument3*)&This->lpVtbl, &xmlString );
     if(hr == S_OK)
     {
         DWORD len = SysStringLen(xmlString) * sizeof(WCHAR);
@@ -820,21 +840,21 @@ static HRESULT WINAPI support_error_QueryInterface(
     REFIID riid, void** ppvObj )
 {
     domdoc *This = impl_from_ISupportErrorInfo(iface);
-    return IXMLDOMDocument3_QueryInterface(&This->IXMLDOMDocument3_iface, riid, ppvObj);
+    return IXMLDOMDocument3_QueryInterface((IXMLDOMDocument3 *)This, riid, ppvObj);
 }
 
 static ULONG WINAPI support_error_AddRef(
     ISupportErrorInfo *iface )
 {
     domdoc *This = impl_from_ISupportErrorInfo(iface);
-    return IXMLDOMDocument3_AddRef(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_AddRef((IXMLDOMDocument3 *)This);
 }
 
 static ULONG WINAPI support_error_Release(
     ISupportErrorInfo *iface )
 {
     domdoc *This = impl_from_ISupportErrorInfo(iface);
-    return IXMLDOMDocument3_Release(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_Release((IXMLDOMDocument3 *)This);
 }
 
 static HRESULT WINAPI support_error_InterfaceSupportsErrorInfo(
@@ -874,19 +894,19 @@ static HRESULT WINAPI domdoc_QueryInterface( IXMLDOMDocument3 *iface, REFIID rii
     else if (IsEqualGUID(&IID_IPersistStream, riid) ||
              IsEqualGUID(&IID_IPersistStreamInit, riid))
     {
-        *ppvObject = &This->IPersistStreamInit_iface;
+        *ppvObject = &(This->lpvtblIPersistStreamInit);
     }
     else if (IsEqualGUID(&IID_IObjectWithSite, riid))
     {
-        *ppvObject = &This->IObjectWithSite_iface;
+        *ppvObject = &(This->lpvtblIObjectWithSite);
     }
     else if (IsEqualGUID(&IID_IObjectSafety, riid))
     {
-        *ppvObject = &This->IObjectSafety_iface;
+        *ppvObject = &(This->lpvtblIObjectSafety);
     }
     else if( IsEqualGUID( riid, &IID_ISupportErrorInfo ))
     {
-        *ppvObject = &This->ISupportErrorInfo_iface;
+        *ppvObject = &This->lpvtblISupportErrorInfo;
     }
     else if(node_query_interface(&This->node, riid, ppvObject))
     {
@@ -894,7 +914,7 @@ static HRESULT WINAPI domdoc_QueryInterface( IXMLDOMDocument3 *iface, REFIID rii
     }
     else if (IsEqualGUID( riid, &IID_IConnectionPointContainer ))
     {
-        *ppvObject = &This->IConnectionPointContainer_iface;
+        *ppvObject = &This->lpVtblConnectionPointContainer;
     }
     else
     {
@@ -1023,8 +1043,8 @@ static HRESULT WINAPI domdoc_Invoke(
     hr = get_typeinfo(IXMLDOMDocument2_tid, &typeinfo);
     if(SUCCEEDED(hr))
     {
-        hr = ITypeInfo_Invoke(typeinfo, &This->IXMLDOMDocument3_iface, dispIdMember, wFlags,
-                pDispParams, pVarResult, pExcepInfo, puArgErr);
+        hr = ITypeInfo_Invoke(typeinfo, &(This->lpVtbl), dispIdMember, wFlags, pDispParams,
+                pVarResult, pExcepInfo, puArgErr);
         ITypeInfo_Release(typeinfo);
     }
 
@@ -1374,7 +1394,8 @@ static HRESULT WINAPI domdoc_get_xml(
     if(!buf)
         return E_OUTOFMEMORY;
 
-    options = XML_SAVE_FORMAT | XML_SAVE_NO_DECL;
+    options  = xmldoc_has_decl(get_doc(This)) ? XML_SAVE_NO_DECL : 0;
+    options |= XML_SAVE_FORMAT;
     ctxt = xmlSaveToIO(domdoc_get_xml_writecallback, NULL, buf, "UTF-8", options);
 
     if(!ctxt)
@@ -2056,7 +2077,7 @@ static HRESULT domdoc_onDataAvailable(void *obj, char *ptr, DWORD len)
     domdoc *This = obj;
     xmlDocPtr xmldoc;
 
-    xmldoc = doparse(This, ptr, len, XML_CHAR_ENCODING_NONE);
+    xmldoc = doparse(This, ptr, len, NULL);
     if(xmldoc) {
         xmldoc->_private = create_priv();
         return attach_xmldoc(This, xmldoc);
@@ -2095,8 +2116,6 @@ static HRESULT WINAPI domdoc_load(
 
     TRACE("(%p)->(%s)\n", This, debugstr_variant(&source));
 
-    if (!isSuccessful)
-        return E_POINTER;
     *isSuccessful = VARIANT_FALSE;
 
     assert( &This->node );
@@ -2110,51 +2129,6 @@ static HRESULT WINAPI domdoc_load(
         if (!V_BSTRREF(&source)) return E_INVALIDARG;
         filename = *V_BSTRREF(&source);
         break;
-    case VT_ARRAY|VT_UI1:
-        {
-            SAFEARRAY *psa = V_ARRAY(&source);
-            char *str;
-            LONG len;
-            UINT dim = SafeArrayGetDim(psa);
-
-            switch (dim)
-            {
-            case 0:
-                ERR("SAFEARRAY == NULL\n");
-                hr = This->error = E_INVALIDARG;
-                break;
-            case 1:
-                /* Only takes UTF-8 strings.
-                 * NOT NULL-terminated. */
-                SafeArrayAccessData(psa, (void**)&str);
-                SafeArrayGetUBound(psa, 1, &len);
-
-                if ((xmldoc = doparse(This, str, ++len, XML_CHAR_ENCODING_UTF8)))
-                {
-                    hr = This->error = S_OK;
-                    *isSuccessful = VARIANT_TRUE;
-                    TRACE("parsed document %p\n", xmldoc);
-                }
-                else
-                {
-                    This->error = E_FAIL;
-                    TRACE("failed to parse document\n");
-                }
-
-                SafeArrayUnaccessData(psa);
-
-                if(xmldoc)
-                {
-                    xmldoc->_private = create_priv();
-                    return attach_xmldoc(This, xmldoc);
-                }
-                break;
-            default:
-                FIXME("unhandled SAFEARRAY dim: %d\n", dim);
-                hr = This->error = E_NOTIMPL;
-            }
-        }
-        break;
     case VT_UNKNOWN:
         hr = IUnknown_QueryInterface(V_UNKNOWN(&source), &IID_IXMLDOMDocument3, (void**)&pNewDoc);
         if(hr == S_OK)
@@ -2311,6 +2285,21 @@ static HRESULT WINAPI domdoc_abort(
 }
 
 
+static BOOL bstr_to_utf8( BSTR bstr, char **pstr, int *plen )
+{
+    UINT len;
+    LPSTR str;
+
+    len = WideCharToMultiByte( CP_UTF8, 0, bstr, -1, NULL, 0, NULL, NULL );
+    str = heap_alloc( len );
+    if ( !str )
+        return FALSE;
+    WideCharToMultiByte( CP_UTF8, 0, bstr, -1, str, len, NULL, NULL );
+    *plen = len;
+    *pstr = str;
+    return TRUE;
+}
+
 /* don't rely on data to be in BSTR format, treat it as WCHAR string */
 static HRESULT WINAPI domdoc_loadXML(
     IXMLDOMDocument3 *iface,
@@ -2318,8 +2307,11 @@ static HRESULT WINAPI domdoc_loadXML(
     VARIANT_BOOL* isSuccessful )
 {
     domdoc *This = impl_from_IXMLDOMDocument3( iface );
+    static const xmlChar encoding[] = "UTF-8";
     xmlDocPtr xmldoc = NULL;
     HRESULT hr = S_FALSE, hr2;
+    char *str;
+    int len;
 
     TRACE("(%p)->(%s %p)\n", This, debugstr_w( bstrXML ), isSuccessful );
 
@@ -2329,9 +2321,10 @@ static HRESULT WINAPI domdoc_loadXML(
     {
         *isSuccessful = VARIANT_FALSE;
 
-        if ( bstrXML )
+        if ( bstrXML && bstr_to_utf8( bstrXML, &str, &len ) )
         {
-            xmldoc = doparse(This, (LPCSTR)bstrXML, lstrlenW(bstrXML) * sizeof(*bstrXML), XML_CHAR_ENCODING_UTF16LE);
+            xmldoc = doparse(This, str, len, encoding);
+            heap_free( str );
             if ( !xmldoc )
             {
                 This->error = E_FAIL;
@@ -2405,76 +2398,72 @@ static HRESULT WINAPI domdoc_save(
     xmlNodePtr xmldecl;
     HRESULT ret = S_OK;
 
-    TRACE("(%p)->(%s)\n", This, debugstr_variant(&destination));
+    TRACE("(%p)->(var(vt %d, %s))\n", This, V_VT(&destination),
+          V_VT(&destination) == VT_BSTR ? debugstr_w(V_BSTR(&destination)) : NULL);
 
-    switch (V_VT(&destination))
+    if(V_VT(&destination) != VT_BSTR && V_VT(&destination) != VT_UNKNOWN)
     {
-    case VT_UNKNOWN:
-        {
-            IUnknown *pUnk = V_UNKNOWN(&destination);
-            IXMLDOMDocument2 *document;
-            IStream *stream;
-
-            ret = IUnknown_QueryInterface(pUnk, &IID_IXMLDOMDocument3, (void**)&document);
-            if(ret == S_OK)
-            {
-                VARIANT_BOOL success;
-                BSTR xml;
-
-                ret = IXMLDOMDocument3_get_xml(iface, &xml);
-                if(ret == S_OK)
-                {
-                    ret = IXMLDOMDocument3_loadXML(document, xml, &success);
-                    SysFreeString(xml);
-                }
-
-                IXMLDOMDocument3_Release(document);
-                return ret;
-            }
-
-            ret = IUnknown_QueryInterface(pUnk, &IID_IStream, (void**)&stream);
-            if(ret == S_OK)
-            {
-                ctx = xmlSaveToIO(domdoc_stream_save_writecallback,
-                    domdoc_stream_save_closecallback, stream, NULL, XML_SAVE_NO_DECL);
-
-                if(!ctx)
-                {
-                    IStream_Release(stream);
-                    return E_FAIL;
-                }
-            }
-        }
-        break;
-
-    case VT_BSTR:
-    case VT_BSTR | VT_BYREF:
-        {
-            /* save with file path */
-            HANDLE handle = CreateFileW( (V_VT(&destination) & VT_BYREF)? *V_BSTRREF(&destination) : V_BSTR(&destination),
-                                         GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
-            if( handle == INVALID_HANDLE_VALUE )
-            {
-                WARN("failed to create file\n");
-                return E_FAIL;
-            }
-
-            /* disable top XML declaration */
-            ctx = xmlSaveToIO(domdoc_save_writecallback, domdoc_save_closecallback,
-                              handle, NULL, XML_SAVE_NO_DECL);
-            if (!ctx)
-            {
-                CloseHandle(handle);
-                return E_FAIL;
-            }
-        }
-        break;
-
-    default:
-        FIXME("Unhandled VARIANT: %s\n", debugstr_variant(&destination));
+        FIXME("Unhandled vt %d\n", V_VT(&destination));
         return S_FALSE;
     }
 
+    if(V_VT(&destination) == VT_UNKNOWN)
+    {
+        IUnknown *pUnk = V_UNKNOWN(&destination);
+        IXMLDOMDocument2 *document;
+        IStream *stream;
+
+        ret = IUnknown_QueryInterface(pUnk, &IID_IXMLDOMDocument3, (void**)&document);
+        if(ret == S_OK)
+        {
+            VARIANT_BOOL success;
+            BSTR xml;
+
+            ret = IXMLDOMDocument3_get_xml(iface, &xml);
+            if(ret == S_OK)
+            {
+                ret = IXMLDOMDocument3_loadXML(document, xml, &success);
+                SysFreeString(xml);
+            }
+
+            IXMLDOMDocument3_Release(document);
+            return ret;
+        }
+
+        ret = IUnknown_QueryInterface(pUnk, &IID_IStream, (void**)&stream);
+        if(ret == S_OK)
+        {
+            ctx = xmlSaveToIO(domdoc_stream_save_writecallback,
+                domdoc_stream_save_closecallback, stream, NULL, XML_SAVE_NO_DECL);
+
+            if(!ctx)
+            {
+                IStream_Release(stream);
+                return E_FAIL;
+            }
+        }
+    }
+    else
+    {
+        /* save with file path */
+        HANDLE handle = CreateFileW( V_BSTR(&destination), GENERIC_WRITE, 0,
+                                    NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
+        if( handle == INVALID_HANDLE_VALUE )
+        {
+            WARN("failed to create file\n");
+            return E_FAIL;
+        }
+
+        /* disable top XML declaration */
+        ctx = xmlSaveToIO(domdoc_save_writecallback, domdoc_save_closecallback,
+                          handle, NULL, XML_SAVE_NO_DECL);
+        if (!ctx)
+        {
+            CloseHandle(handle);
+            return E_FAIL;
+        }
+    }
+
     xmldecl = xmldoc_unlink_xmldecl(get_doc(This));
     if (xmlSaveDoc(ctx, get_doc(This)) == -1) ret = S_FALSE;
     xmldoc_link_xmldecl(get_doc(This), xmldecl);
@@ -2875,6 +2864,7 @@ static HRESULT WINAPI domdoc_setProperty(
         {
             ctx = xmlXPathNewContext(This->node.node->doc);
             pTokBegin = nsStr;
+            pTokEnd = nsStr;
             for (; *pTokBegin; pTokBegin = pTokEnd)
             {
                 if (pNsEntry != NULL)
@@ -2901,6 +2891,7 @@ static HRESULT WINAPI domdoc_setProperty(
                 {
                     /*valid for XSLPattern?*/
                     FIXME("Setting default xmlns not supported - skipping.\n");
+                    pTokBegin = pTokEnd;
                     continue;
                 }
                 else if (*pTokBegin == ':')
@@ -3138,19 +3129,19 @@ static HRESULT WINAPI ConnectionPointContainer_QueryInterface(IConnectionPointCo
                                                               REFIID riid, void **ppv)
 {
     domdoc *This = impl_from_IConnectionPointContainer(iface);
-    return IXMLDOMDocument3_QueryInterface(&This->IXMLDOMDocument3_iface, riid, ppv);
+    return IXMLDOMDocument3_QueryInterface((IXMLDOMDocument3*)This, riid, ppv);
 }
 
 static ULONG WINAPI ConnectionPointContainer_AddRef(IConnectionPointContainer *iface)
 {
     domdoc *This = impl_from_IConnectionPointContainer(iface);
-    return IXMLDOMDocument3_AddRef(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_AddRef((IXMLDOMDocument3*)This);
 }
 
 static ULONG WINAPI ConnectionPointContainer_Release(IConnectionPointContainer *iface)
 {
     domdoc *This = impl_from_IConnectionPointContainer(iface);
-    return IXMLDOMDocument3_Release(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_Release((IXMLDOMDocument3*)This);
 }
 
 static HRESULT WINAPI ConnectionPointContainer_EnumConnectionPoints(IConnectionPointContainer *iface,
@@ -3174,7 +3165,7 @@ static HRESULT WINAPI ConnectionPointContainer_FindConnectionPoint(IConnectionPo
     for(iter = This->cp_list; iter; iter = iter->next)
     {
         if (IsEqualGUID(iter->iid, riid))
-            *cp = &iter->IConnectionPoint_iface;
+            *cp = (IConnectionPoint*)&iter->lpVtblConnectionPoint;
     }
 
     if (*cp)
@@ -3306,7 +3297,7 @@ static const IConnectionPointVtbl ConnectionPointVtbl =
 
 static void ConnectionPoint_Init(ConnectionPoint *cp, struct domdoc *doc, REFIID riid)
 {
-    cp->IConnectionPoint_iface.lpVtbl = &ConnectionPointVtbl;
+    cp->lpVtblConnectionPoint = &ConnectionPointVtbl;
     cp->doc = doc;
     cp->iid = riid;
     cp->sinks = NULL;
@@ -3315,7 +3306,7 @@ static void ConnectionPoint_Init(ConnectionPoint *cp, struct domdoc *doc, REFIID
     cp->next = doc->cp_list;
     doc->cp_list = cp;
 
-    cp->container = &doc->IConnectionPointContainer_iface;
+    cp->container = (IConnectionPointContainer*)&doc->lpVtblConnectionPointContainer;
 }
 
 /* domdoc implementation of IObjectWithSite */
@@ -3323,19 +3314,19 @@ static HRESULT WINAPI
 domdoc_ObjectWithSite_QueryInterface( IObjectWithSite* iface, REFIID riid, void** ppvObject )
 {
     domdoc *This = impl_from_IObjectWithSite(iface);
-    return IXMLDOMDocument3_QueryInterface(&This->IXMLDOMDocument3_iface, riid, ppvObject);
+    return IXMLDOMDocument3_QueryInterface( (IXMLDOMDocument3 *)This, riid, ppvObject );
 }
 
 static ULONG WINAPI domdoc_ObjectWithSite_AddRef( IObjectWithSite* iface )
 {
     domdoc *This = impl_from_IObjectWithSite(iface);
-    return IXMLDOMDocument3_AddRef(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_AddRef((IXMLDOMDocument3 *)This);
 }
 
 static ULONG WINAPI domdoc_ObjectWithSite_Release( IObjectWithSite* iface )
 {
     domdoc *This = impl_from_IObjectWithSite(iface);
-    return IXMLDOMDocument3_Release(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_Release((IXMLDOMDocument3 *)This);
 }
 
 static HRESULT WINAPI domdoc_ObjectWithSite_GetSite( IObjectWithSite *iface, REFIID iid, void **ppvSite )
@@ -3389,19 +3380,19 @@ static const IObjectWithSiteVtbl domdocObjectSite =
 static HRESULT WINAPI domdoc_Safety_QueryInterface(IObjectSafety *iface, REFIID riid, void **ppv)
 {
     domdoc *This = impl_from_IObjectSafety(iface);
-    return IXMLDOMDocument3_QueryInterface(&This->IXMLDOMDocument3_iface, riid, ppv);
+    return IXMLDOMDocument3_QueryInterface( (IXMLDOMDocument3 *)This, riid, ppv );
 }
 
 static ULONG WINAPI domdoc_Safety_AddRef(IObjectSafety *iface)
 {
     domdoc *This = impl_from_IObjectSafety(iface);
-    return IXMLDOMDocument3_AddRef(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_AddRef((IXMLDOMDocument3 *)This);
 }
 
 static ULONG WINAPI domdoc_Safety_Release(IObjectSafety *iface)
 {
     domdoc *This = impl_from_IObjectSafety(iface);
-    return IXMLDOMDocument3_Release(&This->IXMLDOMDocument3_iface);
+    return IXMLDOMDocument3_Release((IXMLDOMDocument3 *)This);
 }
 
 #define SAFETY_SUPPORTED_OPTIONS (INTERFACESAFE_FOR_UNTRUSTED_CALLER|INTERFACESAFE_FOR_UNTRUSTED_DATA|INTERFACE_USES_SECURITY_MANAGER)
@@ -3430,8 +3421,7 @@ static HRESULT WINAPI domdoc_Safety_SetInterfaceSafetyOptions(IObjectSafety *ifa
     if ((mask & ~SAFETY_SUPPORTED_OPTIONS) != 0)
         return E_FAIL;
 
-    This->safeopt = (This->safeopt & ~mask) | (mask & enabled);
-
+    This->safeopt = enabled & mask & SAFETY_SUPPORTED_OPTIONS;
     return S_OK;
 }
 
@@ -3466,12 +3456,12 @@ HRESULT get_domdoc_from_xmldoc(xmlDocPtr xmldoc, IXMLDOMDocument3 **document)
     if( !doc )
         return E_OUTOFMEMORY;
 
-    doc->IXMLDOMDocument3_iface.lpVtbl = &domdoc_vtbl;
-    doc->IPersistStreamInit_iface.lpVtbl = &xmldoc_IPersistStreamInit_VTable;
-    doc->IObjectWithSite_iface.lpVtbl = &domdocObjectSite;
-    doc->IObjectSafety_iface.lpVtbl = &domdocObjectSafetyVtbl;
-    doc->ISupportErrorInfo_iface.lpVtbl = &support_error_vtbl;
-    doc->IConnectionPointContainer_iface.lpVtbl = &ConnectionPointContainerVtbl;
+    doc->lpVtbl = &domdoc_vtbl;
+    doc->lpvtblIPersistStreamInit = &xmldoc_IPersistStreamInit_VTable;
+    doc->lpvtblIObjectWithSite = &domdocObjectSite;
+    doc->lpvtblIObjectSafety = &domdocObjectSafetyVtbl;
+    doc->lpvtblISupportErrorInfo = &support_error_vtbl;
+    doc->lpVtblConnectionPointContainer = &ConnectionPointContainerVtbl;
     doc->ref = 1;
     doc->async = VARIANT_TRUE;
     doc->validating = 0;
@@ -3490,28 +3480,27 @@ HRESULT get_domdoc_from_xmldoc(xmlDocPtr xmldoc, IXMLDOMDocument3 **document)
     ConnectionPoint_Init(&doc->cp_propnotif, doc, &IID_IPropertyNotifySink);
     ConnectionPoint_Init(&doc->cp_domdocevents, doc, &DIID_XMLDOMDocumentEvents);
 
-    init_xmlnode(&doc->node, (xmlNodePtr)xmldoc, (IXMLDOMNode*)&doc->IXMLDOMDocument3_iface,
-            &domdoc_dispex);
+    init_xmlnode(&doc->node, (xmlNodePtr)xmldoc, (IXMLDOMNode*)&doc->lpVtbl, &domdoc_dispex);
 
-    *document = &doc->IXMLDOMDocument3_iface;
+    *document = (IXMLDOMDocument3*)&doc->lpVtbl;
 
     TRACE("returning iface %p\n", *document);
     return S_OK;
 }
 
-HRESULT DOMDocument_create(MSXML_VERSION version, IUnknown *pUnkOuter, void **ppObj)
+HRESULT DOMDocument_create(const GUID *clsid, IUnknown *pUnkOuter, void **ppObj)
 {
     xmlDocPtr xmldoc;
     HRESULT hr;
 
-    TRACE("(%d, %p, %p)\n", version, pUnkOuter, ppObj);
+    TRACE("(%s, %p, %p)\n", debugstr_guid(clsid), pUnkOuter, ppObj);
 
     xmldoc = xmlNewDoc(NULL);
     if(!xmldoc)
         return E_OUTOFMEMORY;
 
     xmldoc->_private = create_priv();
-    priv_from_xmlDocPtr(xmldoc)->properties = create_properties(version);
+    priv_from_xmlDocPtr(xmldoc)->properties = create_properties(clsid);
 
     hr = get_domdoc_from_xmldoc(xmldoc, (IXMLDOMDocument3**)ppObj);
     if(FAILED(hr))
@@ -3541,7 +3530,7 @@ IUnknown* create_domdoc( xmlNodePtr document )
 
 #else
 
-HRESULT DOMDocument_create(MSXML_VERSION version, IUnknown *pUnkOuter, void **ppObj)
+HRESULT DOMDocument_create(const GUID *clsid, IUnknown *pUnkOuter, void **ppObj)
 {
     MESSAGE("This program tried to use a DOMDocument object, but\n"
             "libxml2 support was not present at compile time.\n");
