@@ -199,54 +199,6 @@ VOID WINAPI SetDebugErrorLevel( DWORD dwLevel )
 }
 
 
-/******************************************************************************
- *                    GetProcessDefaultLayout [USER32.@]
- *
- * Gets the default layout for parentless windows.
- * Right now, just returns 0 (left-to-right).
- *
- * RETURNS
- *    Success: Nonzero
- *    Failure: Zero
- *
- * BUGS
- *    No RTL
- */
-BOOL WINAPI GetProcessDefaultLayout( DWORD *pdwDefaultLayout )
-{
-    if ( !pdwDefaultLayout ) {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return FALSE;
-     }
-    FIXME( "( %p ): No BiDi\n", pdwDefaultLayout );
-    *pdwDefaultLayout = 0;
-    return TRUE;
-}
-
-
-/******************************************************************************
- *                    SetProcessDefaultLayout [USER32.@]
- *
- * Sets the default layout for parentless windows.
- * Right now, only accepts 0 (left-to-right).
- *
- * RETURNS
- *    Success: Nonzero
- *    Failure: Zero
- *
- * BUGS
- *    No RTL
- */
-BOOL WINAPI SetProcessDefaultLayout( DWORD dwDefaultLayout )
-{
-    if ( dwDefaultLayout == 0 )
-        return TRUE;
-    FIXME( "( %08x ): No BiDi\n", dwDefaultLayout );
-    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
-    return FALSE;
-}
-
-
 /***********************************************************************
  *		SetWindowStationUser (USER32.@)
  */
@@ -277,6 +229,8 @@ DWORD WINAPI SetLogonNotifyWindow(HWINSTA hwinsta,HWND hwnd)
 static const WCHAR primary_device_name[] = {'\\','\\','.','\\','D','I','S','P','L','A','Y','1',0};
 static const WCHAR primary_device_string[] = {'X','1','1',' ','W','i','n','d','o','w','i','n','g',' ',
                                               'S','y','s','t','e','m',0};
+static const WCHAR primary_device_deviceid[] = {'P','C','I','\\','V','E','N','_','0','0','0','0','&',
+                                                'D','E','V','_','0','0','0','0',0};
 
 /***********************************************************************
  *		EnumDisplayDevicesA (USER32.@)
@@ -331,7 +285,7 @@ BOOL WINAPI EnumDisplayDevicesW( LPCWSTR lpDevice, DWORD i, LPDISPLAY_DEVICEW lp
         DISPLAY_DEVICE_VGA_COMPATIBLE;
 
     if(lpDisplayDevice->cb >= offsetof(DISPLAY_DEVICEW, DeviceID) + sizeof(lpDisplayDevice->DeviceID))
-        lpDisplayDevice->DeviceID[0] = 0;
+        memcpy(lpDisplayDevice->DeviceID, primary_device_deviceid, sizeof(primary_device_deviceid));
     if(lpDisplayDevice->cb >= offsetof(DISPLAY_DEVICEW, DeviceKey) + sizeof(lpDisplayDevice->DeviceKey))
         lpDisplayDevice->DeviceKey[0] = 0;
 
@@ -344,6 +298,7 @@ struct monitor_enum_info
     UINT     max_area;
     UINT     min_distance;
     HMONITOR primary;
+    HMONITOR nearest;
     HMONITOR ret;
 };
 
@@ -376,7 +331,7 @@ static BOOL CALLBACK monitor_enum( HMONITOR monitor, HDC hdc, LPRECT rect, LPARA
         if (distance < info->min_distance)
         {
             info->min_distance = distance;
-            info->ret = monitor;
+            info->nearest = monitor;
         }
     }
     if (!info->primary)
@@ -403,9 +358,14 @@ HMONITOR WINAPI MonitorFromRect( LPRECT rect, DWORD flags )
     info.max_area     = 0;
     info.min_distance = ~0u;
     info.primary      = 0;
+    info.nearest      = 0;
     info.ret          = 0;
     if (!EnumDisplayMonitors( 0, NULL, monitor_enum, (LPARAM)&info )) return 0;
-    if (!info.ret && (flags & MONITOR_DEFAULTTOPRIMARY)) info.ret = info.primary;
+    if (!info.ret)
+    {
+        if (flags & MONITOR_DEFAULTTOPRIMARY) info.ret = info.primary;
+        else if (flags & MONITOR_DEFAULTTONEAREST) info.ret = info.nearest;
+    }
 
     TRACE( "%s flags %x returning %p\n", wine_dbgstr_rect(rect), flags, info.ret );
     return info.ret;
@@ -430,6 +390,8 @@ HMONITOR WINAPI MonitorFromWindow(HWND hWnd, DWORD dwFlags)
     RECT rect;
     WINDOWPLACEMENT wp;
 
+    TRACE("(%p, 0x%08x)\n", hWnd, dwFlags);
+
     if (IsIconic(hWnd) && GetWindowPlacement(hWnd, &wp))
         return MonitorFromRect( &wp.rcNormalPosition, dwFlags );
 
@@ -469,7 +431,12 @@ BOOL WINAPI GetMonitorInfoA(HMONITOR hMonitor, LPMONITORINFO lpMonitorInfo)
  */
 BOOL WINAPI GetMonitorInfoW(HMONITOR hMonitor, LPMONITORINFO lpMonitorInfo)
 {
-    return USER_Driver->pGetMonitorInfo( hMonitor, lpMonitorInfo );
+    BOOL ret = USER_Driver->pGetMonitorInfo( hMonitor, lpMonitorInfo );
+    if (ret)
+        TRACE("flags %04x, monitor %s, work %s\n", lpMonitorInfo->dwFlags,
+              wine_dbgstr_rect(&lpMonitorInfo->rcMonitor),
+              wine_dbgstr_rect(&lpMonitorInfo->rcWork));
+    return ret;
 }
 
 /***********************************************************************
@@ -526,8 +493,8 @@ DWORD WINAPI RegisterTasklist (DWORD x)
  */
 HDEVNOTIFY WINAPI RegisterDeviceNotificationA(HANDLE hnd, LPVOID notifyfilter, DWORD flags)
 {
-    FIXME("(hwnd=%p, filter=%p,flags=0x%08x),\n"
-          "\treturns a fake device notification handle!\n", hnd,notifyfilter,flags );
+    FIXME("(hwnd=%p, filter=%p,flags=0x%08x) returns a fake device notification handle!\n",
+          hnd,notifyfilter,flags );
     return (HDEVNOTIFY) 0xcafecafe;
 }
 
@@ -556,8 +523,9 @@ HDEVNOTIFY WINAPI RegisterDeviceNotificationA(HANDLE hnd, LPVOID notifyfilter, D
  */
 HDEVNOTIFY WINAPI RegisterDeviceNotificationW(HANDLE hRecepient, LPVOID pNotificationFilter, DWORD dwFlags)
 {
-    FIXME("(hwnd=%p, filter=%p,flags=0x%08x), STUB!\n", hRecepient,pNotificationFilter,dwFlags );
-    return 0;
+    FIXME("(hwnd=%p, filter=%p,flags=0x%08x) returns a fake device notification handle!\n",
+          hRecepient,pNotificationFilter,dwFlags );
+    return (HDEVNOTIFY) 0xcafeaffe;
 }
 
 /***********************************************************************
@@ -575,7 +543,16 @@ BOOL  WINAPI UnregisterDeviceNotification(HDEVNOTIFY hnd)
  */
 DWORD WINAPI GetAppCompatFlags( HTASK hTask )
 {
-    FIXME("stub\n");
+    FIXME("(%p) stub\n", hTask);
+    return 0;
+}
+
+/***********************************************************************
+ *           GetAppCompatFlags2   (USER32.@)
+ */
+DWORD WINAPI GetAppCompatFlags2( HTASK hTask )
+{
+    FIXME("(%p) stub\n", hTask);
     return 0;
 }
 
@@ -603,26 +580,6 @@ VOID WINAPI LoadLocalFonts(VOID)
 }
 
 
-/***********************************************************************
- *		USER_489 (USER.489)
- */
-LONG WINAPI stub_USER_489(void) { FIXME("stub\n"); return 0; }
-
-/***********************************************************************
- *		USER_490 (USER.490)
- */
-LONG WINAPI stub_USER_490(void) { FIXME("stub\n"); return 0; }
-
-/***********************************************************************
- *		USER_492 (USER.492)
- */
-LONG WINAPI stub_USER_492(void) { FIXME("stub\n"); return 0; }
-
-/***********************************************************************
- *		USER_496 (USER.496)
- */
-LONG WINAPI stub_USER_496(void) { FIXME("stub\n"); return 0; }
-
 /***********************************************************************
  *		User32InitializeImmEntryTable
  */
@@ -683,3 +640,14 @@ LRESULT WINAPI SendIMEMessageExW(HWND p1, LPARAM p2)
   SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
   return 0;
 }
+
+/**********************************************************************
+ * DisableProcessWindowsGhosting [USER32.@]
+ *
+ */
+VOID WINAPI DisableProcessWindowsGhosting(VOID)
+{
+  FIXME(": stub\n");
+  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+  return;
+}
