@@ -2,6 +2,7 @@
  * Wine Driver for ALSA
  *
  * Copyright	2002 Eric Pouech
+ * Copyright	2006 Jaroslav Kysela
  * Copyright	2007 Maarten Lankhorst
  *
  * This file has a few shared generic subroutines shared among the alsa
@@ -126,7 +127,7 @@ void ALSA_WaitRingMessage(ALSA_MSG_RING* omr, DWORD sleep)
  */
 int ALSA_AddRingMessage(ALSA_MSG_RING* omr, enum win_wm_message msg, DWORD_PTR param, BOOL wait)
 {
-    HANDLE	hEvent = INVALID_HANDLE_VALUE;
+    HANDLE	hEvent = NULL;
 
     EnterCriticalSection(&omr->msg_crst);
     if ((omr->msg_toget == ((omr->msg_tosave + 1) % omr->ring_buffer_size)))
@@ -150,7 +151,7 @@ int ALSA_AddRingMessage(ALSA_MSG_RING* omr, enum win_wm_message msg, DWORD_PTR p
     if (wait)
     {
         hEvent = CreateEventW(NULL, FALSE, FALSE, NULL);
-        if (hEvent == INVALID_HANDLE_VALUE)
+        if (!hEvent)
         {
             ERR("can't create event !?\n");
             LeaveCriticalSection(&omr->msg_crst);
@@ -172,7 +173,7 @@ int ALSA_AddRingMessage(ALSA_MSG_RING* omr, enum win_wm_message msg, DWORD_PTR p
     {
         omr->messages[omr->msg_tosave].msg = msg;
         omr->messages[omr->msg_tosave].param = param;
-        omr->messages[omr->msg_tosave].hEvent = INVALID_HANDLE_VALUE;
+        omr->messages[omr->msg_tosave].hEvent = NULL;
         omr->msg_tosave = (omr->msg_tosave + 1) % omr->ring_buffer_size;
     }
     LeaveCriticalSection(&omr->msg_crst);
@@ -220,7 +221,6 @@ int ALSA_RetrieveRingMessage(ALSA_MSG_RING* omr, enum win_wm_message *msg,
 /* These strings used only for tracing */
 const char * ALSA_getCmdString(enum win_wm_message msg)
 {
-    static char unknown[32];
 #define MSG_TO_STR(x) case x: return #x
     switch(msg) {
     MSG_TO_STR(WINE_WM_PAUSING);
@@ -234,13 +234,11 @@ const char * ALSA_getCmdString(enum win_wm_message msg)
     MSG_TO_STR(WINE_WM_STOPPING);
     }
 #undef MSG_TO_STR
-    sprintf(unknown, "UNKNOWN(0x%08x)", msg);
-    return unknown;
+    return wine_dbg_sprintf("UNKNOWN(0x%08x)", msg);
 }
 
 const char * ALSA_getMessage(UINT msg)
 {
-    static char unknown[32];
 #define MSG_TO_STR(x) case x: return #x
     switch(msg) {
     MSG_TO_STR(DRVM_INIT);
@@ -282,13 +280,11 @@ const char * ALSA_getMessage(UINT msg)
     MSG_TO_STR(DRV_QUERYDSOUNDDESC);
     }
 #undef MSG_TO_STR
-    sprintf(unknown, "UNKNOWN(0x%04x)", msg);
-    return unknown;
+    return wine_dbg_sprintf("UNKNOWN(0x%04x)", msg);
 }
 
 const char * ALSA_getFormat(WORD wFormatTag)
 {
-    static char unknown[32];
 #define FMT_TO_STR(x) case x: return #x
     switch(wFormatTag) {
     FMT_TO_STR(WAVE_FORMAT_PCM);
@@ -298,8 +294,7 @@ const char * ALSA_getFormat(WORD wFormatTag)
     FMT_TO_STR(WAVE_FORMAT_ADPCM);
     }
 #undef FMT_TO_STR
-    sprintf(unknown, "UNKNOWN(0x%04x)", wFormatTag);
-    return unknown;
+    return wine_dbg_sprintf("UNKNOWN(0x%04x)", wFormatTag);
 }
 
 /* Allow 1% deviation for sample rates (some ES137x cards) */
@@ -356,17 +351,13 @@ void ALSA_copyFormat(LPWAVEFORMATEX wf1, LPWAVEFORMATPCMEX wf2)
 {
     unsigned int iLength;
 
-    ZeroMemory(wf2, sizeof(wf2));
+    ZeroMemory(wf2, sizeof(*wf2));
     if (wf1->wFormatTag == WAVE_FORMAT_PCM)
         iLength = sizeof(PCMWAVEFORMAT);
     else if (wf1->wFormatTag == WAVE_FORMAT_EXTENSIBLE)
         iLength = sizeof(WAVEFORMATPCMEX);
     else
         iLength = sizeof(WAVEFORMATEX) + wf1->cbSize;
-    if (iLength > sizeof(WAVEFORMATPCMEX)) {
-        ERR("calculated %u bytes, capping\n", iLength);
-        iLength = sizeof(WAVEFORMATPCMEX);
-    }
     memcpy(wf2, wf1, iLength);
 }
 
@@ -400,17 +391,6 @@ BOOL ALSA_supportedFormat(LPWAVEFORMATEX wf)
         } else
             WARN("only KSDATAFORMAT_SUBTYPE_PCM and KSDATAFORMAT_SUBTYPE_IEEE_FLOAT "
                  "supported\n");
-    } else if (wf->wFormatTag == WAVE_FORMAT_MULAW || wf->wFormatTag == WAVE_FORMAT_ALAW) {
-        if (wf->wBitsPerSample==8)
-            return TRUE;
-        else
-            ERR("WAVE_FORMAT_MULAW and WAVE_FORMAT_ALAW wBitsPerSample must = 8\n");
-
-    } else if (wf->wFormatTag == WAVE_FORMAT_ADPCM) {
-        if (wf->wBitsPerSample==4)
-            return TRUE;
-        else
-            ERR("WAVE_FORMAT_ADPCM wBitsPerSample must = 4\n");
     } else
         WARN("only WAVE_FORMAT_PCM and WAVE_FORMAT_EXTENSIBLE supported\n");
 
@@ -441,6 +421,8 @@ int ALSA_CheckSetVolume(snd_hctl_t *hctl, int *out_left, int *out_right,
     snd_ctl_elem_info_t *       eleminfop = NULL;
     snd_ctl_elem_value_t *      elemvaluep = NULL;
     snd_ctl_elem_id_t *         elemidp = NULL;
+    const char *names[] = {"PCM Playback Volume", "Line Playback Volume", NULL};
+    const char **name;
 
 
 #define EXIT_ON_ERROR(f,txt,exitcode) do \
@@ -467,81 +449,88 @@ int ALSA_CheckSetVolume(snd_hctl_t *hctl, int *out_left, int *out_right,
 
     /* Setup and find an element id that exactly matches the characteristic we want
     ** FIXME:  It is probably short sighted to hard code and fixate on PCM Playback Volume */
-    snd_ctl_elem_id_set_name(elemidp, "PCM Playback Volume");
-    snd_ctl_elem_id_set_interface(elemidp, SND_CTL_ELEM_IFACE_MIXER);
-    elem = snd_hctl_find_elem(hctl, elemidp);
-    if (elem)
+
+    for( name = names; *name; name++ )
     {
-        /* Read and return volume information */
-        EXIT_ON_ERROR(snd_hctl_elem_info(elem, eleminfop), "snd_hctl_elem_info", MMSYSERR_NOTSUPPORTED);
-        value_count = snd_ctl_elem_info_get_count(eleminfop);
-        if (out_min || out_max || out_step)
-        {
-	    if (!snd_ctl_elem_info_is_readable(eleminfop))
-            {
-                ERR("snd_ctl_elem_info_is_readable returned false; cannot return info\n");
-                rc = MMSYSERR_NOTSUPPORTED;
-                goto out;
-            }
+	snd_ctl_elem_id_set_name(elemidp, *name);
+	snd_ctl_elem_id_set_interface(elemidp, SND_CTL_ELEM_IFACE_MIXER);
+	elem = snd_hctl_find_elem(hctl, elemidp);
+	if (elem)
+	{
+	    /* Read and return volume information */
+	    EXIT_ON_ERROR(snd_hctl_elem_info(elem, eleminfop), "snd_hctl_elem_info", MMSYSERR_NOTSUPPORTED);
+	    value_count = snd_ctl_elem_info_get_count(eleminfop);
+	    if (out_min || out_max || out_step)
+	    {
+		if (!snd_ctl_elem_info_is_readable(eleminfop))
+		{
+		    ERR("snd_ctl_elem_info_is_readable returned false; cannot return info\n");
+		    rc = MMSYSERR_NOTSUPPORTED;
+		    goto out;
+		}
 
-            if (out_min)
-                *out_min = snd_ctl_elem_info_get_min(eleminfop);
+		if (out_min)
+		    *out_min = snd_ctl_elem_info_get_min(eleminfop);
 
-            if (out_max)
-                *out_max = snd_ctl_elem_info_get_max(eleminfop);
+		if (out_max)
+		    *out_max = snd_ctl_elem_info_get_max(eleminfop);
 
-            if (out_step)
-                *out_step = snd_ctl_elem_info_get_step(eleminfop);
-        }
+		if (out_step)
+		    *out_step = snd_ctl_elem_info_get_step(eleminfop);
+	    }
 
-        if (out_left || out_right)
-        {
-            EXIT_ON_ERROR(snd_hctl_elem_read(elem, elemvaluep), "snd_hctl_elem_read", MMSYSERR_NOTSUPPORTED);
+	    if (out_left || out_right)
+	    {
+		EXIT_ON_ERROR(snd_hctl_elem_read(elem, elemvaluep), "snd_hctl_elem_read", MMSYSERR_NOTSUPPORTED);
 
-            if (out_left)
-                *out_left = snd_ctl_elem_value_get_integer(elemvaluep, 0);
+		if (out_left)
+		    *out_left = snd_ctl_elem_value_get_integer(elemvaluep, 0);
 
-            if (out_right)
-            {
-                if (value_count == 1)
-                    *out_right = snd_ctl_elem_value_get_integer(elemvaluep, 0);
-                else if (value_count == 2)
-                    *out_right = snd_ctl_elem_value_get_integer(elemvaluep, 1);
-                else
-                {
-                    ERR("Unexpected value count %d from snd_ctl_elem_info_get_count while getting volume info\n", value_count);
-                    rc = -1;
-                    goto out;
-                }
-            }
-        }
+		if (out_right)
+		{
+		    if (value_count == 1)
+			*out_right = snd_ctl_elem_value_get_integer(elemvaluep, 0);
+		    else if (value_count == 2)
+			*out_right = snd_ctl_elem_value_get_integer(elemvaluep, 1);
+		    else
+		    {
+			ERR("Unexpected value count %d from snd_ctl_elem_info_get_count while getting volume info\n", value_count);
+			rc = -1;
+			goto out;
+		    }
+		}
+	    }
 
-        /* Set the volume */
-        if (new_left || new_right)
-        {
-            EXIT_ON_ERROR(snd_hctl_elem_read(elem, elemvaluep), "snd_hctl_elem_read", MMSYSERR_NOTSUPPORTED);
-            if (new_left)
-	        snd_ctl_elem_value_set_integer(elemvaluep, 0, *new_left);
-            if (new_right)
-            {
-                if (value_count == 1)
-	            snd_ctl_elem_value_set_integer(elemvaluep, 0, *new_right);
-                else if (value_count == 2)
-	            snd_ctl_elem_value_set_integer(elemvaluep, 1, *new_right);
-                else
-                {
-                    ERR("Unexpected value count %d from snd_ctl_elem_info_get_count while setting volume info\n", value_count);
-                    rc = -1;
-                    goto out;
-                }
-            }
+	    /* Set the volume */
+	    if (new_left || new_right)
+	    {
+		EXIT_ON_ERROR(snd_hctl_elem_read(elem, elemvaluep), "snd_hctl_elem_read", MMSYSERR_NOTSUPPORTED);
+		if (new_left)
+		    snd_ctl_elem_value_set_integer(elemvaluep, 0, *new_left);
+		if (new_right)
+		{
+		    if (value_count == 1)
+			snd_ctl_elem_value_set_integer(elemvaluep, 0, *new_right);
+		    else if (value_count == 2)
+			snd_ctl_elem_value_set_integer(elemvaluep, 1, *new_right);
+		    else
+		    {
+			ERR("Unexpected value count %d from snd_ctl_elem_info_get_count while setting volume info\n", value_count);
+			rc = -1;
+			goto out;
+		    }
+		}
 
-            EXIT_ON_ERROR(snd_hctl_elem_write(elem, elemvaluep), "snd_hctl_elem_write", MMSYSERR_NOTSUPPORTED);
-        }
+		EXIT_ON_ERROR(snd_hctl_elem_write(elem, elemvaluep), "snd_hctl_elem_write", MMSYSERR_NOTSUPPORTED);
+	    }
+
+	    break;
+	}
     }
-    else
+
+    if( !*name )
     {
-        ERR("Could not find 'PCM Playback Volume' element\n");
+        ERR("Could not find '{PCM,Line} Playback Volume' element\n");
         rc = MMSYSERR_NOTSUPPORTED;
     }
 
@@ -562,24 +551,42 @@ out:
 
 
 /**************************************************************************
- * 			ALSA_XRUNRecovery		[internal]
+ * 			wine_snd_pcm_recover		[internal]
  *
- * used to recovery from XRUN errors (buffer underflow/overflow)
+ * Code slightly modified from alsa-lib v1.0.23 snd_pcm_recover implementation.
+ * used to recover from XRUN errors (buffer underflow/overflow)
  */
-int ALSA_XRUNRecovery(WINE_WAVEDEV * wwo, int err)
+int wine_snd_pcm_recover(snd_pcm_t *pcm, int err, int silent)
 {
-    if (err == -EPIPE) {    /* under-run */
-        err = snd_pcm_prepare(wwo->pcm);
-        if (err < 0)
-             ERR( "underrun recovery failed. prepare failed: %s\n", snd_strerror(err));
+    if (err > 0)
+        err = -err;
+    if (err == -EINTR)	/* nothing to do, continue */
         return 0;
-    } else if (err == -ESTRPIPE) {
-        while ((err = snd_pcm_resume(wwo->pcm)) == -EAGAIN)
-            sleep(1);       /* wait until the suspend flag is released */
+    if (err == -EPIPE) {
+        const char *s;
+        if (snd_pcm_stream(pcm) == SND_PCM_STREAM_PLAYBACK)
+            s = "underrun";
+        else
+            s = "overrun";
+        if (!silent)
+            ERR("%s occurred\n", s);
+        err = snd_pcm_prepare(pcm);
         if (err < 0) {
-            err = snd_pcm_prepare(wwo->pcm);
-            if (err < 0)
-                ERR("recovery from suspend failed, prepare failed: %s\n", snd_strerror(err));
+            ERR("cannot recover from %s, prepare failed: %s\n", s, snd_strerror(err));
+            return err;
+        }
+        return 0;
+    }
+    if (err == -ESTRPIPE) {
+        while ((err = snd_pcm_resume(pcm)) == -EAGAIN)
+            /* wait until suspend flag is released */
+            poll(NULL, 0, 1000);
+        if (err < 0) {
+            err = snd_pcm_prepare(pcm);
+            if (err < 0) {
+                ERR("cannot recover from suspend, prepare failed: %s\n", snd_strerror(err));
+                return err;
+            }
         }
         return 0;
     }
@@ -731,20 +738,20 @@ LRESULT CALLBACK ALSA_DriverProc(DWORD_PTR dwDevID, HDRVR hDriv, UINT wMsg,
 
     switch(wMsg) {
 #ifdef HAVE_ALSA
-    case DRV_LOAD:		ALSA_WaveInit();
-				ALSA_MidiInit();
-				return 1;
-    case DRV_FREE:		return 1;
-    case DRV_OPEN:		return 1;
-    case DRV_CLOSE:		return 1;
-    case DRV_ENABLE:		return 1;
-    case DRV_DISABLE:		return 1;
-    case DRV_QUERYCONFIGURE:	return 1;
+    case DRV_LOAD:
+    case DRV_FREE:
+    case DRV_OPEN:
+    case DRV_CLOSE:
+    case DRV_ENABLE:
+    case DRV_DISABLE:
+    case DRV_QUERYCONFIGURE:
+        return 1;
     case DRV_CONFIGURE:		MessageBoxA(0, "ALSA MultiMedia Driver !", "ALSA Driver", MB_OK);	return 1;
-    case DRV_INSTALL:		return DRVCNF_RESTART;
-    case DRV_REMOVE:		return DRVCNF_RESTART;
+    case DRV_INSTALL:
+    case DRV_REMOVE:
+        return DRV_SUCCESS;
 #endif
     default:
-	return DefDriverProc(dwDevID, hDriv, wMsg, dwParam1, dwParam2);
+	return 0;
     }
 }
