@@ -51,11 +51,6 @@
 #include "winuser.h"
 #include "winnls.h"
 #include "mmddk.h"
-#include "mmreg.h"
-#include "dsound.h"
-#include "dsdriver.h"
-#include "ks.h"
-#include "ksmedia.h"
 
 #include "alsa.h"
 #include "wine/library.h"
@@ -64,6 +59,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(wave);
 
+#ifdef HAVE_ALSA
+
 WINE_WAVEDEV	*WInDev;
 DWORD            ALSA_WidNumMallocedDevs;
 DWORD            ALSA_WidNumDevs;
@@ -79,8 +76,11 @@ static void widNotifyClient(WINE_WAVEDEV* wwi, WORD wMsg, DWORD_PTR dwParam1, DW
    case WIM_OPEN:
    case WIM_CLOSE:
    case WIM_DATA:
-       DriverCallback(wwi->waveDesc.dwCallback, wwi->wFlags, (HDRVR)wwi->waveDesc.hWave,
-                      wMsg, wwi->waveDesc.dwInstance, dwParam1, dwParam2);
+       if (wwi->wFlags != DCB_NULL &&
+	   !DriverCallback(wwi->waveDesc.dwCallback, wwi->wFlags, (HDRVR)wwi->waveDesc.hWave,
+			   wMsg, wwi->waveDesc.dwInstance, dwParam1, dwParam2)) {
+	   WARN("can't notify client !\n");
+       }
        break;
    default:
        FIXME("Unknown callback message %u\n", wMsg);
@@ -778,3 +778,17 @@ DWORD WINAPI ALSA_widMessage(UINT wDevID, UINT wMsg, DWORD_PTR dwUser,
     }
     return MMSYSERR_NOTSUPPORTED;
 }
+
+#else /* HAVE_ALSA */
+
+/**************************************************************************
+ * 				widMessage (WINEALSA.@)
+ */
+DWORD WINAPI ALSA_widMessage(WORD wDevID, WORD wMsg, DWORD_PTR dwUser,
+                             DWORD_PTR dwParam1, DWORD_PTR dwParam2)
+{
+    FIXME("(%u, %04X, %08lX, %08lX, %08lX):stub\n", wDevID, wMsg, dwUser, dwParam1, dwParam2);
+    return MMSYSERR_NOTENABLED;
+}
+
+#endif /* HAVE_ALSA */
