@@ -50,16 +50,18 @@
 #include "dsound.h"
 #include "dsdriver.h"
 #include "ks.h"
+#include "ksguid.h"
+#include "ksmedia.h"
 #include "coreaudio.h"
 #include "wine/unicode.h"
 #include "wine/library.h"
 #include "wine/debug.h"
 #include "wine/list.h"
 
-#include "initguid.h"
-#include "ksmedia.h"
-
 WINE_DEFAULT_DEBUG_CHANNEL(wave);
+
+#if defined(HAVE_COREAUDIO_COREAUDIO_H) && defined(HAVE_AUDIOUNIT_AUDIOUNIT_H)
+
 WINE_DECLARE_DEBUG_CHANNEL(coreaudio);
 
 /*
@@ -793,9 +795,13 @@ static void wodNotifyClient(WINE_WAVEOUT_INSTANCE* wwo, WORD wMsg, DWORD_PTR dwP
         case WOM_OPEN:
         case WOM_CLOSE:
         case WOM_DONE:
-            DriverCallback(wwo->waveDesc.dwCallback, wwo->wFlags,
-                           (HDRVR)wwo->waveDesc.hWave, wMsg, wwo->waveDesc.dwInstance,
-                           dwParam1, dwParam2);
+            if (wwo->wFlags != DCB_NULL &&
+                !DriverCallback(wwo->waveDesc.dwCallback, wwo->wFlags,
+                                (HDRVR)wwo->waveDesc.hWave, wMsg, wwo->waveDesc.dwInstance,
+                                dwParam1, dwParam2))
+            {
+                WARN("can't notify client !\n");
+            }
             break;
         default:
             FIXME("Unknown callback message %u\n", wMsg);
@@ -1749,9 +1755,13 @@ static void widNotifyClient(WINE_WAVEIN* wwi, WORD wMsg, DWORD_PTR dwParam1, DWO
         case WIM_OPEN:
         case WIM_CLOSE:
         case WIM_DATA:
-            DriverCallback(wwi->waveDesc.dwCallback, wwi->wFlags,
-                           (HDRVR)wwi->waveDesc.hWave, wMsg, wwi->waveDesc.dwInstance,
-                           dwParam1, dwParam2);
+            if (wwi->wFlags != DCB_NULL &&
+                !DriverCallback(wwi->waveDesc.dwCallback, wwi->wFlags,
+                                (HDRVR)wwi->waveDesc.hWave, wMsg, wwi->waveDesc.dwInstance,
+                                dwParam1, dwParam2))
+            {
+                WARN("can't notify client !\n");
+            }
             break;
         default:
             FIXME("Unknown callback message %u\n", wMsg);
@@ -2537,3 +2547,27 @@ OSStatus CoreAudio_wiAudioUnitIOProc(void *inRefCon,
     if (needNotify) wodSendNotifyInputCompletionsMessage(wwi);
     return err;
 }
+
+#else
+
+/**************************************************************************
+ *                              widMessage (WINECOREAUDIO.6)
+ */
+DWORD WINAPI CoreAudio_widMessage(WORD wDevID, WORD wMsg, DWORD dwUser,
+                            DWORD dwParam1, DWORD dwParam2)
+{
+    FIXME("(%u, %04X, %08X, %08X, %08X): CoreAudio support not compiled into wine\n", wDevID, wMsg, dwUser, dwParam1, dwParam2);
+    return MMSYSERR_NOTENABLED;
+}
+
+/**************************************************************************
+* 				wodMessage (WINECOREAUDIO.7)
+*/
+DWORD WINAPI CoreAudio_wodMessage(WORD wDevID, WORD wMsg, DWORD dwUser, 
+                                  DWORD dwParam1, DWORD dwParam2)
+{
+    FIXME("(%u, %04X, %08X, %08X, %08X): CoreAudio support not compiled into wine\n", wDevID, wMsg, dwUser, dwParam1, dwParam2);
+    return MMSYSERR_NOTENABLED;
+}
+
+#endif
