@@ -34,8 +34,9 @@ HRESULT basetexture_init(IWineD3DBaseTextureImpl *texture, const struct wined3d_
 {
     HRESULT hr;
 
-    hr = resource_init(&texture->resource, resource_type, device,
-            0, usage, format, pool, parent, parent_ops, resource_ops);
+    hr = resource_init(&texture->resource, device, resource_type, format,
+            WINED3DMULTISAMPLE_NONE, 0, usage, pool, 0, 0, 0, 0,
+            parent, parent_ops, resource_ops);
     if (FAILED(hr))
     {
         WARN("Failed to initialize resource, returning %#x\n", hr);
@@ -77,6 +78,19 @@ HRESULT basetexture_init(IWineD3DBaseTextureImpl *texture, const struct wined3d_
 
 void basetexture_cleanup(IWineD3DBaseTextureImpl *texture)
 {
+    UINT sub_count = texture->baseTexture.level_count * texture->baseTexture.layer_count;
+    UINT i;
+
+    TRACE("texture %p.\n", texture);
+
+    for (i = 0; i < sub_count; ++i)
+    {
+        struct wined3d_resource *sub_resource = texture->baseTexture.sub_resources[i];
+
+        if (sub_resource)
+            texture->baseTexture.texture_ops->texture_sub_resource_cleanup(sub_resource);
+    }
+
     basetexture_unload(texture);
     HeapFree(GetProcessHeap(), 0, texture->baseTexture.sub_resources);
     resource_cleanup(&texture->resource);
@@ -337,6 +351,25 @@ HRESULT basetexture_bind(IWineD3DBaseTextureImpl *texture,
     return hr;
 }
 
+HRESULT basetexture_add_dirty_region(IWineD3DBaseTextureImpl *texture,
+        UINT layer, const WINED3DBOX *dirty_region)
+{
+    struct wined3d_resource *sub_resource;
+
+    TRACE("texture %p, layer %u, dirty_region %p.\n", texture, layer, dirty_region);
+
+    if (!(sub_resource = basetexture_get_sub_resource(texture, layer * texture->baseTexture.level_count)))
+    {
+        WARN("Failed to get sub-resource.\n");
+        return WINED3DERR_INVALIDCALL;
+    }
+
+    basetexture_set_dirty(texture, TRUE);
+    texture->baseTexture.texture_ops->texture_sub_resource_add_dirty_region(sub_resource, dirty_region);
+
+    return WINED3D_OK;
+}
+
 /* GL locking is done by the caller */
 static void apply_wrap(const struct wined3d_gl_info *gl_info, GLenum target,
         WINED3DTEXTUREADDRESS d3d_wrap, GLenum param, BOOL cond_np2)
@@ -370,7 +403,7 @@ void basetexture_apply_state_changes(IWineD3DBaseTextureImpl *texture,
         const DWORD samplerStates[WINED3D_HIGHEST_SAMPLER_STATE + 1],
         const struct wined3d_gl_info *gl_info)
 {
-    BOOL cond_np2 = IWineD3DBaseTexture_IsCondNP2((IWineD3DBaseTexture *)texture);
+    BOOL cond_np2 = texture->baseTexture.cond_np2;
     GLenum textureDimensions = texture->baseTexture.target;
     DWORD state;
     DWORD aniso;
