@@ -399,6 +399,16 @@ static struct fbo_entry *context_find_fbo_entry(struct wined3d_context *context,
     const struct wined3d_gl_info *gl_info = context->gl_info;
     struct fbo_entry *entry;
 
+    if (depth_stencil && render_targets && render_targets[0])
+    {
+        if (depth_stencil->resource.width < render_targets[0]->resource.width ||
+            depth_stencil->resource.height < render_targets[0]->resource.height)
+        {
+            WARN("Depth stencil is smaller than the primary color buffer, disabling\n");
+            depth_stencil = NULL;
+        }
+    }
+
     LIST_FOR_EACH_ENTRY(entry, &context->fbo_list, struct fbo_entry, entry)
     {
         if (!memcmp(entry->render_targets,
@@ -1663,8 +1673,8 @@ static void SetupForBlit(IWineD3DDeviceImpl *This, struct wined3d_context *conte
     int i;
     const struct StateEntry *StateTable = This->StateTable;
     const struct wined3d_gl_info *gl_info = context->gl_info;
-    UINT width = context->current_rt->currentDesc.Width;
-    UINT height = context->current_rt->currentDesc.Height;
+    UINT width = context->current_rt->resource.width;
+    UINT height = context->current_rt->resource.height;
     DWORD sampler;
 
     TRACE("Setting up context %p for blitting\n", context);
@@ -1970,7 +1980,6 @@ static inline void context_set_render_offscreen(struct wined3d_context *context,
 
     Context_MarkStateDirty(context, STATE_POINTSPRITECOORDORIGIN, StateTable);
     Context_MarkStateDirty(context, STATE_TRANSFORM(WINED3DTS_PROJECTION), StateTable);
-    Context_MarkStateDirty(context, STATE_VDECL, StateTable);
     Context_MarkStateDirty(context, STATE_VIEWPORT, StateTable);
     Context_MarkStateDirty(context, STATE_SCISSORRECT, StateTable);
     Context_MarkStateDirty(context, STATE_FRONTFACE, StateTable);
