@@ -193,7 +193,7 @@ static const DWORD vertex_states_sampler[] =
  */
 static HRESULT stateblock_allocate_shader_constants(struct wined3d_stateblock *object)
 {
-    struct wined3d_device *device = object->device;
+    IWineD3DDeviceImpl *device = object->device;
 
     /* Allocate space for floating point constants */
     object->state.ps_consts_f = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
@@ -329,7 +329,7 @@ static void stateblock_savedstates_set_vertex(SAVEDSTATES *states, const DWORD n
 
 void stateblock_init_contained_states(struct wined3d_stateblock *stateblock)
 {
-    struct wined3d_device *device = stateblock->device;
+    IWineD3DDeviceImpl *device = stateblock->device;
     unsigned int i, j;
 
     for (i = 0; i <= WINEHIGHEST_RENDER_STATE >> 5; ++i)
@@ -474,46 +474,34 @@ ULONG CDECL wined3d_stateblock_decref(struct wined3d_stateblock *stateblock)
 
     if (!refcount)
     {
-        struct wined3d_buffer *buffer;
         int counter;
 
         if (stateblock->state.vertex_declaration)
-            wined3d_vertex_declaration_decref(stateblock->state.vertex_declaration);
+            wined3d_vertex_declaration_incref(stateblock->state.vertex_declaration);
 
         for (counter = 0; counter < MAX_COMBINED_SAMPLERS; counter++)
         {
-            struct wined3d_texture *texture = stateblock->state.textures[counter];
-            if (texture)
-            {
-                stateblock->state.textures[counter] = NULL;
-                wined3d_texture_decref(texture);
-            }
+            if (stateblock->state.textures[counter])
+                IWineD3DBaseTexture_Release((IWineD3DBaseTexture *)stateblock->state.textures[counter]);
         }
 
         for (counter = 0; counter < MAX_STREAMS; ++counter)
         {
-            buffer = stateblock->state.streams[counter].buffer;
+            struct wined3d_buffer *buffer = stateblock->state.streams[counter].buffer;
             if (buffer)
             {
-                stateblock->state.streams[counter].buffer = NULL;
                 if (wined3d_buffer_decref(buffer))
                 {
                     WARN("Buffer %p still referenced by stateblock, stream %u.\n", buffer, counter);
                 }
             }
         }
-
-        buffer = stateblock->state.index_buffer;
-        if (buffer)
-        {
-            stateblock->state.index_buffer = NULL;
-            wined3d_buffer_decref(buffer);
-        }
-
+        if (stateblock->state.index_buffer)
+            wined3d_buffer_decref(stateblock->state.index_buffer);
         if (stateblock->state.vertex_shader)
-            wined3d_shader_decref(stateblock->state.vertex_shader);
+            IWineD3DVertexShader_Release((IWineD3DVertexShader *)stateblock->state.vertex_shader);
         if (stateblock->state.pixel_shader)
-            wined3d_shader_decref(stateblock->state.pixel_shader);
+            IWineD3DPixelShader_Release((IWineD3DPixelShader *)stateblock->state.pixel_shader);
 
         for (counter = 0; counter < LIGHTMAP_SIZE; ++counter)
         {
@@ -611,9 +599,9 @@ HRESULT CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock)
                 stateblock->state.vertex_shader, src_state->vertex_shader);
 
         if (src_state->vertex_shader)
-            wined3d_shader_incref(src_state->vertex_shader);
+            IWineD3DVertexShader_AddRef((IWineD3DVertexShader *)src_state->vertex_shader);
         if (stateblock->state.vertex_shader)
-            wined3d_shader_decref(stateblock->state.vertex_shader);
+            IWineD3DVertexShader_Release((IWineD3DVertexShader *)stateblock->state.vertex_shader);
         stateblock->state.vertex_shader = src_state->vertex_shader;
     }
 
@@ -852,9 +840,9 @@ HRESULT CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock)
                 i, src_state->textures[i], stateblock->state.textures[i]);
 
         if (src_state->textures[i])
-            wined3d_texture_incref(src_state->textures[i]);
+            IWineD3DBaseTexture_AddRef((IWineD3DBaseTexture *)src_state->textures[i]);
         if (stateblock->state.textures[i])
-            wined3d_texture_decref(stateblock->state.textures[i]);
+            IWineD3DBaseTexture_Release((IWineD3DBaseTexture *)stateblock->state.textures[i]);
         stateblock->state.textures[i] = src_state->textures[i];
     }
 
@@ -872,9 +860,9 @@ HRESULT CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock)
     if (stateblock->changed.pixelShader && stateblock->state.pixel_shader != src_state->pixel_shader)
     {
         if (src_state->pixel_shader)
-            wined3d_shader_incref(src_state->pixel_shader);
+            IWineD3DPixelShader_AddRef((IWineD3DPixelShader *)src_state->pixel_shader);
         if (stateblock->state.pixel_shader)
-            wined3d_shader_decref(stateblock->state.pixel_shader);
+            IWineD3DPixelShader_Release((IWineD3DPixelShader *)stateblock->state.pixel_shader);
         stateblock->state.pixel_shader = src_state->pixel_shader;
     }
 
@@ -885,7 +873,7 @@ HRESULT CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock)
     return WINED3D_OK;
 }
 
-static void apply_lights(struct wined3d_device *device, const struct wined3d_state *state)
+static void apply_lights(IWineD3DDevice *device, const struct wined3d_state *state)
 {
     UINT i;
 
@@ -897,15 +885,15 @@ static void apply_lights(struct wined3d_device *device, const struct wined3d_sta
         {
             const struct wined3d_light_info *light = LIST_ENTRY(e, struct wined3d_light_info, entry);
 
-            wined3d_device_set_light(device, light->OriginalIndex, &light->OriginalParms);
-            wined3d_device_set_light_enable(device, light->OriginalIndex, light->glIndex != -1);
+            IWineD3DDevice_SetLight(device, light->OriginalIndex, &light->OriginalParms);
+            IWineD3DDevice_SetLightEnable(device, light->OriginalIndex, light->glIndex != -1);
         }
     }
 }
 
 HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblock)
 {
-    struct wined3d_device *device = stateblock->device;
+    IWineD3DDevice *device = (IWineD3DDevice *)stateblock->device;
     unsigned int i;
     DWORD map;
 
@@ -913,51 +901,51 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
     TRACE("Blocktype: %#x.\n", stateblock->blockType);
 
     if (stateblock->changed.vertexShader)
-        wined3d_device_set_vertex_shader(device, stateblock->state.vertex_shader);
+        IWineD3DDevice_SetVertexShader(device, (IWineD3DVertexShader *)stateblock->state.vertex_shader);
 
     /* Vertex Shader Constants. */
     for (i = 0; i < stateblock->num_contained_vs_consts_f; ++i)
     {
-        wined3d_device_set_vs_consts_f(device, stateblock->contained_vs_consts_f[i],
+        IWineD3DDevice_SetVertexShaderConstantF(device, stateblock->contained_vs_consts_f[i],
                 stateblock->state.vs_consts_f + stateblock->contained_vs_consts_f[i] * 4, 1);
     }
     for (i = 0; i < stateblock->num_contained_vs_consts_i; ++i)
     {
-        wined3d_device_set_vs_consts_i(device, stateblock->contained_vs_consts_i[i],
+        IWineD3DDevice_SetVertexShaderConstantI(device, stateblock->contained_vs_consts_i[i],
                 stateblock->state.vs_consts_i + stateblock->contained_vs_consts_i[i] * 4, 1);
     }
     for (i = 0; i < stateblock->num_contained_vs_consts_b; ++i)
     {
-        wined3d_device_set_vs_consts_b(device, stateblock->contained_vs_consts_b[i],
+        IWineD3DDevice_SetVertexShaderConstantB(device, stateblock->contained_vs_consts_b[i],
                 stateblock->state.vs_consts_b + stateblock->contained_vs_consts_b[i], 1);
     }
 
     apply_lights(device, &stateblock->state);
 
     if (stateblock->changed.pixelShader)
-        wined3d_device_set_pixel_shader(device, stateblock->state.pixel_shader);
+        IWineD3DDevice_SetPixelShader(device, (IWineD3DPixelShader *)stateblock->state.pixel_shader);
 
     /* Pixel Shader Constants. */
     for (i = 0; i < stateblock->num_contained_ps_consts_f; ++i)
     {
-        wined3d_device_set_ps_consts_f(device, stateblock->contained_ps_consts_f[i],
+        IWineD3DDevice_SetPixelShaderConstantF(device, stateblock->contained_ps_consts_f[i],
                 stateblock->state.ps_consts_f + stateblock->contained_ps_consts_f[i] * 4, 1);
     }
     for (i = 0; i < stateblock->num_contained_ps_consts_i; ++i)
     {
-        wined3d_device_set_ps_consts_i(device, stateblock->contained_ps_consts_i[i],
+        IWineD3DDevice_SetPixelShaderConstantI(device, stateblock->contained_ps_consts_i[i],
                 stateblock->state.ps_consts_i + stateblock->contained_ps_consts_i[i] * 4, 1);
     }
     for (i = 0; i < stateblock->num_contained_ps_consts_b; ++i)
     {
-        wined3d_device_set_ps_consts_b(device, stateblock->contained_ps_consts_b[i],
+        IWineD3DDevice_SetPixelShaderConstantB(device, stateblock->contained_ps_consts_b[i],
                 stateblock->state.ps_consts_b + stateblock->contained_ps_consts_b[i], 1);
     }
 
     /* Render states. */
     for (i = 0; i < stateblock->num_contained_render_states; ++i)
     {
-        wined3d_device_set_render_state(device, stateblock->contained_render_states[i],
+        IWineD3DDevice_SetRenderState(device, stateblock->contained_render_states[i],
                 stateblock->state.render_states[stateblock->contained_render_states[i]]);
     }
 
@@ -967,7 +955,7 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
         DWORD stage = stateblock->contained_tss_states[i].stage;
         DWORD state = stateblock->contained_tss_states[i].state;
 
-        wined3d_device_set_texture_stage_state(device, stage, state, stateblock->state.texture_states[stage][state]);
+        IWineD3DDevice_SetTextureStageState(device, stage, state, stateblock->state.texture_states[stage][state]);
     }
 
     /* Sampler states. */
@@ -978,13 +966,13 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
         DWORD value = stateblock->state.sampler_states[stage][state];
 
         if (stage >= MAX_FRAGMENT_SAMPLERS) stage += WINED3DVERTEXTEXTURESAMPLER0 - MAX_FRAGMENT_SAMPLERS;
-        wined3d_device_set_sampler_state(device, stage, state, value);
+        IWineD3DDevice_SetSamplerState(device, stage, state, value);
     }
 
     /* Transform states. */
     for (i = 0; i < stateblock->num_contained_transform_states; ++i)
     {
-        wined3d_device_set_transform(device, stateblock->contained_transform_states[i],
+        IWineD3DDevice_SetTransform(device, stateblock->contained_transform_states[i],
                 &stateblock->state.transforms[stateblock->contained_transform_states[i]]);
     }
 
@@ -996,27 +984,35 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
 
     if (stateblock->changed.indices)
     {
-        wined3d_device_set_index_buffer(device, stateblock->state.index_buffer, stateblock->state.index_format);
-        wined3d_device_set_base_vertex_index(device, stateblock->state.base_vertex_index);
+        IWineD3DDevice_SetIndexBuffer(device, stateblock->state.index_buffer, stateblock->state.index_format);
+        IWineD3DDevice_SetBaseVertexIndex(device, stateblock->state.base_vertex_index);
     }
 
     if (stateblock->changed.vertexDecl && stateblock->state.vertex_declaration)
-        wined3d_device_set_vertex_declaration(device, stateblock->state.vertex_declaration);
+    {
+        IWineD3DDevice_SetVertexDeclaration(device, stateblock->state.vertex_declaration);
+    }
 
     if (stateblock->changed.material)
-        wined3d_device_set_material(device, &stateblock->state.material);
+    {
+        IWineD3DDevice_SetMaterial(device, &stateblock->state.material);
+    }
 
     if (stateblock->changed.viewport)
-        wined3d_device_set_viewport(device, &stateblock->state.viewport);
+    {
+        IWineD3DDevice_SetViewport(device, &stateblock->state.viewport);
+    }
 
     if (stateblock->changed.scissorRect)
-        wined3d_device_set_scissor_rect(device, &stateblock->state.scissor_rect);
+    {
+        IWineD3DDevice_SetScissorRect(device, &stateblock->state.scissor_rect);
+    }
 
     map = stateblock->changed.streamSource;
     for (i = 0; map; map >>= 1, ++i)
     {
         if (map & 1)
-            wined3d_device_set_stream_source(device, i,
+            IWineD3DDevice_SetStreamSource(device, i,
                     stateblock->state.streams[i].buffer,
                     0, stateblock->state.streams[i].stride);
     }
@@ -1025,7 +1021,7 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
     for (i = 0; map; map >>= 1, ++i)
     {
         if (map & 1)
-            wined3d_device_set_stream_source_freq(device, i,
+            IWineD3DDevice_SetStreamSourceFreq(device, i,
                     stateblock->state.streams[i].frequency | stateblock->state.streams[i].flags);
     }
 
@@ -1037,7 +1033,7 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
         if (!(map & 1)) continue;
 
         stage = i < MAX_FRAGMENT_SAMPLERS ? i : WINED3DVERTEXTEXTURESAMPLER0 + i - MAX_FRAGMENT_SAMPLERS;
-        wined3d_device_set_texture(device, stage, stateblock->state.textures[i]);
+        IWineD3DDevice_SetTexture(device, stage, (IWineD3DBaseTexture *)stateblock->state.textures[i]);
     }
 
     map = stateblock->changed.clipplane;
@@ -1047,11 +1043,11 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
 
         if (!(map & 1)) continue;
 
-        clip[0] = (float) stateblock->state.clip_planes[i][0];
-        clip[1] = (float) stateblock->state.clip_planes[i][1];
-        clip[2] = (float) stateblock->state.clip_planes[i][2];
-        clip[3] = (float) stateblock->state.clip_planes[i][3];
-        wined3d_device_set_clip_plane(device, i, clip);
+        clip[0] = stateblock->state.clip_planes[i][0];
+        clip[1] = stateblock->state.clip_planes[i][1];
+        clip[2] = stateblock->state.clip_planes[i][2];
+        clip[3] = stateblock->state.clip_planes[i][3];
+        IWineD3DDevice_SetClipPlane(device, i, clip);
     }
 
     stateblock->device->stateBlock->state.lowest_disabled_stage = MAX_TEXTURES - 1;
@@ -1071,7 +1067,7 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
 
 void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
 {
-    struct wined3d_device *device = stateblock->device;
+    IWineD3DDeviceImpl *device = stateblock->device;
     const struct wined3d_gl_info *gl_info = &device->adapter->gl_info;
     struct wined3d_state *state = &stateblock->state;
     union {
@@ -1083,8 +1079,8 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
         DWORD d;
     } tmpfloat;
     unsigned int i;
-    struct wined3d_swapchain *swapchain;
-    struct wined3d_surface *backbuffer;
+    IWineD3DSwapChain *swapchain;
+    IWineD3DSurface *backbuffer;
     HRESULT hr;
 
     TRACE("stateblock %p.\n", stateblock);
@@ -1099,8 +1095,6 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
         memcpy(&state->transforms[WINED3DTS_WORLDMATRIX(i)], identity, sizeof(identity));
     }
 
-    state->fb = &device->fb;
-
     TRACE("Render states\n");
     /* Render states: */
     if (device->auto_depth_stencil)
@@ -1135,6 +1129,7 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
     tmpfloat.f = 1.0f;
     state->render_states[WINED3DRS_FOGDENSITY] = tmpfloat.d;
     state->render_states[WINED3DRS_EDGEANTIALIAS] = FALSE;
+    state->render_states[WINED3DRS_ZBIAS] = 0;
     state->render_states[WINED3DRS_RANGEFOGENABLE] = FALSE;
     state->render_states[WINED3DRS_STENCILENABLE] = FALSE;
     state->render_states[WINED3DRS_STENCILFAIL] = WINED3DSTENCILOP_KEEP;
@@ -1233,6 +1228,10 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
     state->render_states[WINED3DRS_DESTBLENDALPHA] = WINED3DBLEND_ZERO;
     state->render_states[WINED3DRS_BLENDOPALPHA] = WINED3DBLENDOP_ADD;
 
+    /* clipping status */
+    state->clip_status.ClipUnion = 0;
+    state->clip_status.ClipIntersection = 0xFFFFFFFF;
+
     /* Texture Stage States - Put directly into state block, we will call function below */
     for (i = 0; i < MAX_TEXTURES; ++i)
     {
@@ -1285,16 +1284,16 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
     }
 
     /* check the return values, because the GetBackBuffer call isn't valid for ddraw */
-    hr = wined3d_device_get_swapchain(device, 0, &swapchain);
+    hr = IWineD3DDevice_GetSwapChain((IWineD3DDevice *)device, 0, &swapchain);
     if (SUCCEEDED(hr) && swapchain)
     {
-        hr = wined3d_swapchain_get_back_buffer(swapchain, 0, WINED3DBACKBUFFER_TYPE_MONO, &backbuffer);
+        hr = IWineD3DSwapChain_GetBackBuffer(swapchain, 0, WINED3DBACKBUFFER_TYPE_MONO, &backbuffer);
         if (SUCCEEDED(hr) && backbuffer)
         {
             struct wined3d_resource_desc desc;
 
-            wined3d_resource_get_desc(&backbuffer->resource, &desc);
-            wined3d_surface_decref(backbuffer);
+            wined3d_resource_get_desc(&((IWineD3DSurfaceImpl *)backbuffer)->resource, &desc);
+            IWineD3DSurface_Release(backbuffer);
 
             /* Set the default scissor rect values */
             state->scissor_rect.left = 0;
@@ -1306,19 +1305,18 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
         /* Set the default viewport */
         state->viewport.X = 0;
         state->viewport.Y = 0;
-        state->viewport.Width = swapchain->presentParms.BackBufferWidth;
-        state->viewport.Height = swapchain->presentParms.BackBufferHeight;
+        state->viewport.Width = ((IWineD3DSwapChainImpl *)swapchain)->presentParms.BackBufferWidth;
+        state->viewport.Height = ((IWineD3DSwapChainImpl *)swapchain)->presentParms.BackBufferHeight;
         state->viewport.MinZ = 0.0f;
         state->viewport.MaxZ = 1.0f;
 
-        wined3d_swapchain_decref(swapchain);
+        IWineD3DSwapChain_Release(swapchain);
     }
 
     TRACE("Done.\n");
 }
 
-static HRESULT stateblock_init(struct wined3d_stateblock *stateblock,
-        struct wined3d_device *device, WINED3DSTATEBLOCKTYPE type)
+HRESULT stateblock_init(struct wined3d_stateblock *stateblock, IWineD3DDeviceImpl *device, WINED3DSTATEBLOCKTYPE type)
 {
     unsigned int i;
     HRESULT hr;
@@ -1369,33 +1367,3 @@ static HRESULT stateblock_init(struct wined3d_stateblock *stateblock,
 
     return WINED3D_OK;
 }
-
-HRESULT CDECL wined3d_stateblock_create(struct wined3d_device *device,
-        WINED3DSTATEBLOCKTYPE type, struct wined3d_stateblock **stateblock)
-{
-    struct wined3d_stateblock *object;
-    HRESULT hr;
-
-    TRACE("device %p, type %#x, stateblock %p.\n",
-            device, type, stateblock);
-
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
-    if (!object)
-    {
-        ERR("Failed to allocate stateblock memory.\n");
-        return E_OUTOFMEMORY;
-    }
-
-    hr = stateblock_init(object, device, type);
-    if (FAILED(hr))
-    {
-        WARN("Failed to initialize stateblock, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        return hr;
-    }
-
-    TRACE("Created stateblock %p.\n", object);
-    *stateblock = object;
-
-    return WINED3D_OK;
-}
