@@ -369,8 +369,8 @@ IWineGDISurfaceImpl_PrivateSetup(IWineD3DSurface *iface)
     }
 
     /* We don't mind the nonpow2 stuff in GDI */
-    This->pow2Width = This->currentDesc.Width;
-    This->pow2Height = This->currentDesc.Height;
+    This->pow2Width = This->resource.width;
+    This->pow2Height = This->resource.height;
 
     return WINED3D_OK;
 }
@@ -452,7 +452,6 @@ const IWineD3DSurfaceVtbl IWineGDISurface_Vtbl =
     IWineD3DBaseSurfaceImpl_GetType,
     /* IWineD3DSurface */
     IWineD3DBaseSurfaceImpl_GetResource,
-    IWineD3DBaseSurfaceImpl_GetDesc,
     IWineGDISurfaceImpl_Map,
     IWineGDISurfaceImpl_Unmap,
     IWineGDISurfaceImpl_GetDC,
