@@ -60,10 +60,6 @@
 #ifdef HAVE_SYS_POLL_H
 #include <sys/poll.h>
 #endif
-#ifdef HAVE_SYS_ERRNO_H
-#include <sys/errno.h>
-#endif
-#include <sys/soundcard.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -71,6 +67,7 @@
 #include "winuser.h"
 #include "winnls.h"
 #include "mmddk.h"
+#include "oss.h"
 #include "wine/unicode.h"
 #include "wine/debug.h"
 
@@ -155,7 +152,7 @@ static	int 	MIDI_UnixToWindowsDeviceType(int type)
      * MOD_SQSYNTH      square wave internal synth
      * MOD_FMSYNTH      FM internal synth
      * MOD_MAPPER       MIDI mapper
-     * MOD_WAVETABLE    hardware wavetable internal synth
+     * MOD_WAVETABLE    hardware watetable internal synth
      * MOD_SWSYNTH      software internal synth
      */
 
