@@ -47,29 +47,7 @@
 #define ioctlsocket ioctl
 #endif /* __MINGW32__ */
 
-extern HMODULE WININET_hModule DECLSPEC_HIDDEN;
-
-#ifndef INET6_ADDRSTRLEN
-#define INET6_ADDRSTRLEN 46
-#endif
-
-typedef struct {
-    WCHAR *name;
-    INTERNET_PORT port;
-    struct sockaddr_storage addr;
-    socklen_t addr_len;
-    char addr_str[INET6_ADDRSTRLEN];
-
-    LONG ref;
-    DWORD64 keep_until;
-
-    struct list entry;
-    struct list conn_pool;
-} server_t;
-
-void server_addref(server_t*) DECLSPEC_HIDDEN;
-void server_release(server_t*) DECLSPEC_HIDDEN;
-BOOL collect_connections(BOOL) DECLSPEC_HIDDEN;
+extern HMODULE WININET_hModule;
 
 /* used for netconnection.c stuff */
 typedef struct
@@ -77,13 +55,8 @@ typedef struct
     BOOL useSSL;
     int socketFD;
     void *ssl_s;
-    server_t *server;
     DWORD security_flags;
-
-    BOOL keep_alive;
-    DWORD64 keep_until;
-    struct list pool_entry;
-} netconn_t;
+} WININET_NETCONNECTION;
 
 static inline void * __WINE_ALLOC_SIZE(1) heap_alloc(size_t len)
 {
@@ -118,7 +91,7 @@ static inline LPWSTR heap_strdupW(LPCWSTR str)
         DWORD size;
 
         size = (strlenW(str)+1)*sizeof(WCHAR);
-        ret = heap_alloc(size);
+        ret = HeapAlloc(GetProcessHeap(), 0, size);
         if(ret)
             memcpy(ret, str, size);
     }
@@ -138,7 +111,7 @@ static inline LPWSTR heap_strndupW(LPCWSTR str, UINT max_len)
         if(str[len] == '\0')
             break;
 
-    ret = heap_alloc(sizeof(WCHAR)*(len+1));
+    ret = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR)*(len+1));
     if(ret) {
         memcpy(ret, str, sizeof(WCHAR)*len);
         ret[len] = '\0';
@@ -155,7 +128,7 @@ static inline WCHAR *heap_strdupAtoW(const char *str)
         DWORD len;
 
         len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
-        ret = heap_alloc(len*sizeof(WCHAR));
+        ret = HeapAlloc(GetProcessHeap(), 0, len*sizeof(WCHAR));
         if(ret)
             MultiByteToWideChar(CP_ACP, 0, str, -1, ret, len);
     }
@@ -169,7 +142,7 @@ static inline char *heap_strdupWtoA(LPCWSTR str)
 
     if(str) {
         DWORD size = WideCharToMultiByte(CP_ACP, 0, str, -1, NULL, 0, NULL, NULL);
-        ret = heap_alloc(size);
+        ret = HeapAlloc(GetProcessHeap(), 0, size);
         if(ret)
             WideCharToMultiByte(CP_ACP, 0, str, -1, ret, size, NULL, NULL);
     }
@@ -255,6 +228,7 @@ typedef struct
     DWORD   accessType;
 } appinfo_t;
 
+
 typedef struct
 {
     object_header_t hdr;
@@ -265,6 +239,8 @@ typedef struct
     LPWSTR  password;
     INTERNET_PORT hostPort; /* the final destination port of the request */
     INTERNET_PORT serverPort; /* the port of the server we directly connect to */
+    struct sockaddr_storage socketAddress;
+    socklen_t sa_len;
 } http_session_t;
 
 #define HDR_ISREQUEST		0x0001
@@ -282,19 +258,7 @@ typedef struct
 
 struct HttpAuthInfo;
 
-typedef struct data_stream_vtbl_t data_stream_vtbl_t;
-
-typedef struct {
-    const data_stream_vtbl_t *vtbl;
-}  data_stream_t;
-
-typedef struct {
-    data_stream_t data_stream;
-    DWORD content_length;
-    DWORD content_read;
-} netconn_stream_t;
-
-#define READ_BUFFER_SIZE 8192
+typedef struct gzip_stream_t gzip_stream_t;
 
 typedef struct
 {
@@ -303,8 +267,7 @@ typedef struct
     LPWSTR path;
     LPWSTR verb;
     LPWSTR rawHeaders;
-    netconn_t *netconn;
-    DWORD security_flags;
+    WININET_NETCONNECTION netConnection;
     LPWSTR version;
     LPWSTR statusText;
     DWORD bytesToWrite;
@@ -320,15 +283,14 @@ typedef struct
 
     CRITICAL_SECTION read_section;  /* section to protect the following fields */
     DWORD contentLength;  /* total number of bytes to be read */
+    DWORD contentRead;    /* bytes of the content read so far */
     BOOL  read_chunked;   /* are we reading in chunked mode? */
-    BOOL  read_gzip;      /* are we reading in gzip mode? */
     DWORD read_pos;       /* current read position in read_buf */
     DWORD read_size;      /* valid data size in read_buf */
-    BYTE  read_buf[READ_BUFFER_SIZE]; /* buffer for already read but not returned data */
+    BYTE  read_buf[4096]; /* buffer for already read but not returned data */
 
     BOOL decoding;
-    data_stream_t *data_stream;
-    netconn_stream_t netconn_stream;
+    gzip_stream_t *gzip_stream;
 } http_request_t;
 
 
@@ -475,63 +437,64 @@ typedef struct WORKREQ
 
 } WORKREQUEST, *LPWORKREQUEST;
 
-void *alloc_object(object_header_t*,const object_vtbl_t*,size_t) DECLSPEC_HIDDEN;
-object_header_t *get_handle_object( HINTERNET hinternet ) DECLSPEC_HIDDEN;
-object_header_t *WININET_AddRef( object_header_t *info ) DECLSPEC_HIDDEN;
-BOOL WININET_Release( object_header_t *info ) DECLSPEC_HIDDEN;
+void *alloc_object(object_header_t*,const object_vtbl_t*,size_t);
+object_header_t *get_handle_object( HINTERNET hinternet );
+object_header_t *WININET_AddRef( object_header_t *info );
+BOOL WININET_Release( object_header_t *info );
 
-DWORD INET_QueryOption( object_header_t *, DWORD, void *, DWORD *, BOOL ) DECLSPEC_HIDDEN;
+DWORD INET_QueryOption( object_header_t *, DWORD, void *, DWORD *, BOOL );
 
-time_t ConvertTimeString(LPCWSTR asctime) DECLSPEC_HIDDEN;
+time_t ConvertTimeString(LPCWSTR asctime);
 
 HINTERNET FTP_Connect(appinfo_t *hIC, LPCWSTR lpszServerName,
 	INTERNET_PORT nServerPort, LPCWSTR lpszUserName,
 	LPCWSTR lpszPassword, DWORD dwFlags, DWORD_PTR dwContext,
-	DWORD dwInternalFlags) DECLSPEC_HIDDEN;
+	DWORD dwInternalFlags);
 
 DWORD HTTP_Connect(appinfo_t*,LPCWSTR,
         INTERNET_PORT nServerPort, LPCWSTR lpszUserName,
         LPCWSTR lpszPassword, DWORD dwFlags, DWORD_PTR dwContext,
-        DWORD dwInternalFlags, HINTERNET*) DECLSPEC_HIDDEN;
+        DWORD dwInternalFlags, HINTERNET*);
 
 BOOL GetAddress(LPCWSTR lpszServerName, INTERNET_PORT nServerPort,
-	struct sockaddr *psa, socklen_t *sa_len) DECLSPEC_HIDDEN;
+	struct sockaddr *psa, socklen_t *sa_len);
 
-BOOL get_cookie(const WCHAR*,const WCHAR*,WCHAR*,DWORD*) DECLSPEC_HIDDEN;
-BOOL set_cookie(const WCHAR*,const WCHAR*,const WCHAR*,const WCHAR*) DECLSPEC_HIDDEN;
-
-void INTERNET_SetLastError(DWORD dwError) DECLSPEC_HIDDEN;
-DWORD INTERNET_GetLastError(void) DECLSPEC_HIDDEN;
-DWORD INTERNET_AsyncCall(LPWORKREQUEST lpWorkRequest) DECLSPEC_HIDDEN;
-LPSTR INTERNET_GetResponseBuffer(void) DECLSPEC_HIDDEN;
-LPSTR INTERNET_GetNextLine(INT nSocket, LPDWORD dwLen) DECLSPEC_HIDDEN;
+void INTERNET_SetLastError(DWORD dwError);
+DWORD INTERNET_GetLastError(void);
+DWORD INTERNET_AsyncCall(LPWORKREQUEST lpWorkRequest);
+LPSTR INTERNET_GetResponseBuffer(void);
+LPSTR INTERNET_GetNextLine(INT nSocket, LPDWORD dwLen);
 
 VOID SendAsyncCallback(object_header_t *hdr, DWORD_PTR dwContext,
                        DWORD dwInternetStatus, LPVOID lpvStatusInfo,
-                       DWORD dwStatusInfoLength) DECLSPEC_HIDDEN;
+                       DWORD dwStatusInfoLength);
 
 VOID INTERNET_SendCallback(object_header_t *hdr, DWORD_PTR dwContext,
                            DWORD dwInternetStatus, LPVOID lpvStatusInfo,
-                           DWORD dwStatusInfoLength) DECLSPEC_HIDDEN;
-BOOL INTERNET_FindProxyForProtocol(LPCWSTR szProxy, LPCWSTR proto, WCHAR *foundProxy, DWORD *foundProxyLen) DECLSPEC_HIDDEN;
+                           DWORD dwStatusInfoLength);
+BOOL INTERNET_FindProxyForProtocol(LPCWSTR szProxy, LPCWSTR proto, WCHAR *foundProxy, DWORD *foundProxyLen);
 
-DWORD create_netconn(BOOL,server_t*,DWORD,netconn_t**) DECLSPEC_HIDDEN;
-void free_netconn(netconn_t*) DECLSPEC_HIDDEN;
-void NETCON_unload(void) DECLSPEC_HIDDEN;
-DWORD NETCON_secure_connect(netconn_t *connection, LPWSTR hostname) DECLSPEC_HIDDEN;
-DWORD NETCON_send(netconn_t *connection, const void *msg, size_t len, int flags,
-		int *sent /* out */) DECLSPEC_HIDDEN;
-DWORD NETCON_recv(netconn_t *connection, void *buf, size_t len, int flags,
-		int *recvd /* out */) DECLSPEC_HIDDEN;
-BOOL NETCON_query_data_available(netconn_t *connection, DWORD *available) DECLSPEC_HIDDEN;
-BOOL NETCON_is_alive(netconn_t*) DECLSPEC_HIDDEN;
-LPCVOID NETCON_GetCert(netconn_t *connection) DECLSPEC_HIDDEN;
-int NETCON_GetCipherStrength(netconn_t*) DECLSPEC_HIDDEN;
-DWORD NETCON_set_timeout(netconn_t *connection, BOOL send, int value) DECLSPEC_HIDDEN;
-int sock_get_error(int) DECLSPEC_HIDDEN;
+BOOL NETCON_connected(WININET_NETCONNECTION *connection);
+DWORD NETCON_init(WININET_NETCONNECTION *connnection, BOOL useSSL);
+void NETCON_unload(void);
+DWORD NETCON_create(WININET_NETCONNECTION *connection, int domain,
+	      int type, int protocol);
+DWORD NETCON_close(WININET_NETCONNECTION *connection);
+DWORD NETCON_connect(WININET_NETCONNECTION *connection, const struct sockaddr *serv_addr,
+		    unsigned int addrlen);
+DWORD NETCON_secure_connect(WININET_NETCONNECTION *connection, LPWSTR hostname);
+DWORD NETCON_send(WININET_NETCONNECTION *connection, const void *msg, size_t len, int flags,
+		int *sent /* out */);
+DWORD NETCON_recv(WININET_NETCONNECTION *connection, void *buf, size_t len, int flags,
+		int *recvd /* out */);
+BOOL NETCON_query_data_available(WININET_NETCONNECTION *connection, DWORD *available);
+LPCVOID NETCON_GetCert(WININET_NETCONNECTION *connection);
+int NETCON_GetCipherStrength(WININET_NETCONNECTION *connection);
+DWORD NETCON_set_timeout(WININET_NETCONNECTION *connection, BOOL send, int value);
+int sock_get_error(int);
 
-extern void URLCacheContainers_CreateDefaults(void) DECLSPEC_HIDDEN;
-extern void URLCacheContainers_DeleteAll(void) DECLSPEC_HIDDEN;
+extern void URLCacheContainers_CreateDefaults(void);
+extern void URLCacheContainers_DeleteAll(void);
 
 #define MAX_REPLY_LEN	 	0x5B4
 
