@@ -58,9 +58,6 @@
 #ifdef HAVE_NETINET_IN_H
 # include <netinet/in.h>
 #endif
-#ifdef HAVE_NETINET_TCP_H
-# include <netinet/tcp.h>
-#endif
 #ifdef HAVE_OPENSSL_SSL_H
 # include <openssl/ssl.h>
 # include <openssl/opensslv.h>
@@ -73,7 +70,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-#include <assert.h>
 
 #include "wine/library.h"
 #include "windef.h"
@@ -105,6 +101,16 @@ WINE_DEFAULT_DEBUG_CHANNEL(wininet);
 
 #include <openssl/err.h>
 
+static CRITICAL_SECTION init_ssl_cs;
+static CRITICAL_SECTION_DEBUG init_ssl_cs_debug =
+{
+    0, 0, &init_ssl_cs,
+    { &init_ssl_cs_debug.ProcessLocksList,
+      &init_ssl_cs_debug.ProcessLocksList },
+    0, 0, { (DWORD_PTR)(__FILE__ ": init_ssl_cs") }
+};
+static CRITICAL_SECTION init_ssl_cs = { &init_ssl_cs_debug, -1, 0, 0, 0, 0 };
+
 static void *OpenSSL_ssl_handle;
 static void *OpenSSL_crypto_handle;
 
@@ -156,7 +162,6 @@ MAKE_FUNCPTR(ERR_free_strings);
 MAKE_FUNCPTR(ERR_get_error);
 MAKE_FUNCPTR(ERR_error_string);
 MAKE_FUNCPTR(X509_STORE_CTX_get_ex_data);
-MAKE_FUNCPTR(X509_STORE_CTX_get_chain);
 MAKE_FUNCPTR(i2d_X509);
 MAKE_FUNCPTR(sk_num);
 MAKE_FUNCPTR(sk_value);
@@ -195,7 +200,7 @@ static PCCERT_CONTEXT X509_to_cert_context(X509 *cert)
      */
     if (!p)
     {
-        buffer = heap_alloc(len);
+        buffer = HeapAlloc(GetProcessHeap(),0,len);
         p = buffer;
         len = pi2d_X509(cert,&p);
     }
@@ -210,7 +215,7 @@ static PCCERT_CONTEXT X509_to_cert_context(X509 *cert)
     if (malloced)
         free(buffer);
     else
-        heap_free(buffer);
+        HeapFree(GetProcessHeap(),0,buffer);
 
     return ret;
 }
@@ -312,7 +317,7 @@ static int netconn_secure_verify(int preverify_ok, X509_STORE_CTX *ctx)
     BOOL ret = FALSE;
     HCERTSTORE store = CertOpenStore(CERT_STORE_PROV_MEMORY, 0, 0,
         CERT_STORE_CREATE_NEW_FLAG, NULL);
-    netconn_t *conn;
+    WININET_NETCONNECTION *conn;
 
     ssl = pX509_STORE_CTX_get_ex_data(ctx,
         pSSL_get_ex_data_X509_STORE_CTX_idx());
@@ -323,18 +328,21 @@ static int netconn_secure_verify(int preverify_ok, X509_STORE_CTX *ctx)
         X509 *cert;
         int i;
         PCCERT_CONTEXT endCert = NULL;
-        struct stack_st *chain = (struct stack_st *)pX509_STORE_CTX_get_chain( ctx );
 
         ret = TRUE;
-        for (i = 0; ret && i < psk_num(chain); i++)
+        for (i = 0; ret && i < psk_num((struct stack_st *)ctx->chain); i++)
         {
             PCCERT_CONTEXT context;
 
-            cert = (X509 *)psk_value(chain, i);
+            cert = (X509 *)psk_value((struct stack_st *)ctx->chain, i);
             if ((context = X509_to_cert_context(cert)))
             {
-                ret = CertAddCertificateContextToStore(store, context,
-                        CERT_STORE_ADD_ALWAYS, i ? NULL : &endCert);
+                if (i == 0)
+                    ret = CertAddCertificateContextToStore(store, context,
+                        CERT_STORE_ADD_ALWAYS, &endCert);
+                else
+                    ret = CertAddCertificateContextToStore(store, context,
+                        CERT_STORE_ADD_ALWAYS, NULL);
                 CertFreeCertificateContext(context);
             }
         }
@@ -358,208 +366,159 @@ static int netconn_secure_verify(int preverify_ok, X509_STORE_CTX *ctx)
 
 #endif
 
-static CRITICAL_SECTION init_ssl_cs;
-static CRITICAL_SECTION_DEBUG init_ssl_cs_debug =
-{
-    0, 0, &init_ssl_cs,
-    { &init_ssl_cs_debug.ProcessLocksList,
-      &init_ssl_cs_debug.ProcessLocksList },
-    0, 0, { (DWORD_PTR)(__FILE__ ": init_ssl_cs") }
-};
-static CRITICAL_SECTION init_ssl_cs = { &init_ssl_cs_debug, -1, 0, 0, 0, 0 };
-
-static DWORD init_openssl(void)
+DWORD NETCON_init(WININET_NETCONNECTION *connection, BOOL useSSL)
 {
+    connection->useSSL = useSSL;
+    connection->socketFD = -1;
+    if (useSSL)
+    {
 #if defined(SONAME_LIBSSL) && defined(SONAME_LIBCRYPTO)
-    int i;
+        int i;
 
-    if(OpenSSL_ssl_handle)
-        return ERROR_SUCCESS;
+        TRACE("using SSL connection\n");
+        EnterCriticalSection(&init_ssl_cs);
+	if (OpenSSL_ssl_handle) /* already initialized everything */
+        {
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_SUCCESS;
+        }
+	OpenSSL_ssl_handle = wine_dlopen(SONAME_LIBSSL, RTLD_NOW, NULL, 0);
+	if (!OpenSSL_ssl_handle)
+	{
+	    ERR("trying to use a SSL connection, but couldn't load %s. Expect trouble.\n",
+		SONAME_LIBSSL);
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
+	}
+	OpenSSL_crypto_handle = wine_dlopen(SONAME_LIBCRYPTO, RTLD_NOW, NULL, 0);
+	if (!OpenSSL_crypto_handle)
+	{
+	    ERR("trying to use a SSL connection, but couldn't load %s. Expect trouble.\n",
+		SONAME_LIBCRYPTO);
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
+	}
 
-    OpenSSL_ssl_handle = wine_dlopen(SONAME_LIBSSL, RTLD_NOW, NULL, 0);
-    if(!OpenSSL_ssl_handle) {
-        ERR("trying to use a SSL connection, but couldn't load %s. Expect trouble.\n", SONAME_LIBSSL);
-        return ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
-    }
-
-    OpenSSL_crypto_handle = wine_dlopen(SONAME_LIBCRYPTO, RTLD_NOW, NULL, 0);
-    if(!OpenSSL_crypto_handle) {
-        ERR("trying to use a SSL connection, but couldn't load %s. Expect trouble.\n", SONAME_LIBCRYPTO);
-        return ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
-    }
-
-    /* mmm nice ugly macroness */
+        /* mmm nice ugly macroness */
 #define DYNSSL(x) \
     p##x = wine_dlsym(OpenSSL_ssl_handle, #x, NULL, 0); \
-    if (!p##x) { \
+    if (!p##x) \
+    { \
         ERR("failed to load symbol %s\n", #x); \
+        LeaveCriticalSection(&init_ssl_cs); \
         return ERROR_INTERNET_SECURITY_CHANNEL_ERROR; \
     }
 
-    DYNSSL(SSL_library_init);
-    DYNSSL(SSL_load_error_strings);
-    DYNSSL(SSLv23_method);
-    DYNSSL(SSL_CTX_free);
-    DYNSSL(SSL_CTX_new);
-    DYNSSL(SSL_new);
-    DYNSSL(SSL_free);
-    DYNSSL(SSL_set_fd);
-    DYNSSL(SSL_connect);
-    DYNSSL(SSL_shutdown);
-    DYNSSL(SSL_write);
-    DYNSSL(SSL_read);
-    DYNSSL(SSL_pending);
-    DYNSSL(SSL_get_error);
-    DYNSSL(SSL_get_ex_new_index);
-    DYNSSL(SSL_get_ex_data);
-    DYNSSL(SSL_set_ex_data);
-    DYNSSL(SSL_get_ex_data_X509_STORE_CTX_idx);
-    DYNSSL(SSL_get_peer_certificate);
-    DYNSSL(SSL_CTX_get_timeout);
-    DYNSSL(SSL_CTX_set_timeout);
-    DYNSSL(SSL_CTX_set_default_verify_paths);
-    DYNSSL(SSL_CTX_set_verify);
-    DYNSSL(SSL_get_current_cipher);
-    DYNSSL(SSL_CIPHER_get_bits);
+	DYNSSL(SSL_library_init);
+	DYNSSL(SSL_load_error_strings);
+	DYNSSL(SSLv23_method);
+	DYNSSL(SSL_CTX_free);
+	DYNSSL(SSL_CTX_new);
+	DYNSSL(SSL_new);
+	DYNSSL(SSL_free);
+	DYNSSL(SSL_set_fd);
+	DYNSSL(SSL_connect);
+	DYNSSL(SSL_shutdown);
+	DYNSSL(SSL_write);
+	DYNSSL(SSL_read);
+	DYNSSL(SSL_pending);
+        DYNSSL(SSL_get_error);
+	DYNSSL(SSL_get_ex_new_index);
+	DYNSSL(SSL_get_ex_data);
+	DYNSSL(SSL_set_ex_data);
+	DYNSSL(SSL_get_ex_data_X509_STORE_CTX_idx);
+	DYNSSL(SSL_get_peer_certificate);
+	DYNSSL(SSL_CTX_get_timeout);
+	DYNSSL(SSL_CTX_set_timeout);
+	DYNSSL(SSL_CTX_set_default_verify_paths);
+	DYNSSL(SSL_CTX_set_verify);
+        DYNSSL(SSL_get_current_cipher);
+        DYNSSL(SSL_CIPHER_get_bits);
 #undef DYNSSL
 
 #define DYNCRYPTO(x) \
     p##x = wine_dlsym(OpenSSL_crypto_handle, #x, NULL, 0); \
-    if (!p##x) { \
+    if (!p##x) \
+    { \
         ERR("failed to load symbol %s\n", #x); \
+        LeaveCriticalSection(&init_ssl_cs); \
         return ERROR_INTERNET_SECURITY_CHANNEL_ERROR; \
     }
-
-    DYNCRYPTO(BIO_new_fp);
-    DYNCRYPTO(CRYPTO_num_locks);
-    DYNCRYPTO(CRYPTO_set_id_callback);
-    DYNCRYPTO(CRYPTO_set_locking_callback);
-    DYNCRYPTO(ERR_free_strings);
-    DYNCRYPTO(ERR_get_error);
-    DYNCRYPTO(ERR_error_string);
-    DYNCRYPTO(X509_STORE_CTX_get_ex_data);
-    DYNCRYPTO(X509_STORE_CTX_get_chain);
-    DYNCRYPTO(i2d_X509);
-    DYNCRYPTO(sk_num);
-    DYNCRYPTO(sk_value);
+	DYNCRYPTO(BIO_new_fp);
+	DYNCRYPTO(CRYPTO_num_locks);
+	DYNCRYPTO(CRYPTO_set_id_callback);
+	DYNCRYPTO(CRYPTO_set_locking_callback);
+	DYNCRYPTO(ERR_free_strings);
+	DYNCRYPTO(ERR_get_error);
+	DYNCRYPTO(ERR_error_string);
+	DYNCRYPTO(X509_STORE_CTX_get_ex_data);
+	DYNCRYPTO(i2d_X509);
+	DYNCRYPTO(sk_num);
+	DYNCRYPTO(sk_value);
 #undef DYNCRYPTO
 
-    pSSL_library_init();
-    pSSL_load_error_strings();
-    pBIO_new_fp(stderr, BIO_NOCLOSE); /* FIXME: should use winedebug stuff */
+	pSSL_library_init();
+	pSSL_load_error_strings();
+	pBIO_new_fp(stderr, BIO_NOCLOSE); /* FIXME: should use winedebug stuff */
 
-    meth = pSSLv23_method();
-    ctx = pSSL_CTX_new(meth);
-    if(!pSSL_CTX_set_default_verify_paths(ctx)) {
-        ERR("SSL_CTX_set_default_verify_paths failed: %s\n",
-            pERR_error_string(pERR_get_error(), 0));
-        return ERROR_OUTOFMEMORY;
-    }
+	meth = pSSLv23_method();
+        ctx = pSSL_CTX_new(meth);
+        if (!pSSL_CTX_set_default_verify_paths(ctx))
+        {
+            ERR("SSL_CTX_set_default_verify_paths failed: %s\n",
+                pERR_error_string(pERR_get_error(), 0));
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        hostname_idx = pSSL_get_ex_new_index(0, (void *)"hostname index",
+                NULL, NULL, NULL);
+        if (hostname_idx == -1)
+        {
+            ERR("SSL_get_ex_new_index failed; %s\n",
+                pERR_error_string(pERR_get_error(), 0));
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        error_idx = pSSL_get_ex_new_index(0, (void *)"error index",
+                NULL, NULL, NULL);
+        if (error_idx == -1)
+        {
+            ERR("SSL_get_ex_new_index failed; %s\n",
+                pERR_error_string(pERR_get_error(), 0));
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        conn_idx = pSSL_get_ex_new_index(0, (void *)"netconn index",
+                NULL, NULL, NULL);
+        if (conn_idx == -1)
+        {
+            ERR("SSL_get_ex_new_index failed; %s\n",
+                pERR_error_string(pERR_get_error(), 0));
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        pSSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, netconn_secure_verify);
 
-    hostname_idx = pSSL_get_ex_new_index(0, (void *)"hostname index", NULL, NULL, NULL);
-    if(hostname_idx == -1) {
-        ERR("SSL_get_ex_new_index failed; %s\n", pERR_error_string(pERR_get_error(), 0));
-        return ERROR_OUTOFMEMORY;
-    }
-
-    error_idx = pSSL_get_ex_new_index(0, (void *)"error index", NULL, NULL, NULL);
-    if(error_idx == -1) {
-        ERR("SSL_get_ex_new_index failed; %s\n", pERR_error_string(pERR_get_error(), 0));
-        return ERROR_OUTOFMEMORY;
-    }
-
-    conn_idx = pSSL_get_ex_new_index(0, (void *)"netconn index", NULL, NULL, NULL);
-    if(conn_idx == -1) {
-        ERR("SSL_get_ex_new_index failed; %s\n", pERR_error_string(pERR_get_error(), 0));
-        return ERROR_OUTOFMEMORY;
-    }
-
-    pSSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, netconn_secure_verify);
-
-    pCRYPTO_set_id_callback(ssl_thread_id);
-    num_ssl_locks = pCRYPTO_num_locks();
-    ssl_locks = heap_alloc(num_ssl_locks * sizeof(CRITICAL_SECTION));
-    if(!ssl_locks)
-        return ERROR_OUTOFMEMORY;
-
-    for(i = 0; i < num_ssl_locks; i++)
-        InitializeCriticalSection(&ssl_locks[i]);
-    pCRYPTO_set_locking_callback(ssl_lock_callback);
-
-    return ERROR_SUCCESS;
-#else
-    FIXME("can't use SSL, not compiled in.\n");
-    return ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
-#endif
-}
-
-DWORD create_netconn(BOOL useSSL, server_t *server, DWORD security_flags, netconn_t **ret)
-{
-    netconn_t *netconn;
-    int result, flag;
-
-    if(useSSL) {
-        DWORD res;
-
-        TRACE("using SSL connection\n");
-
-        EnterCriticalSection(&init_ssl_cs);
-        res = init_openssl();
+        pCRYPTO_set_id_callback(ssl_thread_id);
+        num_ssl_locks = pCRYPTO_num_locks();
+        ssl_locks = HeapAlloc(GetProcessHeap(), 0, num_ssl_locks * sizeof(CRITICAL_SECTION));
+        if (!ssl_locks)
+        {
+            LeaveCriticalSection(&init_ssl_cs);
+            return ERROR_OUTOFMEMORY;
+        }
+        for (i = 0; i < num_ssl_locks; i++)
+            InitializeCriticalSection(&ssl_locks[i]);
+        pCRYPTO_set_locking_callback(ssl_lock_callback);
         LeaveCriticalSection(&init_ssl_cs);
-        if(res != ERROR_SUCCESS)
-            return res;
-    }
-
-    netconn = heap_alloc_zero(sizeof(*netconn));
-    if(!netconn)
-        return ERROR_OUTOFMEMORY;
-
-    netconn->useSSL = useSSL;
-    netconn->socketFD = -1;
-    netconn->security_flags = security_flags;
-    list_init(&netconn->pool_entry);
-
-    assert(server->addr_len);
-    result = netconn->socketFD = socket(server->addr.ss_family, SOCK_STREAM, 0);
-    if(result != -1) {
-        result = connect(netconn->socketFD, (struct sockaddr*)&server->addr, server->addr_len);
-        if(result == -1)
-            closesocket(netconn->socketFD);
-    }
-    if(result == -1) {
-        heap_free(netconn);
-        return sock_get_error(errno);
-    }
-
-#ifdef TCP_NODELAY
-    flag = 1;
-    result = setsockopt(netconn->socketFD, IPPROTO_TCP, TCP_NODELAY, (void*)&flag, sizeof(flag));
-    if(result < 0)
-        WARN("setsockopt(TCP_NODELAY) failed\n");
+#else
+	FIXME("can't use SSL, not compiled in.\n");
+        return ERROR_INTERNET_SECURITY_CHANNEL_ERROR;
 #endif
-
-    server_addref(server);
-    netconn->server = server;
-
-    *ret = netconn;
+    }
     return ERROR_SUCCESS;
 }
 
-void free_netconn(netconn_t *netconn)
-{
-    server_release(netconn->server);
-
-#ifdef SONAME_LIBSSL
-    if (netconn->ssl_s) {
-        pSSL_shutdown(netconn->ssl_s);
-        pSSL_free(netconn->ssl_s);
-    }
-#endif
-
-    closesocket(netconn->socketFD);
-    heap_free(netconn);
-}
-
 void NETCON_unload(void)
 {
 #if defined(SONAME_LIBSSL) && defined(SONAME_LIBCRYPTO)
@@ -578,11 +537,16 @@ void NETCON_unload(void)
     {
         int i;
         for (i = 0; i < num_ssl_locks; i++) DeleteCriticalSection(&ssl_locks[i]);
-        heap_free(ssl_locks);
+        HeapFree(GetProcessHeap(), 0, ssl_locks);
     }
 #endif
 }
 
+BOOL NETCON_connected(WININET_NETCONNECTION *connection)
+{
+    return connection->socketFD != -1;
+}
+
 /* translate a unix error code into a winsock one */
 int sock_get_error( int err )
 {
@@ -650,16 +614,62 @@ int sock_get_error( int err )
     return err;
 }
 
+/******************************************************************************
+ * NETCON_create
+ * Basically calls 'socket()'
+ */
+DWORD NETCON_create(WININET_NETCONNECTION *connection, int domain,
+	      int type, int protocol)
+{
+#ifdef SONAME_LIBSSL
+    if (connection->ssl_s)
+        return ERROR_NOT_SUPPORTED;
+#endif
+
+    connection->socketFD = socket(domain, type, protocol);
+    if (connection->socketFD == -1)
+        return sock_get_error(errno);
+
+    return ERROR_SUCCESS;
+}
+
+/******************************************************************************
+ * NETCON_close
+ * Basically calls 'close()' unless we should use SSL
+ */
+DWORD NETCON_close(WININET_NETCONNECTION *connection)
+{
+    int result;
+
+    if (!NETCON_connected(connection)) return ERROR_SUCCESS;
+
+#ifdef SONAME_LIBSSL
+    if (connection->ssl_s)
+    {
+        pSSL_shutdown(connection->ssl_s);
+        pSSL_free(connection->ssl_s);
+        connection->ssl_s = NULL;
+    }
+#endif
+
+    result = closesocket(connection->socketFD);
+    connection->socketFD = -1;
+
+    if (result == -1)
+        return sock_get_error(errno);
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * NETCON_secure_connect
  * Initiates a secure connection over an existing plaintext connection.
  */
-DWORD NETCON_secure_connect(netconn_t *connection, LPWSTR hostname)
+DWORD NETCON_secure_connect(WININET_NETCONNECTION *connection, LPWSTR hostname)
 {
-    DWORD res = ERROR_NOT_SUPPORTED;
-#ifdef SONAME_LIBSSL
     void *ssl_s;
+    DWORD res = ERROR_NOT_SUPPORTED;
 
+#ifdef SONAME_LIBSSL
     /* can't connect if we are already connected */
     if (connection->ssl_s)
     {
@@ -719,14 +729,37 @@ fail:
     return res;
 }
 
+/******************************************************************************
+ * NETCON_connect
+ * Connects to the specified address.
+ */
+DWORD NETCON_connect(WININET_NETCONNECTION *connection, const struct sockaddr *serv_addr,
+		    unsigned int addrlen)
+{
+    int result;
+
+    result = connect(connection->socketFD, serv_addr, addrlen);
+    if (result == -1)
+    {
+        WARN("Unable to connect to host (%s)\n", strerror(errno));
+
+        closesocket(connection->socketFD);
+        connection->socketFD = -1;
+        return sock_get_error(errno);
+    }
+
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * NETCON_send
  * Basically calls 'send()' unless we should use SSL
  * number of chars send is put in *sent
  */
-DWORD NETCON_send(netconn_t *connection, const void *msg, size_t len, int flags,
+DWORD NETCON_send(WININET_NETCONNECTION *connection, const void *msg, size_t len, int flags,
 		int *sent /* out */)
 {
+    if (!NETCON_connected(connection)) return ERROR_INTERNET_CONNECTION_ABORTED;
     if (!connection->useSSL)
     {
 	*sent = send(connection->socketFD, msg, len, flags);
@@ -758,15 +791,18 @@ DWORD NETCON_send(netconn_t *connection, const void *msg, size_t len, int flags,
  * Basically calls 'recv()' unless we should use SSL
  * number of chars received is put in *recvd
  */
-DWORD NETCON_recv(netconn_t *connection, void *buf, size_t len, int flags,
+DWORD NETCON_recv(WININET_NETCONNECTION *connection, void *buf, size_t len, int flags,
 		int *recvd /* out */)
 {
     *recvd = 0;
+    if (!NETCON_connected(connection)) return ERROR_INTERNET_CONNECTION_ABORTED;
     if (!len)
         return ERROR_SUCCESS;
     if (!connection->useSSL)
     {
 	*recvd = recv(connection->socketFD, buf, len, flags);
+        if(!*recvd)
+            NETCON_close(connection);
 	return *recvd == -1 ? sock_get_error(errno) :  ERROR_SUCCESS;
     }
     else
@@ -780,8 +816,10 @@ DWORD NETCON_recv(netconn_t *connection, void *buf, size_t len, int flags,
 
         /* Check if EOF was received */
         if(!*recvd && (pSSL_get_error(connection->ssl_s, *recvd)==SSL_ERROR_ZERO_RETURN
-                    || pSSL_get_error(connection->ssl_s, *recvd)==SSL_ERROR_SYSCALL))
+                    || pSSL_get_error(connection->ssl_s, *recvd)==SSL_ERROR_SYSCALL)) {
+            NETCON_close(connection);
             return ERROR_SUCCESS;
+        }
 
         return *recvd > 0 ? ERROR_SUCCESS : ERROR_INTERNET_CONNECTION_ABORTED;
 #else
@@ -795,9 +833,11 @@ DWORD NETCON_recv(netconn_t *connection, void *buf, size_t len, int flags,
  * Returns the number of bytes of peeked data plus the number of bytes of
  * queued, but unread data.
  */
-BOOL NETCON_query_data_available(netconn_t *connection, DWORD *available)
+BOOL NETCON_query_data_available(WININET_NETCONNECTION *connection, DWORD *available)
 {
     *available = 0;
+    if (!NETCON_connected(connection))
+        return FALSE;
 
     if (!connection->useSSL)
     {
@@ -820,37 +860,7 @@ BOOL NETCON_query_data_available(netconn_t *connection, DWORD *available)
     return TRUE;
 }
 
-BOOL NETCON_is_alive(netconn_t *netconn)
-{
-#ifdef MSG_DONTWAIT
-    ssize_t len;
-    BYTE b;
-
-    len = recv(netconn->socketFD, &b, 1, MSG_PEEK|MSG_DONTWAIT);
-    return len == 1 || (len == -1 && errno == EWOULDBLOCK);
-#elif defined(__MINGW32__) || defined(_MSC_VER)
-    ULONG mode;
-    int len;
-    char b;
-
-    mode = 1;
-    if(!ioctlsocket(netconn->socketFD, FIONBIO, &mode))
-        return FALSE;
-
-    len = recv(netconn->socketFD, &b, 1, MSG_PEEK);
-
-    mode = 0;
-    if(!ioctlsocket(netconn->socketFD, FIONBIO, &mode))
-        return FALSE;
-
-    return len == 1 || (len == -1 && errno == WSAEWOULDBLOCK);
-#else
-    FIXME("not supported on this platform\n");
-    return TRUE;
-#endif
-}
-
-LPCVOID NETCON_GetCert(netconn_t *connection)
+LPCVOID NETCON_GetCert(WININET_NETCONNECTION *connection)
 {
 #ifdef SONAME_LIBSSL
     X509* cert;
@@ -867,7 +877,7 @@ LPCVOID NETCON_GetCert(netconn_t *connection)
 #endif
 }
 
-int NETCON_GetCipherStrength(netconn_t *connection)
+int NETCON_GetCipherStrength(WININET_NETCONNECTION *connection)
 {
 #ifdef SONAME_LIBSSL
 #if defined(OPENSSL_VERSION_NUMBER) && (OPENSSL_VERSION_NUMBER >= 0x0090707f)
@@ -889,11 +899,16 @@ int NETCON_GetCipherStrength(netconn_t *connection)
 #endif
 }
 
-DWORD NETCON_set_timeout(netconn_t *connection, BOOL send, int value)
+DWORD NETCON_set_timeout(WININET_NETCONNECTION *connection, BOOL send, int value)
 {
     int result;
     struct timeval tv;
 
+    /* FIXME: we should probably store the timeout in the connection to set
+     * when we do connect */
+    if (!NETCON_connected(connection))
+        return ERROR_SUCCESS;
+
     /* value is in milliseconds, convert to struct timeval */
     tv.tv_sec = value / 1000;
     tv.tv_usec = (value % 1000) * 1000;
