@@ -159,6 +159,7 @@ static const struct
     { "amd64",   CPU_x86_64 },
     { "x86_64",  CPU_x86_64 },
     { "sparc",   CPU_SPARC },
+    { "alpha",   CPU_ALPHA },
     { "powerpc", CPU_POWERPC },
     { "arm",     CPU_ARM }
 };
@@ -218,6 +219,8 @@ static const enum target_cpu build_cpu = CPU_x86;
 static const enum target_cpu build_cpu = CPU_x86_64;
 #elif defined(__sparc__)
 static const enum target_cpu build_cpu = CPU_SPARC;
+#elif defined(__ALPHA__)
+static const enum target_cpu build_cpu = CPU_ALPHA;
 #elif defined(__powerpc__)
 static const enum target_cpu build_cpu = CPU_POWERPC;
 #elif defined(__arm__)
@@ -1252,7 +1255,7 @@ int main(int argc, char **argv)
     {
         if (argv[i][0] == '-')  /* option */
 	{
-	    /* determine if this switch is followed by a separate argument */
+	    /* determine if tihs switch is followed by a separate argument */
 	    next_is_arg = 0;
 	    option_arg = 0;
 	    switch(argv[i][1])
@@ -1294,11 +1297,7 @@ int main(int argc, char **argv)
 			next_is_arg = 1;
 		    break;
 	    }
-	    if (next_is_arg)
-            {
-                if (i + 1 >= argc) error("option -%c requires an argument\n", argv[i][1]);
-                option_arg = argv[i+1];
-            }
+	    if (next_is_arg) option_arg = argv[i+1];
 
 	    /* determine what options go 'as is' to the linker & the compiler */
 	    raw_compiler_arg = raw_linker_arg = 0;
@@ -1326,7 +1325,6 @@ int main(int argc, char **argv)
 	    {
 		case 'B':
 		    str = strdup(option_arg);
-		    if (strendswith(str, "/")) str[strlen(str) - 1] = 0;
 		    if (strendswith(str, "/tools/winebuild"))
                     {
                         char *objdir = strdup(str);
@@ -1335,6 +1333,7 @@ int main(int argc, char **argv)
                         /* don't pass it to the compiler, this generates warnings */
                         raw_compiler_arg = raw_linker_arg = 0;
                     }
+		    if (strendswith(str, "/")) str[strlen(str) - 1] = 0;
                     if (!opts.prefix) opts.prefix = strarray_alloc();
                     strarray_add(opts.prefix, str);
 		    break;
@@ -1391,8 +1390,6 @@ int main(int argc, char **argv)
                         opts.force_pointer_size = 8;
 			raw_linker_arg = 1;
                     }
-                    else if (strncmp("-mcpu=", argv[i], 6) == 0)
-                        strarray_add(opts.winebuild_args, argv[i]);
 		    break;
                 case 'n':
                     if (strcmp("-nostdinc", argv[i]) == 0)
